Package: nanotime
Type: Package
Title: Nanosecond-Resolution Time Support for R
Version: 0.3.15
Date: 2026-05-21
Authors@R: c(person("Dirk", "Eddelbuettel", role = c("aut", "cre"), email = "edd@debian.org",
                    comment = c(ORCID = "0000-0001-6419-907X")),
             person("Leonardo", "Silvestri", role = "aut"))
Description: Full 64-bit resolution date and time functionality with
 nanosecond granularity is provided, with easy transition to and from
 the standard 'POSIXct' type. Three additional classes offer interval,
 period and duration functionality for nanosecond-resolution timestamps.
Depends: methods
Imports: bit64 (>= 4.8.0), RcppCCTZ (>= 0.2.9), zoo, Rcpp (>= 1.1.1)
Suggests: tinytest, data.table, xts, ggplot2
LinkingTo: Rcpp, RcppCCTZ, RcppDate
License: GPL (>= 2)
URL: https://github.com/eddelbuettel/nanotime,
        https://eddelbuettel.github.io/nanotime/,
        https://dirk.eddelbuettel.com/code/nanotime.html
BugReports: https://github.com/eddelbuettel/nanotime/issues
RoxygenNote: 7.3.2
Collate: 'nanotime.R' 'nanoival.R' 'nanoduration.R' 'nanoperiod.R'
        'RcppExports.R'
Encoding: UTF-8
NeedsCompilation: yes
VignetteBuilder: Rcpp
Packaged: 2026-05-21 11:52:30 UTC; edd
Author: Dirk Eddelbuettel [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6419-907X>),
  Leonardo Silvestri [aut]
Maintainer: Dirk Eddelbuettel <edd@debian.org>
Repository: CRAN
Date/Publication: 2026-05-21 12:30:12 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-25 01:19:15 UTC; unix
Archs: nanotime.so.dSYM

Package: nanotime
Type: Package
Title: Nanosecond-Resolution Time Support for R
Version: 0.3.15
Date: 2026-05-21
Authors@R: c(person("Dirk", "Eddelbuettel", role = c("aut", "cre"), email = "edd@debian.org",
                    comment = c(ORCID = "0000-0001-6419-907X")),
             person("Leonardo", "Silvestri", role = "aut"))
Description: Full 64-bit resolution date and time functionality with
 nanosecond granularity is provided, with easy transition to and from
 the standard 'POSIXct' type. Three additional classes offer interval,
 period and duration functionality for nanosecond-resolution timestamps.
Depends: methods
Imports: bit64 (>= 4.8.0), RcppCCTZ (>= 0.2.9), zoo, Rcpp (>= 1.1.1)
Suggests: tinytest, data.table, xts, ggplot2
LinkingTo: Rcpp, RcppCCTZ, RcppDate
License: GPL (>= 2)
URL: https://github.com/eddelbuettel/nanotime,
        https://eddelbuettel.github.io/nanotime/,
        https://dirk.eddelbuettel.com/code/nanotime.html
BugReports: https://github.com/eddelbuettel/nanotime/issues
RoxygenNote: 7.3.2
Collate: 'nanotime.R' 'nanoival.R' 'nanoduration.R' 'nanoperiod.R'
        'RcppExports.R'
Encoding: UTF-8
NeedsCompilation: yes
VignetteBuilder: Rcpp
Packaged: 2026-05-21 11:52:30 UTC; edd
Author: Dirk Eddelbuettel [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6419-907X>),
  Leonardo Silvestri [aut]
Maintainer: Dirk Eddelbuettel <edd@debian.org>
Repository: CRAN
Date/Publication: 2026-05-21 12:30:12 UTC

useDynLib(nanotime, .registration = TRUE)
import(methods)
import(bit64)
import(Rcpp)
importFrom(RcppCCTZ, parseDouble, formatDouble)
importFrom(zoo, index2char)
importFrom(utils, head, tail)

S3method(as.Date,nanotime)
S3method(as.POSIXct,nanotime)
S3method(as.POSIXlt,nanotime)
S3method(as.data.frame,nanotime)
S3method(as.integer64,nanotime)
S3method(format,nanotime)
S3method(as.character,nanotime)
S3method(index2char,nanotime)
S3method(c,nanotime)

export(NA_nanotime_)
export(NA_nanoduration_)
export(NA_nanoival_)
export(NA_nanoperiod_)
export(as.nanotime)
export(nanotime.matrix)
exportClasses(nanotime)
exportMethods("+")
exportMethods("-")
exportMethods("[[")
exportMethods("[")
exportMethods("[<-")
exportMethods(Arith)
exportMethods(Compare)
exportMethods(Complex)
exportMethods(Logic)
exportMethods(Math)
exportMethods(Math2)
exportMethods(Summary)
exportMethods(max)
exportMethods(min)
exportMethods(nanotime)
exportMethods(as.nanotime)
exportMethods(print)
exportMethods(range)
exportMethods(show)
exportMethods("names<-")
exportMethods(is.na)
exportMethods(as.nanoduration)
exportMethods(show)
exportMethods(print)
exportMethods(plus)
exportMethods(minus)


exportMethods(intersect.idx)
exportMethods(intersect)
exportMethods(union)
exportMethods(setdiff.idx)
exportMethods(setdiff)
exportMethods(as.nanoival)
exportClasses(nanoival)
export(nanoival)
exportMethods(nanoival.start)
exportMethods(nanoival.end)
exportMethods(nanoival.sopen)
exportMethods(nanoival.eopen)
exportMethods(print)
exportMethods(show)
exportMethods(names)
exportMethods("names<-")
exportMethods("<")
exportMethods("<=")
exportMethods(">")
exportMethods(">=")
exportMethods("[")
exportMethods("[<-")
exportMethods("is.na")
exportMethods("is.na<-")
S3method(c, nanoival)
exportMethods(t)
exportMethods(is.unsorted)
exportMethods(sort)
S3method(seq, nanotime)
exportMethods(seq)
exportMethods(all.equal)
S3method(all.equal, nanotime)

exportClasses(nanoperiod)
export(nanoperiod)
exportMethods(as.nanoperiod)
exportMethods(nanoperiod.month)
exportMethods(nanoperiod.day)
exportMethods(nanoperiod.nanoduration)
S3method(c, nanoperiod)

exportClasses(nanoduration)
export(nanoduration)
S3method(as.integer64,nanoduration)
exportMethods(as.nanoduration)
S3method(c, nanoduration)
S3method(as.data.frame,nanoduration)

S3method(seq, nanoduration)

S3method(format,nanoival)
S3method(format,nanoduration)
S3method(format,nanoperiod)
S3method(as.character,nanoival)

exportMethods(nano_wday)
exportMethods(nano_mday)
exportMethods(nano_month)
exportMethods(nano_year)
exportMethods(nano_ceiling)
exportMethods(nano_floor)

S3method("%in%", nanotime)
exportMethods("%in%")

if (getRversion() > "4.5.0") exportMethods(unique)

# nanotime: Nanosecond-Resolution Time for R

[![CI](https://github.com/eddelbuettel/nanotime/workflows/ci/badge.svg)](https://github.com/eddelbuettel/nanotime/actions?query=workflow%3Aci)
[![License](https://eddelbuettel.github.io/badges/GPL2+.svg)](https://www.gnu.org/licenses/gpl-2.0.html)
[![CRAN](https://www.r-pkg.org/badges/version/nanotime)](https://cran.r-project.org/package=nanotime)
[![r-universe](https://eddelbuettel.r-universe.dev/badges/nanotime)](https://eddelbuettel.r-universe.dev/nanotime)
[![Dependencies](https://tinyverse.netlify.app/badge/nanotime)](https://cran.r-project.org/package=nanotime)
[![Downloads](https://cranlogs.r-pkg.org/badges/nanotime?color=brightgreen)](https://www.r-pkg.org/pkg/nanotime)
[![Code Coverage](https://codecov.io/gh/eddelbuettel/nanotime/graph/badge.svg)](https://app.codecov.io/gh/eddelbuettel/nanotime)
[![Last Commit](https://img.shields.io/github/last-commit/eddelbuettel/nanotime)](https://github.com/eddelbuettel/nanotime)
[![Documentation](https://img.shields.io/badge/documentation-is_here-blue)](https://eddelbuettel.github.io/nanotime/)

### Motivation

R has excellent tools for dates and times. The `Date` and `POSIXct` classes (as well as the 'wide'
representation in `POSIXlt`) are versatile, and a lot of useful tooling has been built around them.

However, `POSIXct` is implemented as a `double` with fractional seconds since the epoch. Given the
53 bits accuracy, it leaves just a bit less than _microsecond_ resolution. Furthermore, using
floating-point arithmetic for an integer concept opens the door to painful issues of error
accumulation.

More and more performance measurements, latency statistics, etc., are now measured more finely, and we
need _nanosecond_ resolution for which commonly an `integer64` is used to represent nanoseconds
since the epoch.

And while R does not have a _native_ type for this, the [bit64](https://cran.r-project.org/package=bit64)
package by [Jens Oehlschlägel](https://github.com/joehl) offers a performant one implemented as a
lightweight S3 class. So this package uses the `integer64` class, along with multiple helper functions
for parsing and formatting at nano-second resolution from the [RcppCCTZ](https://dirk.eddelbuettel.com/code/rcpp.cctz.html)
package which wraps the [CCTZ library](https://github.com/google/cctz) from Google. CCTZ is a modern C++11 library
extending the (C++11-native) `chrono` type.

In addition to the point-in-time type `nanotime`, this package also provides an interval type
`nanoival` which may have open or closed start/end, a period type `nanoperiod` that is a human
representation of time, such as day, month, etc., and a duration type `nanoduration`. These types
are similar to what the [lubridate](https://github.com/tidyverse/lubridate) package proposes.

Set and arithmetic operations on these types are available. All functionality is designed to
correctly handle instances across different time zones. Because these temporal types are based on R
built-in types, most functions have an efficient implementation and the types are suitable for use
in `data.frame` and `data.table`. `nanotime` is also a better choice than the native `POSIXct` in
most of the cases where fractional seconds are needed because it avoids floating point issues.

### Documentation

Package documentation, help pages, a vignette, and more is available
[here](https://eddelbuettel.github.io/nanotime/).


### Demo

See the included demo script [nanosecondDelayExample.R](https://github.com/eddelbuettel/nanotime/blob/master/demo/nanosecondDelayExample.R)
for a (completely simulated and hence made-up) study of network latency measured
in nanoseconds resulting in the figure below

![](https://eddelbuettel.github.io/nanotime/assets/nanotimeDelayDemo.png)

### Examples

#### Simple Parsing and Arithmetic

```r
R> x <- as.nanotime("1970-01-01T00:00:00.000000001+00:00")
R> x
[1] "1970-01-01T00:00:00.000000001+00:00"
R> x + 1e9
[1] "1970-01-01T00:00:01.000000001+00:00"
R> as.nanotime("2020-03-21 Europe/London")
[1] 2020-03-21T00:00:00+00:00
```

#### Vectorised

```r
R> options("width"=60)
R> v <- nanotime(Sys.time()) + 1:5
R> v
[1] 2020-03-22T03:09:20.732122001+00:00
[2] 2020-03-22T03:09:20.732122002+00:00
[3] 2020-03-22T03:09:20.732122003+00:00
[4] 2020-03-22T03:09:20.732122004+00:00
[5] 2020-03-22T03:09:20.732122005+00:00
R>
```

#### Use with `zoo`

```r
R> library(zoo)
R> z <- zoo(cbind(A=1:5, B=5:1), v)
R> options(nanotimeFormat="%H:%M:%E*S")  ## override default format
R> z
R> options(nanotimeFormat=NULL)  ## go back to default format
R> z
```

#### Use with data.table

```r
R> library(data.table)
R> dt <- data.table(v, cbind(A=1:5, B=5:1))
R> fwrite(dt, file="datatableTest.csv")  # write out
R> dtcheck <- fread("datatableTest.csv") # read back
R> dtcheck
R> dtcheck[, v:=nanotime(v)]             # read as a string, need to re-class as nanotime
R> fread("../datatableTest.csv", colClasses=c("nanotime", "integer", "integer"))
```

#### Use with data.frame

This requires version 0.0.2 or later.

```r
R> data.frame(cbind(A=1:5, B=5:1), v=v)
```

#### Intervals

```r
R> ival <- as.nanoival("+2009-01-01 13:12:00 America/New_York -> 2009-02-01 15:11:03 America/New_York-")
R> ival
[1] +2009-01-01T18:12:00+00:00 -> 2009-02-01T20:11:03+00:00-

R> start <- nanotime("2009-01-01 13:12:00 America/New_York")
R> end   <- nanotime("2009-02-01 15:11:00 America/New_York")
R> nanoival(start, end)                   # by default sopen=F,eopen=T
[1] +2009-01-01T18:12:00+00:00 -> 2009-02-01T20:11:00+00:00-
R> nanoival(start, end, sopen=FALSE, eopen=TRUE)
[1] +2009-01-01T18:12:00+00:00 -> 2009-02-01T20:11:00+00:00-

R> intersect(as.nanoival("+2019-03-01 UTC -> 2020-03-01 UTC-"),
             as.nanoival("+2020-01-01 UTC -> 2020-06-01 UTC-"))
[1] +2020-01-01T00:00:00+00:00 -> 2020-03-01T00:00:00+00:00-

R> union(as.nanoival("+2019-03-01 UTC -> 2020-03-01 UTC-"),
         as.nanoival("+2020-01-01 UTC -> 2020-06-01 UTC-"))
[1] +2019-03-01T00:00:00+00:00 -> 2020-06-01T00:00:00+00:00-

R> setdiff(as.nanoival("+2019-03-01 UTC -> 2020-03-01 UTC-"),
           as.nanoival("+2020-01-01 UTC -> 2020-06-01 UTC-"))
[1] +2019-03-01T00:00:00+00:00 -> 2020-01-01T00:00:00+00:00-    
```

#### Periods

```r
R> as.nanoperiod("1y1m1w1d/01:01:01.000_000_001")
[1] 13m8d/01:01:01.000_000_001
R> nanoperiod(months=13, days=-1, duration="01:00:00")
[1] 13m-1d/01:00:00

R> ones <- as.nanoperiod("1y1m1w1d/01:01:01.000_000_001")
R> nanoperiod.month(ones); nanoperiod.day(ones); nanoperiod.nanoduration(ones)
[1] 13
[1] 8
[1] 01:01:01.000_000_001

R> plus(v, as.nanoperiod("1y1m"), tz="UTC")
[1] 2021-04-22T03:09:20.732122001+00:00
[2] 2021-04-22T03:09:20.732122002+00:00
[3] 2021-04-22T03:09:20.732122003+00:00
[4] 2021-04-22T03:09:20.732122004+00:00
[5] 2021-04-22T03:09:20.732122005+00:00
```

#### Durations

```{r}
R> nanoduration(hours=1, minutes=1, seconds=1, nanoseconds=1)
R> as.nanoduration("00:00:01")
R> as.nanoduration("-00:00:01")
R> as.nanoduration("100:00:00")
R> as.nanoduration("00:00:00.000_000_001")
```

#### Sequences

``` {r}
R> from <- as.nanotime("2018-09-14T12:44:00+00:00")
R> seq(from, by=as.nanoperiod("1y"), length.out=4, tz="Europe/London")
[1] 2018-09-14T12:44:00+00:00
[2] 2019-09-14T12:44:00+00:00
[3] 2020-09-14T12:44:00+00:00
[4] 2021-09-14T12:44:00+00:00
```

### Technical Details

The [bit64](https://cran.r-project.org/package=bit64) package (by [Jens
Oehlschlägel](https://github.com/joehl)) supplies the `integer64` type used to store the nanosecond
resolution time as (positive or negative) offsets to the epoch of January 1, 1970. The
[RcppCCTZ](https://dirk.eddelbuettel.com/code/rcpp.cctz.html) package supplies the formatting and
parsing routines based on the (modern C++) library [CCTZ](https://github.com/google/cctz) from
Google, when the parsing cannot be done using a fast built-in parser. `integer64` is also used for
the type `nanoduration`, whereas `nanoival` and `nanoperiod` are stored in a `complex`, i.e. over
128 bits.

### Status

The package is by now fairly mature, has been rewritten once (to go from S3
to S4) and has recently received a sizeable feature extension. There may
still be changes, though there should generally never be breaking ones. The
package also has an extensive test suite, and very good code coverage.

See the [issue tickets](https://github.com/eddelbuettel/nanotime/issues) for an up to date list of
potentially desirable, possibly planned, or at least discussed items.

### Installation

The package is on [CRAN](https://cran.r-project.org) and can be installed via a standard

```r
install.packages("nanotime")
```

whereas in order to install development versions a

```r
remotes::install_github("eddelbuettel/nanotime")  # dev version
```

should suffice.


### Authors

Dirk Eddelbuettel and Leonardo Silvestri

### License

GPL (>= 2)

# Nanotime

### nanosecond precision time

The nanotime module provides a time object that keeps time as the number of
nanoseconds since the UNIX epoch. In other words, it is a 64bit UNIX timestamp
with nanosecond precision.

[https://github.com/jbenet/nanotime](https://github.com/jbenet/nanotime)

## Install

    sudo python setup.py install

## License

Nanotime is under the MIT License.

## Usage

simple:

    >>> import nanotime
    >>> nanotime.now()
    nanotime.nanotime(1309847805647559936)
    >>> str(nanotime.now())
    '2011-07-04 23:36:59.694926080'

conversions:

    >>> nanotime.seconds(100)
    nanotime.nanotime(100000000000)

    >>> import datetime
    >>> nanotime.datetime(datetime.datetime.now())
    nanotime.nanotime(1309847995629583872)

    >>> import time
    >>> nanotime.timestamp(time.time())
    nanotime.nanotime(1309849172822870016)

## Tests

There is one basic test file so far. Run with:

    nosetests

library(nanotime)
suppressMessages(library(data.table))
suppressMessages(library(ggplot2))

set.seed(42)
N <- 300
rainyday <- ISOdatetime(2016,9,28,8,30,0) # made up
shinyday <- ISOdatetime(2016,9,21,8,30,0) # made up too
rdsent <- nanotime(rainyday) + cumsum(10*rpois(N, lambda=4)) 	# random sent time 
sdsent <- nanotime(shinyday) + cumsum(10*rpois(N, lambda=4))	# identical sent process for both
rdrecv <- rdsent + 10*rlnorm(N, 0.30, 0.25)                     # postulate higher mean and sd
sdrecv <- sdsent + 10*rlnorm(N, 0.10, 0.20)		            	# for rainy than shiny
raw <- data.table(rdsent, rdrecv, sdsent, sdrecv)
raw[, `:=`(rainy=as.numeric(rdrecv-rdsent),
           shiny=as.numeric(sdrecv-sdsent))]

## melt into long format
plotdata <- melt(raw[,.(rainy,shiny)], measure.vars=1:2, variable.name="day", value.name="time")
## and plot
ggplot(plotdata, aes(day, time)) + geom_violin(aes(fill=day)) + coord_flip() + 
    ylab("Message Time in Nanoseconds") + xlab("Weather Conditions") +
    ggtitle("Nanosecond Delay", "Under Different Weather Conditions")

tfile <- tempfile(pattern="raw", fileext=".csv")
fwrite(raw, file=tfile)
cooked <- fread(tfile)
## csv files are not 'typed' so need to recover types explicitly
cooked[, `:=`(rdsent=nanotime(rdsent),
              rdrecv=nanotime(rdrecv),
              sdsent=nanotime(sdsent),
              sdrecv=nanotime(sdrecv))]
## now saved and restrored data are identical
all.equal(raw, cooked)

##' @rdname nanotime
setClass("nanotime", contains = "integer64")
## setClass("nanotime", contains = "numeric") also possible if we don't want integer64

##' Nanosecond resolution datetime functionality
##'
##' Functions to operate on nanosecond time resolution using integer64
##' bit representation. Conversion functions for several standard R
##' types are provided, and more will be added as needed.
##'
##' Notice that the conversion from POSIXct explicitly sets the last
##' three digits to zero. Nanosecond time stored in a 64-bit integer
##' has nineteen digits precision where doubles (which are used
##' internally for POSIXct as well) only have sixteen digits.  So
##' rather than showing three more (essentially \emph{random}) digits
##' it is constructed such that these three additional digits are
##' zeros.
##'
##' @section Caveats:
##'
##' Working with dates and times is \emph{difficult}. One needs a
##' representation of both \emph{time points} and \emph{time
##' duration}. In R, think of \code{Date} or \code{POSIXct} objects
##' for the former, and \code{difftime} for the later. Here we have
##' time points \code{nanotime}, an interval type \code{nanoival} and
##' two flavors of duration which are a simple count of nanoseconds
##' \code{nanoduration} and a calendar duration that is able to track
##' concepts such as months and days \code{nanoperiod}. Point in time
##' and intervals are all based on durations relative to the epoch of
##' January 1, 1970.
##'
##' @section Input and Output Format:
##'
##' Formatting and character conversion for \code{nanotime} objects is
##' done by functions from the \pkg{RcppCCTZ} package relying
##' on code from its embedded \code{CCTZ} library. The default format
##' is ISO3339 compliant: \code{\%Y-\%m-\%dT\%H:\%M:\%E9S\%Ez}. It
##' specifies a standard ISO 8601 part for date and time --- as well
##' as nine digits of precision for fractional seconds (down to
##' nanoseconds) and on offset (typically zero as we default to UTC).
##' It can be overriden by using \code{options()} with the key of
##' \code{nanotimeFormat} and a suitable value. Similarly,
##' \code{nanotimeTz} can be used to select a different timezone.
##'
##' For input, some slack it cut, and various shortened formats are
##' accepted by default such as \code{2020-03-10} or \code{2020-03-10
##' 18:16:00}, or \code{2020-03-10 18:16:00.001} (and the \sQuote{T}
##' separator is optional.
##'
##' @section \code{tz} parameter usage in constructors:
##'
##' The \code{tz} parameter is allowed only when constructing a
##' \code{nanotime} from a \code{character}. This is because any
##' \code{numeric}, \code{Date} and \code{POSIXct} is de facto
##' considered an offset since the epoch. On the contrary, a
##' \code{character} is considered interpretable and hence if it does
##' not contain a timezone in its representation, it is possible to
##' specify the \code{tz} argument to specify in which timezone it
##' should be interpreted. This is useful in particular if one wants
##' to convert a \code{Date} to be aligned to the beginning of the day
##' in a specific timezone; in this case one should convert the
##' \code{Date} to a \code{character} before calling the
##' \code{nanotime} constructor with the desired timezone.
##' 
##' @param x,from \code{nanotime} objects
##' @param tz character specifying a timezone which is required for
##'     \code{as.POSIXct}, \code{as.POSIXlt} and can be specified for
##'     \code{as.nanotime}, \code{format} and \code{print}; it can
##'     also be set via \code{options("nanotimeTz")} and uses
##'     \sQuote{UTC} as a default and fallback
##' @param ... further arguments passed to or from methods.
##' @param e1 Operand of class \code{nanotime}
##' @param e2 Operand of class \code{nanotime}
##' @param format A character string. Can also be set via
##'     \code{options("nanotimeFormat")} and uses
##'     \sQuote{\%Y-\%m-\%dT\%H:\%M:\%E9S\%Ez} as a default and
##'     fallback
##' @param digits Required for \code{Math2} signature but ignored here
##' @param object argument for method \code{show}
##' @param na.rm a logical indicating whether missing values should be
##'     removed.
##' @param i index specifying elements to extract or replace.
##' @param j Required for \code{[} signature but ignored here
##' @param drop Required for \code{[} signature but ignored here
##' @param z Required for \code{Complex} signature but ignored here
##' @param value argument for \code{nanotime-class}
##' @param quote indicates if the output of \code{print} should be
##'     quoted
##' @param accurate in the conversion from \code{POSIXct} to
##'     \code{nanotime}, indicates if one wants to preserve the
##'     maximum precision possible; the default is \code{TRUE}, but in
##'     most situations the loss of precision is negligible, and
##'     setting this parameter to \code{FALSE} will make the
##'     conversion nearly an order of magnitude faster
##' @return A nanotime object
##' @author Dirk Eddelbuettel
##' @author Leonardo Silvestri
##' @examples
##' \dontrun{
##' x <- nanotime(1)
##' print(x)
##' as.nanotime("1970-01-01T00:00:00.000000001+00:00")
##' as.nanotime("2020-03-10 Europe/Berlin")
##' as.nanotime("2020-03-10 18:31:23.001", tz="America/New_York")
##' as.nanotime("2020-03-10T040947190301440", format="%Y-%m-%dT%H%M%S%E*f")
##' x <- x + 1
##' print(x)
##' format(x)
##' x <- x + 10
##' print(x)
##' format(x)
##' nanotime(Sys.time()) + 1:3  # three elements each 1 ns apart
##' seq(x, by=as.nanoperiod("1d"), length.out=5, tz="Asia/Tokyo")
##' }
##' @seealso \code{\link{nanoival}}, \code{\link{nanoduration}},
##' \code{\link{nanoperiod}}, \code{\link{seq.nanotime}} as well as
##' the documentation in package \pkg{RcppCCTZ}.
nanotime <- function(from, ...) {
    if (missing(from)) {
        from = NULL
    }
    res <- as.integer64(from)
    names(res) <- names(from)
    new("nanotime", res)
}
##' @noRd
setGeneric("nanotime")

##' @rdname nanotime
as.nanotime <- function(from, ...) {
    if (missing(from)) {
        from = NULL
    }
    res <- as.integer64(from)
    names(res) <- names(from)
    new("nanotime", res)
}
##' @noRd
setGeneric("as.nanotime")

setAs("integer64", "nanotime", function(from) {
    res <- as.integer64(from)
    names(res) <- names(from)
    new("nanotime", res)
})


.getTz <- function(x, tz="") {
    if (tz=="") {
        if (!is.null(tzone <- attr(x, "tzone")))
            tz <- tzone
        else
            tz <- getOption("nanotimeTz", default="UTC")
    }
    tz
}

.getFormat <- function(format="") {
    if (format=="") {
        format <- getOption("nanotimeFormat", default="%Y-%m-%dT%H:%M:%EXS%Ez")
    }
    format
}


.secondaryNanotimeParse <- function(x, format="", tz="") {
    if (length(x) == 0) return(character(0)) # nocov
    format <- gsub("%EXS", "%E9S", .getFormat(format))
    tz <- .getTz(x, tz)
    n <- names(x)
    d <- RcppCCTZ::parseDouble(x, fmt=format, tzstr=tz)
    res <- new("nanotime", as.integer64(d[,1]) * as.integer64(1e9) + as.integer64(d[, 2]))
    if (!is.null(n)) {
        names(S3Part(res, strictS3=TRUE)) <- n
    }
    res
}


.nanotime_character <- function(from, format="", tz="") {
    tryCatch(nanotime_make_impl(from, tz), error=function(e) {
        if (grepl("Cannot retrieve timezone", e$message) ||
            e$message == "timezone is specified twice: in the string and as an argument") {
            stop(e$message)
        } else {
            .secondaryNanotimeParse(from, format, tz)
        }
    })
}

##' @rdname nanotime
setMethod("nanotime", "character", .nanotime_character)

##' @rdname nanotime
setMethod("as.nanotime", "character", .nanotime_character)

setAs("character", "nanotime", function(from) .nanotime_character(from))

##' @rdname nanotime
## This does not lead to S3 dispatch, the call must be 'nanotime.matrix'
nanotime.matrix <- function(x) {
    n <- names(x)
    res <- new("nanotime", as.integer64(x[,1]) * as.integer64(1e9) + as.integer64(x[, 2]))
    if (!is.null(n)) {
        names(res) <- n 						## #nocov
    }
    res
}


.nanotime_posixct <- function(from, accurate=TRUE) {
    n <- names(from)
    from <- as.numeric(from)
    if (accurate) {
        frac <- from - trunc(from)
        res  <- new("nanotime", bit64::as.integer64(from)*1e9 + round(frac*1e9))
    } else {
        res <- new("nanotime", bit64::as.integer64(as.numeric(from) * 1e9))
    }
    if (!is.null(n)) {
        names(S3Part(res, strictS3=TRUE)) <- n
    }
    res
}

##' @rdname nanotime
setMethod("nanotime", signature(from="POSIXct"), .nanotime_posixct)

##' @rdname nanotime
setMethod("as.nanotime", signature(from="POSIXct"), .nanotime_posixct)

setAs("POSIXct", "nanotime", function(from) .nanotime_posixct(from))


##' @rdname nanotime
setMethod("nanotime", "POSIXlt", function(from) nanotime(as.POSIXct(from)))

##' @rdname nanotime
setMethod("as.nanotime", "POSIXlt", function(from) nanotime(as.POSIXct(from)))

setAs("POSIXlt", "nanotime", function(from) nanotime(as.POSIXct(from)))


##' @rdname nanotime
setMethod("nanotime", "Date", function(from) nanotime(as.POSIXct(from)))

##' @rdname nanotime
setMethod("as.nanotime", "Date", function(from) nanotime(as.POSIXct(from)))

setAs("Date", "nanotime", function(from) nanotime(as.POSIXct(from)))


##' @rdname nanotime
setMethod("print",
          "nanotime",
          function(x, format="", tz="", quote=FALSE, ...) {
              format <- .getFormat(format)
              tz <- .getTz(x, tz)
              max.print <- options()$max.print
              if (length(x) > max.print) {                   		## #nocov start
                  f <- head(x, max.print)
                  print(format(f, format, tz, ...), quote=quote)
                  cat(paste(' [ reached getOption("max.print") -- omitted',
                            length(x) - max.print, "entries ]\n"))
              }                                              		## #nocov end
              else {
                  print(format(x, format, tz, ...), quote=quote)
              }
              invisible(x)
          })

##' @rdname nanotime
setMethod("show",
          signature("nanotime"),
          function(object) print(object))  				## #nocov

##' @rdname nanotime
format.nanotime <- function(x, format="", tz="", ...)
{
    if (length(x) == 0) {
        "nanotime(0)"
    } else {
        format <- .getFormat(format)
        tz <- .getTz(x, tz)
        if (length(x) == 0) {
            return(character(0))						## #nocov
        }
        bigint <- as.integer64(x)
        secs  <- as.integer64(bigint / as.integer64(1000000000))
        nanos <- bigint - secs * as.integer64(1000000000)

        nanos_notna <- nanos[!is.na(nanos)]

        ## EXS has special meaning for us: print with the least number of nanotime digits
        if (length(nanos_notna) != 0 && isTRUE(as.logical(grep("%EXS", format)))) {
            if (all(nanos_notna %% 1000000000 == 0)) {
                format <- gsub("%EXS", "%S", format)
            } else if (all(nanos_notna %% 1000000 == 0)) {
                format <- gsub("%EXS", "%E3S", format)
            } else if (all(nanos_notna %% 1000 == 0)) {
                format <- gsub("%EXS", "%E6S", format)
            } else {
                format <- gsub("%EXS", "%E9S", format)
            }
        }

        isna <- is.na(x)
        if (any(isna)) {
            secs[isna] <- 0
            nanos[isna] <- 0
        }
        res <- RcppCCTZ::formatDouble(as.double(secs), as.double(nanos), fmt=format, tgttzstr=tz)
        res[isna] <- as.character(NA)
        n <- names(x)
        if (!is.null(n)) {
            names(res) <- n
        }
        res
    }
}

##' @rdname nanotime
index2char.nanotime <- function(x, ...) {
    bigint <- as.integer64(x)
    secs  <- as.integer64(bigint / as.integer64(1000000000))
    nanos <- bigint - secs * as.integer64(1000000000)
    RcppCCTZ::formatDouble(as.double(secs), as.double(nanos),
                           fmt=getOption("nanotimeFormat", default="%Y-%m-%dT%H:%M:%E9S%Ez"),
                           tgttzstr=getOption("nanotimeTz", default="UTC"))
}

##' @rdname nanotime
as.POSIXct.nanotime <- function(x, tz="", ...) {
    ## if (verbose) warning("Lossy conversion dropping precision")
    if ((tz == "" || is.null(tz)) && length(attr(x, "tzone")) > 0) {
        tz <- attr(x, "tzone")
    }
    pt <- as.POSIXct(as.double(S3Part(x, strictS3=TRUE)/1e9), tz=tz, origin="1970-01-01")
    pt
}

setAs("nanotime", "POSIXct", function(from) as.POSIXct.nanotime(from))


##' @rdname nanotime
as.POSIXlt.nanotime <- function(x, tz="", ...) {
    as.POSIXlt(as.POSIXct(x, tz=tz))
}

setAs("nanotime", "POSIXlt", function(from) as.POSIXlt.nanotime(from))

##' @rdname nanotime
as.Date.nanotime <- function(x, ...) {
    arguments <- list(...)
    if (!("tz" %in% names(args))) {
        arguments <- c(arguments, list(tz="UTC"))
    }
    other_args <- setdiff(names(arguments), list('x', 'tz'))
    if (length(other_args) > 0) {
        stop(paste("'as.Date' called with arguments other than 'tz': ",
                   paste0("'", other_args, "'", collapse=", ")))
    }
    as.Date(ISOdate(year = nano_year(x, arguments$tz),
                    month = nano_month(x, arguments$tz),
                    day = nano_mday(x, arguments$tz)))
}

setAs("nanotime", "Date", function(from) as.Date(as.POSIXct(from)))


##' @rdname nanotime
as.data.frame.nanotime <- function(x, ...) {
    ret <- as.data.frame(S3Part(x, strictS3=TRUE), ...)
    ## this works, but see if there's a more idiomatic and efficient way
    ## of doing this:
    ret[] <- nanotime(S3Part(x, strictS3=TRUE))
    ret
}

##' @rdname nanotime
##' @method as.integer64 nanotime
as.integer64.nanotime <- function(x, ...) {
    S3Part(x, strictS3=TRUE)
}

setAs("nanotime", "integer64", function(from) S3Part(from, strictS3=TRUE))

## ------------ `-`
##' @rdname nanotime
setMethod("-", c("nanotime", "character"),
          function(e1, e2) {
              stop("invalid operand types")
          })

##' @rdname nanotime
setMethod("-", c("nanotime", "nanotime"),
          function(e1, e2) {
              new("nanoduration", S3Part(e1, strictS3=TRUE) - S3Part(e2, strictS3=TRUE))
          })

##' @rdname nanotime
setMethod("-", c("nanotime", "integer64"),
          function(e1, e2) {
              new("nanotime", S3Part(e1, strictS3=TRUE) - e2)
          })

##' @rdname nanotime
setMethod("-", c("nanotime", "numeric"),
          function(e1, e2) {
              new("nanotime", S3Part(e1, strictS3=TRUE) - e2)
          })

##' @rdname nanotime
setMethod("-", c("ANY", "nanotime"),
          function(e1, e2) {
              stop("invalid operand types")
          })

##' @rdname nanotime
setMethod("-", c("nanotime", "ANY"),
          function(e1, e2) {
              if (missing(e2)) {
                 stop("unary '-' is not defined for \"nanotime\" objects")
              }
              else {
                  stop("invalid operand types")    			## #nocov
              }
          })


## ----------- `+`
##' @rdname nanotime
setMethod("+", c("nanotime", "ANY"),
          function(e1, e2) {
              if (missing(e2)) {
                 stop("unary '+' is not defined for \"nanotime\" objects")  ## #nocov
              }
              else {
                  stop("invalid operand types")
              }
          })

##' @rdname nanotime
setMethod("+", c("nanotime", "integer64"),
          function(e1, e2) {
              new("nanotime", S3Part(e1, strictS3=TRUE) + e2)
          })

##' @rdname nanotime
setMethod("+", c("nanotime", "numeric"),
          function(e1, e2) {
              new("nanotime", S3Part(e1, strictS3=TRUE) + e2)
          })

##' @rdname nanotime
setMethod("+", c("ANY", "nanotime"),
          function(e1, e2) {
              stop("invalid operand types")
          })

##' @rdname nanotime
setMethod("+", c("integer64", "nanotime"),
          function(e1, e2) {
              new("nanotime", e1 + S3Part(e2, strictS3=TRUE))  		## #nocov
          })

##' @rdname nanotime
setMethod("+", c("numeric", "nanotime"),
          function(e1, e2) {
              new("nanotime", e1 + S3Part(e2, strictS3=TRUE))
          })

##' @rdname nanotime
setMethod("+", c("nanotime", "nanotime"),
          function(e1, e2) {
              stop("invalid operand types")
          })


## ---------- other ops

##' @rdname nanotime
setMethod("Arith", c("nanotime", "nanotime"),
          function(e1, e2) {
              stop("operation not defined for \"nanotime\" objects")
          })

##' @rdname nanotime
setMethod("Arith", c("nanotime", "ANY"),
          function(e1, e2) {
              stop("operation not defined for \"nanotime\" objects")
          })

##' @rdname nanotime
setMethod("Arith", c("ANY", "nanotime"),
          function(e1, e2) {
              stop("operation not defined for \"nanotime\" objects")    ## #nocov
          })

##' @rdname nanotime
setMethod("Compare", c("nanotime", "character"),
          function(e1, e2) {
              ne2 <- nanotime(e2)
              callNextMethod(e1, ne2)
          })

##' @rdname nanotime
setMethod("Compare", c("character", "nanotime"),
          function(e1, e2) {
              ne1 <- nanotime(e1)
              callNextMethod(ne1, e2)
          })

##' @rdname nanotime
setMethod("Compare", c("nanotime", "POSIXt"),
          function(e1, e2) {
              ne2 <- nanotime(e2)
              callNextMethod(e1, ne2)
          })

##' @rdname nanotime
setMethod("Compare", c("POSIXt", "nanotime"),
          function(e1, e2) {
              ne1 <- nanotime(e1)
              callNextMethod(ne1, e2)
          })

##' @rdname nanotime
setMethod("Compare", c("nanotime", "ANY"),
          function(e1, e2) {
              if (inherits(e2, "nanotime")) {
                  e2 <- S3Part(e2, strictS3=TRUE)
              }
              callNextMethod(S3Part(e1, strictS3=TRUE), e2)
          })

##' @rdname nanotime
setMethod("Logic", c("nanotime", "ANY"),
          function(e1, e2) {
              ## this is the same error message that R gives for "A" | "A"
              stop("operations are possible only for numeric, logical or complex types")
          })

##' @rdname nanotime
setMethod("Logic", c("ANY", "nanotime"),
          function(e1, e2) {
              ## this is the same error message that R gives for "A" | "A"
              stop("operations are possible only for numeric, logical or complex types")
          })

##' @rdname nanotime
setMethod("Math", c("nanotime"),
          function(x) {
              ## this is the same error message that R gives for abs("A")
              stop("non-numeric argument to mathematical function")
          })

##' @rdname nanotime
setMethod("Math2", c("nanotime"),
          function(x, digits) {
              ## this is the same error message that R gives for round("A")
              stop("non-numeric argument to mathematical function")
          })

##' @rdname nanotime
setMethod("Summary", c("nanotime"),
          function(x, ..., na.rm = FALSE) {
              ## this is the same error message that R gives for sum("A")
              stop("invalid 'type' (nanotime) of argument")
          })

##' @rdname nanotime
setMethod("min", c("nanotime"),
          function(x, ..., na.rm = FALSE) {
              new("nanotime", callNextMethod())
          })

##' @rdname nanotime
setMethod("max", c("nanotime"),
          function(x, ..., na.rm = FALSE) {
              new("nanotime", callNextMethod())
          })

##' @rdname nanotime
setMethod("range", c("nanotime"),
          function(x, ..., na.rm = FALSE) {
              new("nanotime", callNextMethod())
          })


##' @rdname nanotime
setMethod("Complex", c("nanotime"),
          function(z) {
              ## this is the same error message that R gives for Arg("A")
              stop("non-numeric argument to function")
          })

## ----------- non ops

##' @rdname nanotime
setMethod("[[",
          signature("nanotime"),
          function (x, i, j, ..., drop=FALSE) {
              new("nanotime", callNextMethod())
})

##' @rdname nanotime
setMethod("[",
          signature("nanotime", "numeric"),
          function (x, i, j, ..., drop=FALSE) {
              if (!missing(j) || length(list(...)) > 0) {
                  warning("unused indices or arguments in 'nanotime' subsetting")
              }
              if (isTRUE(any(i < 0))) {
                  new("nanotime", S3Part(x, strictS3=TRUE)[i])
              } else {
                  nanotime_subset_numeric_impl(x, i)
              }
          })

##' @rdname nanotime
setMethod("[",
          signature("nanotime", "logical"),
          function (x, i, j, ..., drop=FALSE) {
              if (!missing(j) || length(list(...)) > 0) {
                  warning("unused indices or arguments in 'nanotime' subsetting")
              }
              nanotime_subset_logical_impl(x, i)
          })

##' @rdname nanotime
setMethod("[",
          signature("nanotime", "character"),
          function (x, i, j, ..., drop=FALSE) {
              ## we don't implement 'period_subset_character_impl' but
              ## do the gymnastic of finding the NAs here at R level;
              ## it's not as efficient, but using 'character' indexing
              ## is by itself inefficient, so the overhead should have
              ## no practical consequences.
              if (!missing(j) || length(list(...)) > 0) {
                  warning("unused indices or arguments in 'nanotime' subsetting")
              }
              na_index <- !(i %in% names(x)) | is.na(i)
              res <- new("nanotime", S3Part(x, strictS3=TRUE)[i])
              res[na_index] <- NA_nanotime_
              res
          })

##' @rdname nanotime
setMethod("[",
          signature("nanotime", "ANY"),
          function (x, i, j, ..., drop=FALSE) {
              stop("']' not defined on 'nanotime' for index of type 'ANY'")
          })

##' @rdname nanotime
setMethod("[<-",
          signature("nanotime"),
          function (x, i, j, ..., value) {
              new("nanotime", callNextMethod())
          })

##' @rdname nanotime
c.nanotime <- function(...) {
    nanotime(c.integer64(...))
}

##' @rdname nanotime
##' @name nanotime-package
NULL

##' @rdname nanotime
setMethod("names<-",
          signature("nanotime"),
          function(x, value) {
              names(S3Part(x, strictS3=TRUE)) <- value
              x
          })

##' @rdname nanotime
setMethod("is.na",
          signature("nanotime"),
          function(x) {
              callNextMethod(S3Part(x, strictS3=TRUE))
          })


##' Sequence Generation
##'
##' Generate a sequence of \code{nanotime}
##'
##' @param ... arguments passed to or from methods; the only
##'     interesting additional argument is \code{tz} where the
##'     \code{to} argument is of type \code{nanoperiod}
##' @param from,to the starting and (maximal) end values of the
##'     sequence
##' @param by \code{nanoduration} or \code{nanoperiod} increment of
##'     the sequence; note that if the class is \code{nanoperiod} the
##'     additional argument \code{tz} must be speficied and is of
##'     \code{character} type indicating a timezone
##' @param length.out integer indicating the desired length of the sequence
##' @param along.with take the length from the length of this argument.
##' @examples
##' \dontrun{
##' from <- as.nanotime("2018-01-14T12:44:00+00:00")
##' to   <- as.nanotime("2019-01-14T12:44:00+00:00")
##' seq(from, to, by=as.nanoperiod("1m"), tz="America/New_York")
##' seq(from, by=as.nanoperiod("1y"), length.out=4, tz="Europe/London")
##' }
##' @method seq nanotime
seq.nanotime <-
    function(from, to = NULL, by = NULL, length.out = NULL, along.with = NULL, ...)
{
    if((One <- nargs() == 1L)) {
	stop("'seq.nanotime' cannot be called with only one argument")
    }
    if (sum(is.null(to), is.null(by), is.null(length.out) && is.null(along.with)) > 1) {
        stop("at least two of 'to', 'by', and 'length.out' or 'along.with' must be specified")
    }
    is.logint <- function(.) (is.integer(.) || is.logical(.)) && !is.object(.)
    if (!missing(along.with) && !is.null(along.with)) {
	length.out <- length(along.with)
    }
    else if(!missing(length.out) && !is.null(length.out)) {
        len <- length(length.out)
        if(!len) stop("argument 'length.out' must be of length 1")
        if(len > 1L) {
            warning("first element used of 'length.out' argument")
            length.out <- length.out[1L]
        }
	if(!(intn1 <- is.logint(length.out)))
	    length.out <- as.numeric(ceiling(length.out))
    }
    if (length(from) != 1L) stop("'from' must be of length 1")
    if (!missing(to) && !is.null(to) && length(to) != 1L) stop("'to' must be of length 1")
    if (!missing(to) && !is.null(to) && !(inherits(to, "nanotime")))
	stop("'to' must be a 'nanotime'")
    if (is.null(length.out)) {
	if (missing(by) || is.null(by))
	    stop("'by' must be specified if 'length.out' is NULL")
	else { # dealing with 'by'
            if (length(by) != 1L) stop("'by' must be of length 1")
            if (inherits(by, "nanoperiod")) {
                ## fish out the 'tz' parameter:
                args <- list(...)
                if (!any("tz" %in% names(args))) {
                    stop("'tz' is a mandatory argument for sequences with a 'period' step")
                }
                period_seq_from_to_impl(from, to, by, args$tz)
            } else {
                nanotime(seq(as.integer64(from), as.integer64(to), by=by))
            }
	}
    }
    else if(!is.finite(length.out) || length.out < 0L)
	stop("'length.out' must be a non-negative number")
    else if (length.out == 0L) nanotime()
    else if (missing(by) || is.null(by)) {
        ## cannot be with 'period', so just call the S3 function:
        ## calculate 'by' because of 'bit64' bug:
        by = as.integer64((to - from) / (length.out - 1))
        nanotime(seq(as.integer64(from), as.integer64(to), by, NULL, NULL, ...))
    }
    else if (missing(to) || is.null(to)) {
        if (length(by) != 1L) stop("'by' must be of length 1")
        if (inherits(by, "nanoperiod")) {
            ## fish out the 'tz' parameter:
            args <- list(...)
            if (!any("tz" %in% names(args))) {
                stop("'tz' is a mandatory argument for sequences with a 'period' step")
            }
            period_seq_from_length_impl(from, by, as.integer64(length.out), args$tz)
        } else {
            nanotime(seq(as.integer64(from), by=as.integer64(by),
                         length.out=length.out, along.with=along.with, ...))
        }
    }
    else stop("too many arguments")
}


##' @rdname seq.nanotime
setMethod("seq", c("nanotime"), seq.nanotime)


##' Test if Two Objects are (Nearly) Equal
##'
##' Compare \code{target} and \code{current} testing \sQuote{near
##' equality}.  If they are different, comparison is still made to
##' some extent, and a report of the differences is returned.  Do not
##' use \code{all.equal} directly in \code{if} expressions---either
##' use \code{isTRUE(all.equal(....))} or \code{\link{identical}} if
##' appropriate.
##'
##' @param target,current \code{nanotime} arguments to be compared
##' @param ... further arguments for different methods
##' @param tolerance numeric >= 0. Differences smaller than
##'     \code{tolerance} are not reported.  The default value is close
##'     to \code{1.5e-8}.
##' @param  scale \code{NULL} or numeric > 0, typically of length 1 or
##'          \code{length(target)}.  See \sQuote{Details}.
##' @param  countEQ logical indicating if the \code{target == current} cases should be
##'          counted when computing the mean (absolute or relative)
##'          differences.  The default, \code{FALSE} may seem misleading in
##'          cases where \code{target} and \code{current} only differ in a few
##'          places; see the extensive example.
##' @param formatFUN a \code{function} of two arguments, \code{err}, the relative, absolute
##'          or scaled error, and \code{what}, a character string indicating
##'          the _kind_ of error; maybe used, e.g., to format relative and
##'          absolute errors differently.
##' @param check.attributes logical indicating if the \code{attributes} of \code{target}
##'          and \code{current} (other than the names) should be compared.
##'
##' @seealso \code{\link{identical}}, \code{\link{isTRUE}},
##'     \code{\link{==}}, and \code{\link{all}} for exact equality
##'     testing.
##'
##' @method all.equal nanotime
##'
all.equal.nanotime <- function(target, current, tolerance = sqrt(.Machine$double.eps), 
                               scale = NULL, countEQ = FALSE,
                               formatFUN = function(err, what) format(err), ..., check.attributes = TRUE) {
    if (inherits(target, "nanotime")) target  <- as.integer64(target)
    if (inherits(current, "nanotime")) current <- as.integer64(current)
    
    if (!is.numeric(tolerance)) 
        stop("'tolerance' should be numeric")
    if (!is.numeric(scale) && !is.null(scale)) 
        stop("'scale' should be numeric or NULL")
    if (!is.logical(check.attributes)) 
        stop(gettextf("'%s' must be logical", "check.attributes"), 
             domain = NA)
    msg <- NULL
    msg <- if (check.attributes) 
               attr.all.equal(target, current, tolerance = tolerance, 
                              scale = scale, ...)
    if (data.class(target) != data.class(current)) {
        msg <- c(msg, paste0("target is ", data.class(target), 
                             ", current is ", data.class(current)))
        return(msg)
    }
    lt <- length(target)
    lc <- length(current)
    cplx <- is.complex(target)
    if (lt != lc) {
        if (!is.null(msg)) 
            msg <- msg[-grep("\\bLengths\\b", msg)]
        msg <- c(msg, paste0(if (cplx) "Complex" else "Numeric", 
                             ": lengths (", lt, ", ", lc, ") differ"))
        return(msg)
    }
    out <- is.na(target)
    if (any(out != is.na(current))) {
        msg <- c(msg, paste("'is.NA' value mismatch:", sum(is.na(current)), 
                            "in current", sum(out), "in target"))
        return(msg)
    }
    out <- out | target == current
    if (all(out)) 
        return(if (is.null(msg)) TRUE else msg)
    if (countEQ) {
        N <- length(out)
        sabst0 <- sum(abs(target[out]))
    } else {
        sabst0 <- 0
    }
    target <- target[!out]
    current <- current[!out]
    if (!countEQ) 
        N <- length(target)
## the following is in the orignal function, but can't be reached here:
##    if (is.integer(target) && is.integer(current)) 
##        target <- as.double(target)
    xy <- sum(abs(target - current))/N
    what <- if (is.null(scale)) {
                xn <- (sabst0 + sum(abs(target)))/N
                if (is.finite(xn) && xn > tolerance) {
                    xy <- xy/xn
                    "relative"
                } else {
                    "absolute"
                }
            } else {
                stopifnot(all(scale > 0))
                xy <- xy/scale
                if (all(abs(scale - 1) < 1e-07)) 
                    "absolute"
                else "scaled"
            }
    if (cplx) 
        what <- paste(what, "Mod")      # nocov
    if (is.na(xy) || xy > tolerance) 
        msg <- c(msg, paste("Mean", what, "difference:", formatFUN(xy, what)))
    if (is.null(msg)) {
        TRUE                            # nocov
    } else msg
}

##' @rdname all.equal.nanotime
setMethod("all.equal", c(target = "nanotime", current="ANY"), all.equal.nanotime)


##' @rdname nanotime
NA_nanotime_ <- nanotime(NA)

##' Get a component of a date time
##'
##' Get a component of a date time. \code{nano_wday} returns the
##' numeric position in a week, with Sunday == 0. \code{nano_mday}
##' returns the numeric day (i.e. a value from 1 to
##' 31). \code{nano_month} returns the month (i.e. a value from 1 to
##' 12). \code{nano_year} returns the year.
##'
##' Note that the \code{tz} parameter is mandatory because the day
##' boundary is different depending on the time zone and
##' \code{nanotime} does not store the timezone as it is just an
##' offset in nanoseconds from the epoch.
##'
##' @param x a \code{nanotime} object
##' @param tz \code{character} a string representing a timezone
##' @examples
##' \dontrun{
##' nano_wday(as.nanotime("2020-03-14 23:32:00-04:00"), "America/New_York")
##' nano_wday(as.nanotime("2020-03-14 23:32:00 America/New_York"), "Europe/Paris")
##' nano_mday(as.nanotime("2020-03-14 23:32:00-04:00"), "America/New_York")
##' nano_mday(as.nanotime("2020-03-14 23:32:00 America/New_York"), "Europe/Paris")
##' nano_month(as.nanotime("2020-12-31 23:32:00-04:00"), "America/New_York")
##' nano_month(as.nanotime("2020-12-31 23:32:00 America/New_York"), "Europe/Paris")
##' nano_year(as.nanotime("2020-12-31 23:32:00-04:00"), "America/New_York")
##' nano_year(as.nanotime("2020-12-31 23:32:00 America/New_York"), "Europe/Paris")
##' }
##'
##' @rdname nano_year
##' @aliases nano_wday
##' @aliases nano_wday,nanotime-method nano_mday,nanotime-method
##' @aliases nano_month,nanotime-method nano_year,nanotime-method
##'
setGeneric("nano_wday", function(x, tz) standardGeneric("nano_wday"))
setMethod("nano_wday", c("nanotime"), function(x, tz) nanotime_wday_impl(x, tz))

##' @rdname nano_year
##' @aliases nano_mday
##'
setGeneric("nano_mday", function(x, tz) standardGeneric("nano_mday"))
setMethod("nano_mday", c("nanotime"), function(x, tz) nanotime_mday_impl(x, tz))

##' @rdname nano_year
##' @aliases nano_month
##'
setGeneric("nano_month", function(x, tz) standardGeneric("nano_month"))
setMethod("nano_month", c("nanotime"), function(x, tz) nanotime_month_impl(x, tz))

##' @rdname nano_year
##'
setGeneric("nano_year", function(x, tz) standardGeneric("nano_year"))
setMethod("nano_year", c("nanotime"), function(x, tz) nanotime_year_impl(x, tz))

## rounding ops:

##' Rounding down or up a \code{nanotime} type
##'
##' The functions \code{nano_floor} and \code{nano_ceiling} round down or up, respectively. Although
##' the underlying implementation of \code{nanotime} has negative numbers for values before
##' 1970-01-01 UTC, the rounding is always done backward in time for \code{nano_floor} and forward
##' in time for \code{nano_ceiling}. The functions take a \code{nanotime} argument \code{x} which is
##' the instance to round, together with a second argument \code{precision} which indicates an
##' arbitrary precision to which the rounding should be performed. This argument can be either a
##' \code{nanoduration} or or a \code{nanoperiod}. In the latter case, the argument \code{tz} must
##' also be specified in order to give the \code{nanoperiod} a meaning. Finally, the \code{nanotime}
##' argument \code{origin} can be optionally specified to fix the rounding to a specific point in
##' time.
##'
##' This flexible rounding must be understood in the context of a vector. The rounding precision can
##' then be considered as an interval that defines a grid over which the elements are either
##' assigned to the starting value of the interval to which they belong (\code{nano_floor}) or the
##' ending value of the interval to which they belong (\code{nano_ceiling}). This allows for a
##' grouping of a \code{nanotime} vector on which a statistic may then be run. In the examples
##' below, such a use case is shown in the context of a \code{data.table} object.
##'
##' If "business" concepts such as month or days are needed, the \code{argument} precision must be
##' of type \code{period}. It is then mandatory to specify the timezone argument \code{tz} as this
##' ensures timezone correctness of the intervals including for example for the rare hourly
##' transitions of some countries going from a timezone with a whole hour difference with UTC to one
##' with a fractional hour difference. In the case of a \code{period}, the functions align the
##' rounding if the precision is an integer divisor of a larger quantity. For instance, if one
##' specifies a rounding of 6 hours, a divisor of a day, the hours are aligned on days and the
##' rounding is made to a grid at hours 0, 6, 12 and 18 in the specified timezone. If the precision
##' is not a divisor, the grid is aligned to the nearest hour before the first element of the vector
##' to round.
##'
##' The argument \code{origin} controls the reference point of the rounding, allowing arbitrary
##' specification of the reference point of the rounding.
##'
##' @param x a \code{nanotime} object which must be sorted
##' @param precision a \code{nanoduration} or \code{nanoperiod} object
##'     indicating the rounding precision
##' @param ... for future additional arguments
##' @examples
##' \dontrun{
##' ## "classic" rounding:
##' nano_floor(as.nanotime("2010-10-10 11:12:15 UTC"), as.nanoduration("01:00:00"))
##' ## rounding with arbitrary precision:
##' nano_floor(as.nanotime("2010-10-10 11:12:15 UTC"), as.nanoduration("06:00:00"))
##' nano_floor(as.nanotime("2010-10-10 11:23:15 UTC"), as.nanoduration("00:15:00"))
##' nano_ceiling(as.nanotime("2010-10-10 11:23:15 UTC"), as.nanoduration("01:15:23"))
##' ## controlling the reference point via the 'origin' argument:
##' nano_ceiling(as.nanotime("2010-10-10 11:23:15 UTC"),
##'              as.nanoduration("01:15:23"),
##'              origin=as.nanotime("2010-10-10 11:23:15 UTC"))
##' ## using business concepts and rounding across a daylight saving change:
##' v <- seq(as.nanotime("2020-03-08 America/New_York"),
##'          by=as.nanoperiod("06:00:00"), length.out=8, tz="America/New_York")
##' print(nano_floor(v, as.nanoperiod("1d"), tz="America/New_York"), tz="America/New_York")
##' ## using the concept in a 'data.table':
##' library(data.table)
##' n <- 3 * 24
##' idx <- seq(as.nanotime("2020-03-07 America/New_York"),
##'            by=as.nanoperiod("01:00:00"), length.out=n, tz="America/New_York")
##' dt <- data.table(idx, a=1:n, b=2:(n+1))
##' dt_mean <- dt[, list(mean = mean(a)),
##'               by=nano_ceiling(idx, as.nanoperiod("1d"), tz="America/New_York")]
##' }
##'
##' @aliases nano_floor
##' 
##' @rdname rounding
setGeneric("nano_ceiling", def = function(x, precision, ...) standardGeneric("nano_ceiling"))

##' @rdname rounding
setGeneric("nano_floor",   def = function(x, precision, ...) standardGeneric("nano_floor"))


##' Replicate Elements
##'
##' Replicates the values in 'x' similarly to the default method.
##'
##' @param x a vector of \code{nanotime}
##' @param ... further arguments:
##' 
##'        'times' an integer-valued vector giving the (non-negative)
##'            number of times to repeat each element if of length
##'            'length(x)', or to repeat the whole vector if of length
##'            1. Negative or 'NA' values are an error. A 'double'
##'            vector is accepted, other inputs being coerced to an
##'            integer or double vector.
##' 
##'        'length.out' non-negative integer. The desired length of the
##'            output vector. Other inputs will be coerced to a double
##'            vector and the first element taken. Ignored if 'NA' or
##'            invalid.
##' 
##'        'each' non-negative integer. Each element of 'x' is repeated
##'            'each' times.  Other inputs will be coerced to an integer
##'            or double vector and the first element taken. Treated as
##'            '1' if 'NA' or invalid.
setMethod("rep", c(x = "nanotime"), function(x, ...) {
    new("nanotime", callNextMethod())
})

##' @rdname nanotime
as.character.nanotime <- function(x, ...) {
    format(x, ...)
}

if (getRversion() >= "4.5.0")  {
##' @rdname nanotime
setMethod("unique",
          "nanotime",
          function(x) {
              res <- callNextMethod()
              oldClass(res) <- "integer64"
              new("nanotime", res)
          })
}

## .parseISO8601 . .
## align.time . . . .
## apply.monthly . .
## as.environment.xts
## as.xts . . . . . .
## as.xts.methods . .
## axTicksByTime . .
## CLASS . . . . . .
## - coredata.xts . . .
## diff.xts
## dimnames.xts . . .
## endpoints . . . .
## first . . . . . .
## firstof . . . . .
## indexClass . . . .
## indexTZ
## - index
## isOrdered . . . .
## make.index.unique
## merge.xts . . . .
## na.locf.xts . . .
## ndays . . . . . .
## period.apply . . .
## period.max . . . .
## period.min . . . .
## period.prod . . .
## period.sum . . . .
## periodicity . . .
## plot.xts . . . . .
## - rbind.xts
## sample_matrix . .
## split.xts . . . .
## timeBased . . . .
## timeBasedSeq . . .
## to.period . . . .
## xts . . . . . . .
## xtsAPI . . . . . .
## xtsAttributes . .
## xtsInternals . . .
## [.xts . . . . . .

exit_file("Skip xts tests for now")
library(nanotime)

## check for hypothetical future version with nanotime support
.runTheseTests <- requireNamespace("xts", quietly=TRUE) && packageVersion("xts") >= "0.12.0"
if (!.runTheseTests) exit_file("Skip this xts version")

suppressMessages(library(xts))
i1 <- nanotime(Sys.time()) + as.integer64(1:10)
a1 <- matrix(1:30, 10, 3)
x1 <- xts(a1, i1)
expect_equal(coredata(x1), a1)

#test_coredata.xts <- function() {
i1 <- nanotime(Sys.time()) + as.integer64(1:10)
a1 <- matrix(1:30, 10, 3)
x1 <- xts(a1, i1)
expect_equal(coredata(x1), a1)

#test_index.xts <- function() {
i1 <- nanotime(Sys.time()) + as.integer64(1:10)
a1 <- matrix(1:30, 10, 3)
x1 <- xts(a1, i1)
## don't check attributes, as xts adds 'tzone' and 'tclass'
expect_equal(index(x1), i1, check.attributes = FALSE)

#test_rbind.xts <- function() {
i1 <- nanotime(Sys.time()) + as.integer64(1:10)
i2 <- nanotime(Sys.time()) + as.integer64(11:20)
x1 <- xts(matrix(1:30, 10, 3), i1)
x2 <- xts(matrix(1:30, 10, 3), i2)
exp <- xts(rbind(matrix(1:30, 10, 3), matrix(1:30, 10, 3)), nanotime(c(i1,i2)))
expect_equal(rbind(x1, x2), exp)

#test_c.xts <- function() {
i1 <- nanotime(Sys.time())+ as.integer64(1:10)
i2 <- nanotime(Sys.time())+ as.integer64(11:20)
x1 <- xts(matrix(1:30, 10, 3), i1)
x2 <- xts(matrix(1:30, 10, 3), i2)
exp <- xts(rbind(matrix(1:30, 10, 3), matrix(1:30, 10, 3)), nanotime(c(i1,i2)))
expect_equal(c(x1, x2), exp)

#test_cbind.xts <- function() {
i1 <- nanotime(Sys.time())+ as.integer64(1:10)
i2 <- nanotime(Sys.time())+ as.integer64(11:20)
dimnames1 <-list(NULL, c("a1","b1","c1"))
dimnames2 <-list(NULL, c("a2","b2","c2"))
a1 <- matrix(1:30, 10, 3, dimnames=dimnames1)
a2 <- matrix(1:30, 10, 3, dimnames=dimnames2)
x1 <- xts(a1, i1)
x2 <- xts(a2, i2)

exp <- xts(rbind(cbind(a1, matrix(NA, 10, 3, dimnames=dimnames2)),
                 cbind(matrix(NA, 10, 3, dimnames=dimnames1), a2)),
           nanotime(c(i1,i2)))
expect_equal(cbind(x1, x2), exp)

    ## test_plot.xts <- function()
    ## {
    ##   ## plot doesn't work properly
    ##   data(sample_matrix)
    ##   sample.xts <- as.xts(sample_matrix)
    ##   x <- xts(coredata(sample.xts), nanotime(index(sample.xts)))
    ##   plot(x[,"Close"])
    ## }
#test_dimnames.xts <- function() {
dimnames1 <-list(NULL, c("a1","b1","c1"))
i1 <- nanotime(Sys.time())+ as.integer64(1:10)
a1 <- matrix(1:30, 10, 3, dimnames=dimnames1)
x1 <- xts(a1, i1)
expect_equal(dimnames(x1), dimnames1)

#test_align.time <- function() {
i1 <- nanotime(Sys.time()) + as.integer64(1e9*1:200)
a1 <- matrix(1:600, 200, 3)
x1 <- xts(a1, i1)
xx1 <- xts(a1, as.POSIXct(i1))
expect_equal(align.time(x1, n=as.integer64(60*1e9)), a1)

#test_endpoints.xts <- function() {
data(sample_matrix)
sample.xts <- as.xts(sample_matrix)
x <- xts(coredata(sample.xts), nanotime(index(sample.xts)))
expect_equal(endpoints(x), c(0, 30, 58, 89, 119, 150, 180))

suppressMessages({
    library(bit64)
    library(nanotime)
})
options(digits=7)                       # needed for error message of 0.3333333 below

expect_equal_numeric <- function(x,y,...) expect_equal(as.numeric(x), as.numeric(y), ...)

isArm64 <- Sys.info()[["machine"]] == "arm64"

## nanotime constructors
##test_nanotime_generic <- function() {
expect_identical(S3Part(nanotime(1), strict=TRUE), as.integer64(1))
expect_identical(nanotime(1), new("nanotime", as.integer64(1)))
expect_identical(nanotime(as.integer64(1)), new("nanotime", as.integer64(1)))
expect_identical(as.nanotime(1), new("nanotime", as.integer64(1)))
expect_identical(as.nanotime(as.integer64(1)), new("nanotime", as.integer64(1)))

##test_nanotime_character_first_pass <- function() {
## we do a first pass parsing, which is faster than the parsing
## with the format string, and is also more forgiving
expect_identical(nanotime("2018-01-01T05:00:00.99 Europe/London"),             nanotime(as.integer64("1514782800990000000")))
expect_identical(nanotime("2018-01-01T05:00:00.999_999_999 America/New_York"), nanotime(as.integer64("1514800800999999999")))
expect_identical(nanotime("2018-01-01T05:00:00.999_999 America/New_York"),     nanotime(as.integer64("1514800800999999000")))
expect_identical(nanotime("2018-01-01T05:00:00.999 America/New_York"),         nanotime(as.integer64("1514800800999000000")))
expect_identical(nanotime("2018-01-01T05:00:00 America/New_York"),             nanotime(as.integer64("1514800800000000000")))
expect_identical(nanotime("2018-01-01 05:00:00 America/New_York"),             nanotime(as.integer64("1514800800000000000")))
expect_identical(nanotime("2018-01-01 05:00:00 America/New_York"),             nanotime(as.integer64("1514800800000000000")))
expect_identical(nanotime("2018-01-01 America/New_York"),                      nanotime(as.integer64("1514782800000000000")))

expect_identical(nanotime("2018-01-01T05:00:00.99+00:00"),                     nanotime(as.integer64("1514782800990000000")))
expect_identical(nanotime("2018-01-01T05:00:00.99-00:00"),                     nanotime(as.integer64("1514782800990000000")))
expect_identical(nanotime("2018-01-01T05:00:00.999_999_999+05:00"),            nanotime(as.integer64("1514764800999999999")))
expect_identical(nanotime("2018-01-01T05:00:00.999_999+05:00"),                nanotime(as.integer64("1514764800999999000")))
expect_identical(nanotime("2018-01-01T05:00:00.999+05:00"),                    nanotime(as.integer64("1514764800999000000")))
expect_identical(nanotime("2018-01-01T05:00:00+05:00"),                        nanotime(as.integer64("1514764800000000000")))
expect_identical(nanotime("2018-01-01 05:00:00+05:00"),                        nanotime(as.integer64("1514764800000000000")))
expect_identical(nanotime("2018-01-01 05:00:00+05:00"),                        nanotime(as.integer64("1514764800000000000")))
expect_identical(nanotime("2018-01-01   05:00:00+05:00"),                      nanotime(as.integer64("1514764800000000000")))
expect_identical(nanotime("2018-01-01+05:00"),                                 nanotime(as.integer64("1514746800000000000")))

expect_identical(nanotime("2018/01/01T05:00:00.99 Europe/London"),             nanotime(as.integer64("1514782800990000000")))
expect_identical(nanotime("2018 01 01T05:00:00.99 Europe/London"),             nanotime(as.integer64("1514782800990000000")))

## test the alias 'as.nanotime':
expect_identical(as.nanotime("2018-01-01T05:00:00.99 Europe/London"),             nanotime(as.integer64("1514782800990000000")))
expect_identical(as.nanotime("2018-01-01T05:00:00.999_999_999 America/New_York"), nanotime(as.integer64("1514800800999999999")))
expect_identical(as.nanotime("2018-01-01T05:00:00.999_999 America/New_York"),     nanotime(as.integer64("1514800800999999000")))
expect_identical(as.nanotime("2018-01-01T05:00:00.999 America/New_York"),         nanotime(as.integer64("1514800800999000000")))
expect_identical(as.nanotime("2018-01-01T05:00:00 America/New_York"),             nanotime(as.integer64("1514800800000000000")))
expect_identical(as.nanotime("2018-01-01 05:00:00 America/New_York"),             nanotime(as.integer64("1514800800000000000")))
expect_identical(as.nanotime("2018-01-01 05:00:00 America/New_York"),             nanotime(as.integer64("1514800800000000000")))
expect_identical(as.nanotime("2018-01-01 America/New_York"),                      nanotime(as.integer64("1514782800000000000")))

expect_identical(as.nanotime("2018-01-01T05:00:00.99+00:00"),                     nanotime(as.integer64("1514782800990000000")))
expect_identical(as.nanotime("2018-01-01T05:00:00.99-00:00"),                     nanotime(as.integer64("1514782800990000000")))
expect_identical(as.nanotime("2018-01-01T05:00:00.999_999_999+05:00"),            nanotime(as.integer64("1514764800999999999")))
expect_identical(as.nanotime("2018-01-01T05:00:00.999_999+05:00"),                nanotime(as.integer64("1514764800999999000")))
expect_identical(as.nanotime("2018-01-01T05:00:00.999+05:00"),                    nanotime(as.integer64("1514764800999000000")))
expect_identical(as.nanotime("2018-01-01T05:00:00+05:00"),                        nanotime(as.integer64("1514764800000000000")))
expect_identical(as.nanotime("2018-01-01 05:00:00+05:00"),                        nanotime(as.integer64("1514764800000000000")))
expect_identical(as.nanotime("2018-01-01 05:00:00+05:00"),                        nanotime(as.integer64("1514764800000000000")))
expect_identical(as.nanotime("2018-01-01   05:00:00+05:00"),                      nanotime(as.integer64("1514764800000000000")))
expect_identical(as.nanotime("2018-01-01+05:00"),                                 nanotime(as.integer64("1514746800000000000")))

expect_identical(as.nanotime("2018/01/01T05:00:00.99 Europe/London"),             nanotime(as.integer64("1514782800990000000")))
expect_identical(as.nanotime("2018 01 01T05:00:00.99 Europe/London"),             nanotime(as.integer64("1514782800990000000")))

expect_identical(as.nanotime(NULL), nanotime())
expect_identical(as.nanotime(NULL), as.nanotime())

##test_nanotime_character_first_pass_fail <- function() {
## none of these should parse
expect_error(nanotime("2018-01-01T05:00:00.99 America/New_dYork"),  "Cannot retrieve timezone")
expect_error(nanotime("2018--01-01T05:00:00.99 America/New_York"),  "Parse error")
expect_error(nanotime("2018-01-01T05:00:00.99 America/New_York s"), "Parse error")
expect_error(nanotime("2018-01-01T05:00:s0.99 America/New_York"),   "Parse error")
expect_error(nanotime("2018-01-01T05:00:00.a99 America/New_York"), "Parse error")
expect_error(nanotime("2018-01-01T05:00:00.0000000000 America/New_York"), "Parse error")
expect_error(nanotime("201"), "Parse error")
expect_error(nanotime("2018-13-01T05:00:00.99 Europe/London"), "Parse error")
expect_error(nanotime("2018"), "Parse error")
expect_error(nanotime("2018-1"), "Parse error")
expect_error(nanotime("2018-01-32T05:00:00.99 Europe/London"), "Parse error")
expect_error(nanotime("2018-01-01T25:00:00.99 Europe/London"), "Parse error")
expect_error(nanotime("2018-01-01T05:61:00.99 Europe/London"), "Parse error")
expect_error(nanotime("2018-01-01T05:00:61.99 Europe/London"), "Parse error")
expect_error(nanotime("2018-01-01T05:00:00.99999999999 Europe/London"), "Parse error")
expect_error(nanotime("2018-01-01T05:00:00.99 Europe/London%"), "Parse error")
expect_error(nanotime("2018-01-01T05:00:00.99 %"), "Parse error")

expect_error(as.nanotime("2013-01-01 00:00:00 America/New_York", tz="Europe/London"),
             "timezone is specified twice: in the string and as an argument")

##test_nanotime_character_second_pass  <- function() {
## if the parsing above has failed, then we use a second parsing
## that is based on the format string that is provided
oldFormat <- getOption("nanotimeFormat")
oldTz <- getOption("nanotimeTz")
## check that the format and time zone is picked up from the global options:
options(nanotimeFormat="%Y|%m|%d %H:%M")
options(nanotimeTz="America/New_York")
expect_identical(nanotime("1970|01|01 00:00"), nanotime(18000000000000))
## check they are overridden by the parameters:
expect_identical(nanotime("1970|01|01 00:00:01", format="%Y|%m|%d %H:%M:%S", tz="Europe/Paris"),
                 nanotime(-3599000000000))
options(nanotimeFormat=oldFormat)
options(nanotimeTz=oldTz)

##test_nanotime_character_second_pass_fail  <- function() {
oldFormat <- getOption("nanotimeFormat")
oldTz <- getOption("nanotimeTz")
## check that the format and time zone is picked up from the global options:
options(nanotimeFormat="%Y|%m|%d %H:%M")
options(nanotimeTz="America/New_York")
expect_error(nanotime("1970-01-01 00:00"), "Parse error on 1970-01-01 00:00")
## check they are overridden by the parameters:
expect_error(nanotime("1970-01-01 00:00", format="%Y|%m|%d %H:%M:%S", tz="Europe/Paris"), "Parse error")
options(nanotimeFormat=oldFormat)
options(nanotimeTz=oldTz)


##test_nanotime_matrix <- function() {
m <- matrix(c(10*24*3600+0:9, 987654321+0:9), 10, 2)
expect_identical(nanotime.matrix(m),
                 nanotime("1970-01-11T00:00:00.987654321+00:00") + seq(0, 9e9, by=1e9) + 0:9)

##test_nanotime_POSIXct <- function() {
p <- as.POSIXct("1970-01-01 00:00:00", tz="America/New_York", accurate=FALSE)
expect_identical(nanotime(p), nanotime("1970-01-01T00:00:00.000000000-05:00"))
expect_identical(as.nanotime(p), nanotime("1970-01-01T00:00:00.000000000-05:00"))

## See: https://github.com/eddelbuettel/nanotime/issues/115
## This one should break if multiplying the double by 1E9 directly
## without considering the integer and fraction parts separately.
p <- as.POSIXct(as.numeric('0x1.91f18e4d0065p+30'), origin = '1970-01-01')
if (!isArm64) expect_identical(p, as.POSIXct(as.nanotime(p)))

## This one is negative, making sure that negative doubles are also consistent.
p <- as.POSIXct(as.numeric('-0x1.c6e8c4d077ae4p+30'), origin = '1970-01-01')
if (!isArm64) expect_identical(p, as.POSIXct(as.nanotime(p)))

## with the 'accurate' parameter set to FALSE, the round trip should not be equal:
p <- as.POSIXct(as.numeric('0x1.91f18e4d0065p+30'), origin = '1970-01-01')
expect_false(p == as.POSIXct(as.nanotime(p, accurate=FALSE)))

##test_nanotime_POSIXlt <- function() {
l <- as.POSIXlt("1970-01-01 00:00:00", tz="America/New_York")
expect_identical(nanotime(l), nanotime("1970-01-01T00:00:00.000000000-05:00"))
expect_identical(as.nanotime(l), nanotime("1970-01-01T00:00:00.000000000-05:00"))

##test_nanotime_Date <- function() {
d <- as.Date(10, origin="1970-01-01")
expect_identical(nanotime(d), nanotime("1970-01-11T00:00:00.000000000-00:00"))
expect_identical(as.nanotime(d), nanotime("1970-01-11T00:00:00.000000000-00:00"))

##test_nanotime_numeric_keep_names <- function() {
n <- nanotime(c(a=1, b=2))
expect_identical(names(n), c("a","b"))

##test_nanotime_character_keep_names <- function() {
n <- nanotime(c(a="1970-01-01T00:00:00.000000001+00:00",
                b="1970-01-01T00:00:00.000000001+00:00"))
expect_identical(names(n), c("a","b"))

##test_nanotime_POSIXct_keep_names <- function() {
p <- as.POSIXct("1970-01-01 00:00:00", tz="America/New_York")
n <- nanotime(c(a=p, b=p))
expect_identical(names(n), c("a","b"))

##test_nanotime_POSIXlt_keep_names <- function() {
l <- as.POSIXlt("1970-01-01 00:00:00", tz="America/New_York")
n <- nanotime(c(a=l, b=l))
expect_identical(names(n), c("a","b"))


## format, and as.character
##test_format_default <- function() {
oldFormat <- getOption("nanotimeFormat")
oldTz <- getOption("nanotimeTz")
options(nanotimeFormat=NULL)
options(nanotimeTz=NULL)
expect_identical(format(nanotime("1970-01-01T00:00:00.000000000+00:00")),
                 "1970-01-01T00:00:00+00:00")
expect_identical(format(nanotime("1680-07-17T00:00:01.000000000+00:00")),
                 "1680-07-17T00:00:01+00:00")
expect_identical(format(nanotime("2120-01-01T00:00:59.987654321+00:00")),
                 "2120-01-01T00:00:59.987654321+00:00")
expect_identical(format(nanotime(NULL)), "nanotime(0)")
expect_identical(as.character(nanotime("1970-01-01T00:00:00.000000000+00:00")),
                 "1970-01-01T00:00:00+00:00")
expect_identical(as.character(nanotime("1680-07-17T00:00:01.000000000+00:00")),
                 "1680-07-17T00:00:01+00:00")
expect_identical(as.character(nanotime("2120-01-01T00:00:59.987654321+00:00")),
                 "2120-01-01T00:00:59.987654321+00:00")
expect_identical(as.character(format(nanotime(NULL))), "nanotime(0)")
options(nanotimeFormat=oldFormat)
options(nanotimeTz=oldTz)

##test_format_tz <- function() {
oldFormat <- getOption("nanotimeFormat")
oldTz <- getOption("nanotimeTz")
options(nanotimeFormat=NULL)

a_utc = "1970-01-01T00:00:00+00:00"
nt <- nanotime(a_utc)
a_ny <- "1969-12-31T19:00:00-05:00"

options(nanotimeTz=NULL)
expect_identical(format(nt, tz="America/New_York"), a_ny)

options(nanotimeTz="UTC")
expect_identical(format(nt, tz="America/New_York"), a_ny)

attr(nt, "tzone") <- "UTC"
expect_identical(format(nt, tz="America/New_York"), a_ny)
expect_identical(format(nt, tz=""), a_utc)

options(nanotimeFormat=oldFormat)
options(nanotimeTz=oldTz)

##test_format_tzone <- function() {
oldFormat <- getOption("nanotimeFormat")
oldTz <- getOption("nanotimeTz")
options(nanotimeFormat=NULL)

a <- nanotime("1970-01-01T00:00:00.001000000+00:00")
attr(a, "tzone") <- "America/New_York"
a_ny <- "1969-12-31T19:00:00.001-05:00"

expect_identical(format(a), a_ny)

options(nanotimeTz="UTC")
expect_identical(format(a), a_ny)

options(nanotimeFormat=oldFormat)
options(nanotimeTz=oldTz)

##test_format_tz_from_options <- function() {
oldFormat <- getOption("nanotimeFormat")
oldTz <- getOption("nanotimeTz")
options(nanotimeFormat=NULL)
options(nanotimeTz="America/New_York")

a <- nanotime("1970-01-01T00:00:00.000001000+00:00")
a_ny <- "1969-12-31T19:00:00.000001-05:00"

expect_identical(format(a), a_ny)

options(nanotimeFormat=oldFormat)
options(nanotimeTz=oldTz)

##test_format_fmt_default <- function() {
oldFormat <- getOption("nanotimeFormat")
oldTz <- getOption("nanotimeTz")
options(nanotimeFormat=NULL)
options(nanotimeTz=NULL)

a_str <- "1970-01-01T00:00:00+00:00"
a <- nanotime(a_str)

expect_identical(format(a), a_str)

options(nanotimeFormat=oldFormat)
options(nanotimeTz=oldTz)

##test_format_fmt_from_options <- function() {
oldFormat <- getOption("nanotimeFormat")
oldTz <- getOption("nanotimeTz")
options(nanotimeFormat="%Y/%m/%dT%H:%M:%E9S%Ez")
options(nanotimeTz="America/New_York")

a <- nanotime("1970/01/01T00:00:00.000000000+00:00")
a_ny <- "1969/12/31T19:00:00.000000000-05:00"

expect_identical(format(a), a_ny)

options(nanotimeFormat=oldFormat)
options(nanotimeTz=oldTz)

##test_format_fmt_from_parameter <- function() {
oldFormat <- getOption("nanotimeFormat")
oldTz <- getOption("nanotimeTz")
options(nanotimeFormat="%Y/%m/%dT%H:%M:%E9S%Ez")
options(nanotimeTz="America/New_York")

a <- nanotime("1970-01-01 00:00:00.000000000+00:00", format="%Y-%m-%d %H:%M:%E9S%Ez")
                                        # the result of format on a is taken from the global option:
a_ny <- "1969/12/31T19:00:00.000000000-05:00"

expect_identical(format(a), a_ny)

options(nanotimeFormat=oldFormat)
options(nanotimeTz=oldTz)

##test_format_na <- function() {
if (FALSE) {
    f <- format(nanotime(c(1, NA, 2, NaN)))
    exp <- c("1970-01-01T00:00:00.000000001+00:00",
             as.character(NA),
             "1970-01-01T00:00:00.000000002+00:00",
             as.character(NA))
    expect_identical(f, exp)
}


## conversions
##test_as_POSIXct <- function() {
a <- nanotime(0)
attr(a, "tzone") <- "America/New_York"
p <- as.POSIXct(a)
expect_identical(p, as.POSIXct("1969-12-31 19:00:00", tz="America/New_York"))

oldTz <- getOption("nanotimeTz")
options(nanotimeTz="America/New_York")
b <- nanotime(0)
p <- as.POSIXct(b)
## LLL: not fully satisfactory here; nanotime will not have 'tz' set from the environment:
expect_equal(p, as.POSIXct("1969-12-31 19:00:00", tz="America/New_York"), check.tzone=FALSE)

options(nanotimeTz=NULL)
c <- nanotime(0)
p <- as.POSIXct(c)
expect_equal(p, as.POSIXct("1970-01-01 00:00:00", tz="UTC"), check.tzone=FALSE)

options(nanotimeTz=oldTz)

##test_as_Date <- function() {
## test scalar
nanotime_scalar <- nanotime("2023-09-21T22:00:00 America/New_York")
expect_identical(as.Date(nanotime_scalar, tz="UTC"), as.Date("2023-09-22"))
## test vector
nanotime_vec <- rep(nanotime("2023-09-21T22:00:00 America/New_York"), 2)
tz_vec <- c("UTC", "America/New_York")
expect_identical(as.Date(nanotime_vec, tz=tz_vec), as.Date(c("2023-09-22", "2023-09-21")))
## test missing argument 'tz': now UTC filled in
expect_silent(as.Date(nanotime_vec))
expect_identical(as.Date(nanotime_vec), as.Date(nanotime_vec, tz="UTC"))
## test exception for extraneous argument:
expect_error(as.Date(nanotime_vec, y=2, z=3), "'as.Date' called with arguments other than 'tz':  'y', 'z'")
expect_silent(as.Date(x=nanotime_vec))  # check we can still name 'x' without an exception


## c, subset, subassign and binds
##test_c <- function() {
a <- c(nanotime(1), nanotime(2))
expect_identical(a, nanotime(1:2))

a <- c(nanotime(1:2), nanotime(3:4))
expect_identical(a, nanotime(1:4))

## subset

## subset integer
a <- nanotime(1:10)
expect_identical(a[3], nanotime(3))
expect_identical(a[1:10], a)
expect_identical(a[c(1:3, NA)], c(nanotime(1:3), NA_nanotime_))
expect_identical(a[11], NA_nanotime_)
expect_identical(a[c(0, 2)], as.nanotime(2))
expect_warning(a[1, 2], "unused indices or arguments in 'nanotime' subsetting")
expect_error(a[c(1,-2)], "only 0's may be mixed with negative subscripts")
expect_error(a[c(-1, 2)], "only 0's may be mixed with negative subscripts")
    
names(a) <- paste0("a", 1:10)
expect_identical(a[c(1,10)], c(a1=as.nanotime(1), a10=as.nanotime(10)))
res <- c(a1=as.nanotime(1), a10=as.nanotime(10), NA)
names(res)[3] <- NA_character_
expect_identical(a[c(1,10,NA)], res)
expect_identical(a[11], res[3])

## subset logical
a <- nanotime(1:10)
expect_identical(a[c(T,F)], nanotime(c(1,3,5,7,9)))
expect_identical(a[c(T,F,F,F,NA)], nanotime(c(1, NA, 6, NA)))
expect_identical(a[c(1:3, NA)], c(nanotime(1:3), NA_nanotime_))
expect_warning(a[T, F], "unused indices or arguments in 'nanotime' subsetting")

names(a) <- paste0("a", 1:10)
expect_identical(a[c(T,F,F,F,F)], a[c(1,6)])
res <- c(a1=as.nanotime(1), NA)
names(res)[2] <- NA_character_
expect_identical(a[c(T,F,F,F,F,F,F,F,F,NA)], res)

## subset named character
a <- nanotime(1:10)
names(a)  <- paste0("a", 1:10)
expect_identical(a[c("a1", "a3", "a6")], a[c(1,3,6)])
expect_identical(a[c("a1", "a3", NA)], a[c(1,3,NA)])
expect_identical(a[c("a1", "a3", "a11")], a[c(1,3,NA)])
expect_warning(a["a", "b"], "unused indices or arguments in 'nanotime' subsetting")



## subset error
a  <- nanotime(1:10)
expect_error(a[as.integer64(1)], "']' not defined on 'nanotime' for index of type 'ANY'")

##test_subsassign <- function() {
a <- nanotime(1:10)
a[3] <- nanotime(13)
expect_identical(a[3], nanotime(13))
a[1:10] <- nanotime(10:1)
expect_identical(a[1:10], nanotime(10:1))


## summary
##test_summary <- function() {
expected <- nanotime(c(1,2,2,2,3,4))
names(expected) <- c("Min.", "1st Qu.", "Median", "Mean", "3rd Qu.", "Max.")
expect_identical(summary(nanotime(1:4)), expected)

expected <- nanotime(c(1,2,3,3,4,5))
names(expected) <- c("Min.", "1st Qu.", "Median", "Mean", "3rd Qu.", "Max.")
expect_identical(summary(nanotime(1:5)), expected)


## Arith
##test_unary_minus <- function() {
expect_error(-nanotime(1), "unary '-' is not defined for \\\"nanotime\\\" objects")

##test_unary_plus <- function() {
expect_error(+nanotime(1), "unary '\\+' is not defined for \\\"nanotime\\\" objects")

##test_ANY_minus_nano <- function() {
expect_error(1 - nanotime(2), "invalid operand types")

##test_nano_minus_nanoperiod <- function() {
msg <- paste0("binary '\\-' is not defined for 'nanotime' and 'nanoperiod' objects; instead use 'minus\\(e1, e2, tz\\)'")
expect_error(nanotime(1) - as.nanoperiod("2m"), msg) 

##test_nano_minus_ANY <- function() {
expect_error(nanotime(1) - "a", "invalid operand types")

##test_nano_minus_nanoduration <- function() {
nt <- nanotime(2000)
expect_identical(nt - as.nanoduration(2000), nanotime(0))

## +
##test_ANY_plus_nano <- function() {
expect_error("a" + nanotime(2), "invalid operand types")

##test_nano_plus_ANY <- function() {
expect_error(nanotime(1) + "a", "invalid operand types")

##test_nano_plus_nanoperiod <- function() {
msg <- paste0("binary '\\+' is not defined for 'nanotime' and 'nanoperiod' objects; instead use 'plus\\(e1, e2, tz\\)'")
expect_error(nanotime(1) + as.nanoperiod("2m"), msg) 

##test_nano_plus_nanoduration <- function() {
nt <- nanotime(2000)
expect_identical(nt + as.nanoduration(2000), nanotime(4000))

## *
##test_nano_mul_nano <- function() {
expect_error(nanotime(1) * nanotime(1), "operation not defined for \"nanotime\" objects")

##test_nano_mul_nano <- function() {
expect_error(nanotime(1) * nanotime(2), "operation not defined for \"nanotime\" objects")

##test_nano_mul_ANY <- function() {
expect_error(nanotime(1) * 1, "operation not defined for \"nanotime\" objects")

##test_nano_div_ANY <- function() {
## div is in the same 'Arith' group as mul:
expect_error(nanotime(1) / 1, "operation not defined for \"nanotime\" objects")

##test_nano_Logic_ANY <- function() {
expect_error(nanotime(1) | 1,
             "operations are possible only for numeric, logical or complex types")

##test_ANY_Logic_nano <- function() {
expect_error(1 | nanotime(1),
             "operations are possible only for numeric, logical or complex types")

##test_Math_nano <- function() {
expect_error(sin(nanotime(1)),
             "non-numeric argument to mathematical function")

##test_Math2_nano <- function() {
expect_error(round(nanotime(1)),
             "non-numeric argument to mathematical function")


## miscellaneous
##test_is.na <- function() {
expect_true(is.na(nanotime(NA)))
expect_true(is.na(nanotime(NaN)))
nn <- nanotime(1:10)
is.na(nn) <- 1:3
expect_true(all(is.na(nn[1:3])))
expect_true(!any(is.na(nn[4:10])))
expect_true(is.na(NA_nanotime_))


## test square bracket (#44)
##test_square_bracket <- function() {
times <- c(nanotime('2011-12-05 08:30:00.000',format ="%Y-%m-%d %H:%M:%E9S",  tz ="GMT"),
           nanotime('2011-12-05 08:30:00.100',format ="%Y-%m-%d %H:%M:%E9S",  tz ="GMT"),
           nanotime('2011-12-05 08:30:00.825',format ="%Y-%m-%d %H:%M:%E9S",  tz ="GMT"))
expect_identical(times[1], times[[1]])
expect_identical(times[2], times[[2]])
expect_identical(times[3], times[[3]])

## test compare
expect_true(nanotime(1) == nanotime(1))
expect_true(nanotime(1) != nanotime(2))
expect_true(nanotime(1) <= nanotime(1))
expect_true(nanotime(1) >= nanotime(1))
expect_true(nanotime(1) <= nanotime(2))
expect_false(nanotime(1) >= nanotime(2))
expect_true(nanotime(1) < nanotime(2))
expect_false(nanotime(1) > nanotime(2))

## with character
expect_true(nanotime(1) == nanotime("1970-01-01T00:00:00.000000001+00"))
expect_true(nanotime("1970-01-01T00:00:00.000000001+00") != nanotime(2))
expect_true(nanotime("1970-01-01T00:00:00.000000001+00") <= nanotime(1))
expect_true(nanotime("1970-01-01T00:00:00.000000001+00") >= nanotime(1))
expect_true(nanotime(1) <= nanotime("1970-01-01T00:00:00.000000002+00"))
expect_false(nanotime("1970-01-01T00:00:00.000000001+00") >= nanotime(2))
expect_true(nanotime(1) < nanotime("1970-01-01T00:00:00.000000002+00"))
expect_false(nanotime("1970-01-01T00:00:00.000000001+00") > nanotime(2))

## with POSIXt
pt1 <- as.POSIXct(1, origin="1970-01-01", tz="UTC")
pt2 <- as.POSIXct(2, origin="1970-01-01", tz="UTC")
expect_true(pt1 == nanotime("1970-01-01T00:00:01+00"))
expect_true(nanotime("1970-01-01T00:00:01+00") != pt2)
expect_true(nanotime("1970-01-01T00:00:01+00") <= pt1)
expect_true(nanotime("1970-01-01T00:00:01+00") >= pt1)
expect_true(nanotime(1) <= nanotime("1970-01-01T00:00:02+00"))
expect_false(nanotime("1970-01-01T00:00:01+00") >= pt2)
expect_true(nanotime(1) < nanotime("1970-01-01T00:00:00.000000002+00"))
expect_false(nanotime("1970-01-01T00:00:01+00") > pt2)


## all.equal
##test_all.equal_nanotime_any <- function() {
expect_true(!isTRUE(all.equal(nanotime(1), "a")))

##test_all.equal_any_nanotime <- function() {
expect_true(!isTRUE(all.equal("a", nanotime(1))))

##test_all.equal_nanotime_nanotime <- function() {
expect_true(isTRUE(all.equal(nanotime(1), nanotime(1))))
expect_true(!isTRUE(all.equal(nanotime(1), nanotime(2))))  
expect_identical(all.equal(nanotime(1), nanotime(2)), "Mean relative difference: 1")  
expect_identical(all.equal(nanotime(c(1L,NA)), nanotime(1:2)), "'is.NA' value mismatch: 0 in current 1 in target")

expect_error(all.equal(nanotime(1), nanotime(1), tolerance="1"), "'tolerance' should be numeric")
expect_error(all.equal(nanotime(1), nanotime(1), scale="1"), "'scale' should be numeric or NULL")
expect_error(all.equal(nanotime(1), nanotime(1), check.attributes="1"), "'check.attributes' must be logical")
expect_error(all.equal(nanotime(1), nanotime(2), scale=-1), "all\\(scale > 0\\) is not TRUE")

expect_false(isTRUE(all.equal(nanotime(1), as.nanoperiod("1d"))))
expect_identical(all.equal(nanotime(1), nanotime(1:2)), "Numeric: lengths (1, 2) differ")
expect_identical(all.equal(nanotime(c(1,2,3)), nanotime(c(1,1,2)), countEQ=TRUE), "Mean relative difference: 0.3333333")
expect_false(isTRUE(all.equal(nanotime(1), 1i)))
expect_identical(all.equal(nanotime(1), nanotime(3), tolerance=1), "Mean absolute difference: 2")
expect_identical(all.equal(nanotime(1), nanotime(2e9), scale=1e9), "Mean scaled difference: 2")
expect_identical(all.equal(nanotime(1), nanotime(2e9), scale=1), "Mean absolute difference: 2e+09")


## test S4 conversions:

## to 'nanotime':
expect_identical(as("2018-01-01T05:00:00.99 Europe/London", "nanotime"), nanotime(as.integer64("1514782800990000000")))
expect_identical(as(as.Date(10, origin="1970-01-01"), "nanotime"), nanotime("1970-01-11T00:00:00.000000000-00:00"))
expect_identical(as(as.integer64("1514782800990000000"), "nanotime"), nanotime("2018-01-01T05:00:00.99 Europe/London"))
expect_identical(as(as.POSIXct(1514782800, origin="1970-01-01", tz="UTC"), "nanotime"), nanotime("2018-01-01T05:00:00 Europe/London"))
expect_identical(as(as.POSIXlt(as.POSIXct(1514782800, origin="1970-01-01", tz="UTC")), "nanotime"), nanotime("2018-01-01T05:00:00 Europe/London"))

## from 'nanotime':
expect_identical(as(nanotime("1970-01-11T00:00:00.000000000-00:00"), "Date"), as.Date("1970-01-11"))
## don't run this one, it's only right when run in America/New_York timezone!
## expect_identical(as(nanotime("1970-01-11T00:00:00.000000000-00:00"), "POSIXct"), as.POSIXct("1970-01-10 19:00:00"))
expect_identical(as(nanotime("1970-01-11T00:00:00.000000000-00:00"), "integer64"), as.integer64("864000000000000"))
expect_identical(as(nanotime("1970-01-11T00:00:00.000000000-00:00"), "nanoduration"), as.nanoduration("240:00:00"))


## test nanotime sequences:

## straightforward with integer64 and nanoduration1
expect_identical(seq(as.nanotime(1), by=1e9, length.out=10), as.nanotime(seq(1, by=1e9, length.out=10)))
expect_identical(seq(as.nanotime(1), by=1e9, along.with=1:10), as.nanotime(seq(1, by=1e9, length.out=10)))
expect_identical(seq(from=as.nanotime(1), to=as.nanotime(10e9), by=1e9), as.nanotime(seq(1, 10e9, by=1e9)))
expect_identical(seq(from=as.nanotime(1), to=as.nanotime(9e9+1), length.out=10), as.nanotime(seq(1, 10e9, by=1e9)))
expect_identical(seq(as.nanotime(1), by=as.nanoduration(1e9), length.out=10), as.nanotime(seq(1, by=1e9, length.out=10)))
expect_identical(seq(as.nanotime(1), as.nanotime(10e9), by=as.nanoduration(1e9)), as.nanotime(seq(1, 10e9, by=1e9)))

## increment with a period:
expect_identical(seq(as.nanotime(0), by=as.nanoperiod("1m"), length.out=4, tz="UTC"),
                 as.nanotime(c("1970-01-01T00:00:00+00:00", "1970-02-01T00:00:00+00:00",
                               "1970-03-01T00:00:00+00:00", "1970-04-01T00:00:00+00:00")))
expect_identical(seq(as.nanotime(0), by=as.nanoperiod("1m"), length.out=0, tz="UTC"),
                 nanotime())
expect_identical(seq(as.nanotime(0), to=as.nanotime("1970-04-01T00:00:00+00:00"),
                     by=as.nanoperiod("1m"), tz="UTC"),
                 as.nanotime(c("1970-01-01T00:00:00+00:00", "1970-02-01T00:00:00+00:00",
                               "1970-03-01T00:00:00+00:00", "1970-04-01T00:00:00+00:00")))

## other seq errors or warnings:
expect_warning(seq(as.nanotime(1), by=1e9, length.out=10:12), "first element used of 'length.out' argument")
expect_error(seq(as.nanotime(0), by=as.nanoperiod(c("1m","2m")), length.out=4, tz="UTC"),
             "'by' must be of length 1")
expect_error(seq(as.nanotime(0), by=as.nanoperiod("1m"), length.out=-1, tz="UTC"),
             "'length.out' must be a non-negative number")
expect_error(seq(from=as.nanotime(1)), "'seq.nanotime' cannot be called with only one argument")
expect_error(seq(from=as.nanotime(1), to=as.nanotime(2), length.out=as.integer()), "argument 'length.out' must be of length 1")
expect_error(seq(from=as.nanotime(1), length.out=1),
             "at least two of 'to', 'by', and 'length.out' or 'along.with' must be specified")
expect_error(seq(from=as.nanotime(1), to=as.nanoduration(10e9), by=1e9),
             "'to' must be a 'nanotime'")
expect_error(seq(from=as.nanotime(1:2), to=as.nanotime(1), by=1e9),
             "'from' must be of length 1")
expect_error(seq(from=as.nanotime(1), to=as.nanotime(1:2), by=1e9),
             "'to' must be of length 1")          
expect_error(seq(from=as.nanotime(1), to=as.nanotime(2), by=1e9, length.out=1, too_much=2),
             "too many arguments")
expect_error(seq(as.nanotime(1), by=c(1e9,2), length.out=10), "'by' must be of length 1")
expect_error(seq(as.nanotime(1), to=as.nanotime(10), by=c(1e9,2)), "'by' must be of length 1")
expect_error(seq(as.nanotime(1), by=as.nanoperiod("1d"), length.out=10), "'tz' is a mandatory argument for sequences with a 'period' step")
expect_error(seq(as.nanotime(1), to=as.nanotime(10), by=as.nanoperiod("1d")), "'tz' is a mandatory argument for sequences with a 'period' step")

## test date time component access
expect_identical(nano_wday(as.nanotime("2020-03-14 23:32:00-04:00"), "America/New_York"), 6L)
expect_identical(nano_wday(as.nanotime("2020-03-14 23:32:00 America/New_York"), "Europe/Paris"), 0L)
expect_identical(nano_mday(as.nanotime("2020-03-14 23:32:00-04:00"), "America/New_York"), 14L)
expect_identical(nano_mday(as.nanotime("2020-03-14 23:32:00 America/New_York"), "Europe/Paris"), 15L)
expect_identical(nano_month(as.nanotime("2020-12-31 23:32:00-04:00"), "America/New_York"), 12L)
expect_identical(nano_month(as.nanotime("2020-12-31 23:32:00 America/New_York"), "Europe/Paris"), 1L)
expect_identical(nano_year(as.nanotime("2020-12-31 23:32:00-04:00"), "America/New_York"), 2020L)
expect_identical(nano_year(as.nanotime("2020-12-31 23:32:00 America/New_York"), "Europe/Paris"), 2021L)
expect_identical(nano_year(nanotime(1:10), "America/New_York"), rep(1969L, 10))
expect_identical(nano_year(as.nanotime("1916-03-14 12:32:00-04:00"), "America/New_York"), 1916L)

expect_error(nano_wday(as.nanotime("2020-03-14 23:32:00-04:00"), "America/Nu_York"), "Cannot retrieve timezone")
expect_error(nano_mday(as.nanotime("2020-03-14 23:32:00-04:00"), "America/Nu_York"), "Cannot retrieve timezone")
expect_error(nano_month(as.nanotime("2020-03-14 23:32:00-04:00"), "America/Nu_York"), "Cannot retrieve timezone")
expect_error(nano_year(as.nanotime("2020-03-14 23:32:00-04:00"), "America/Nu_York"), "Cannot retrieve timezone")


## 0-length ops:
## ------------

## constructor:
expect_identical(nanotime(character(), character()), nanotime())
expect_identical(nanotime(character(), tz="America/New_York"), nanotime())
expect_identical(nanotime("2020-03-28 UTC", tz=character()), nanotime())

## accessors/funcs
expect_identical(nano_wday(nanotime(), tz=character()), integer())
expect_identical(nano_wday(nanotime(1), tz=character()), integer())
expect_identical(nano_wday(nanotime(), tz="America/New_York"), integer())
expect_identical(nano_mday(nanotime(), tz=character()), integer())
expect_identical(nano_mday(nanotime(1), tz=character()), integer())
expect_identical(nano_mday(nanotime(), tz="America/New_York"), integer())
expect_identical(nano_month(nanotime(), tz=character()), integer())
expect_identical(nano_month(nanotime(1), tz=character()), integer())
expect_identical(nano_month(nanotime(), tz="America/New_York"), integer())
expect_identical(nano_year(nanotime(), tz=character()), integer())
expect_identical(nano_year(nanotime(1), tz=character()), integer())
expect_identical(nano_year(nanotime(), tz="America/New_York"), integer())

## rep
expect_identical(rep(nanotime(1), 2), nanotime(rep(1,2)))
expect_identical(rep(nanotime(1:2), each=2), nanotime(rep(1:2, each=2)))

suppressMessages({
    library(bit64)
    library(nanotime)
})
options(digits=7)                       # needed for error message below

milli <- 1e6
sec <- 1e9
minute <- 60 * sec
hour <- 60 * minute

## constructors
##test_nanoduration <- function() {
expect_identical(nanoduration(1,1,1,1), as.nanoduration("01:01:01.000_000_001"))
expect_identical(nanoduration(-1,0,0,1), as.nanoduration("-00:59:59.999_999_999"))
## R recycling:
expect_identical(nanoduration(1:2,1,1,1), c(as.nanoduration("01:01:01.000_000_001"),
                                        as.nanoduration("02:01:01.000_000_001")))
expect_identical(nanoduration(), as.nanoduration(NULL))
expect_identical(length(nanoduration()), 0L)
expect_identical(nanoduration(), as.nanoduration())


##test_as.nanoduration_character_and_numeric <- function() {
expect_identical(as.nanoduration("1:00:00"), as.nanoduration(hour))
expect_identical(as.nanoduration("-1:00:00"), -as.nanoduration(hour))
expect_identical(as.nanoduration("0:01:00"), as.nanoduration(minute))
expect_identical(as.nanoduration("0:00:01"), as.nanoduration(sec))
expect_identical(as.nanoduration("0:00:00.001"), as.nanoduration(milli))
expect_identical(c(as.nanoduration("1:00:00"), c(as.nanoduration("2:00:00"))),
                 c(as.nanoduration(hour), as.nanoduration(2*hour)))
expect_identical(as.nanoduration(NA_integer64_),
                 as.nanoduration(as.integer64("-9223372036854775808")))
expect_warning(as.integer64.character("-9223372036854775808"))

## check name:
n1 <- as.nanoduration(hour)
names(n1)  <- "a"
expect_identical(as.nanoduration(c(a="1:00:00")), as.nanoduration(n1))

expect_error(as.nanoduration("10:aa"), "cannot parse nanoduration")
expect_error(as.nanoduration("1000a"), "cannot parse nanoduration")
expect_error(as.nanoduration("10:10:10.1000a"), "cannot parse nanoduration")
expect_error(as.nanoduration("a"), "cannot parse nanoduration")
expect_error(as.nanoduration(""), "cannot parse nanoduration")

##test_as.nanoduration_integer64 <- function() {
expect_identical(as.nanoduration(as.integer64(hour)), as.nanoduration(hour))
expect_identical(as.nanoduration(c(a=as.integer64(hour), b=as.integer64(2*hour))),
                 as.nanoduration(c(a=hour, b=2*hour)))

##test_as.nanoduration_integer <- function() {
expect_identical(as.nanoduration(as.integer(sec)), as.nanoduration(sec))
expect_identical(as.nanoduration(c(a=as.integer(milli), b=as.integer(2*milli))),
                 as.nanoduration(c(a=milli, b=2*milli)))

##test_as.integer64 <- function() {
expect_identical(as.integer64(as.nanoduration(hour)), as.integer64(hour))
expect_identical(as.integer64(as.nanoduration(1:1000)), as.integer64(1:1000))



## show/print/format
##test_show <- function() {
d <- nanoduration(1,1,1,1)
expect_identical(format(d), "01:01:01.000_000_001")
expect_identical(format(-d), "-01:01:01.000_000_001")
expect_true(is.na(format(as.nanoduration(NA_integer64_))))
expect_identical(as.character(d), "01:01:01.000_000_001")
expect_identical(as.character(-d), "-01:01:01.000_000_001")
expect_true(is.na(as.character(as.nanoduration(NA_integer64_))))
expect_stdout(show(d))
expect_stdout(show(-d))
expect_stdout(show(as.nanoduration(NA_integer64_)))

##test_print <- function() {
d <- nanoduration(1,1,1,1)
expect_stdout(print(d))
expect_stdout(print(nanoduration()))

##test_print_name <- function() {
d <- nanoduration(1,1,1,1)
names(d) <- "a"
expect_identical(format(d), c(a="01:01:01.000_000_001"))
expect_stdout(print(d))

##test_format <- function() {
d <- nanoduration(1,1,1,1)
expect_identical(format(d), "01:01:01.000_000_001")


## subset:
##test_subset_int <- function() {
dd <- as.nanoduration(1:10)
expect_identical(dd[1:2], as.nanoduration(1:2))
expect_identical(dd[-1:-5], as.nanoduration(6:10))
expect_identical(dd[c(0,0,0,0,0,1,2,3,4,5)], as.nanoduration(1:5))
expect_warning(dd[1, 2], "unused indices or arguments in 'nanoduration' subsetting")

##test_subset_logical <- function() {
dd <- as.nanoduration(1:4)
expect_identical(dd[c(T,F,F,F)], dd[1])
expect_identical(dd[c(F,T,F,F)], dd[2])
expect_identical(dd[c(F,F,T,F)], dd[3])
expect_identical(dd[c(F,F,F,T)], dd[4])
expect_identical(dd[TRUE], dd)
expect_identical(dd[c(F,T,T,F)], dd[2:3])
expect_warning(dd[T, F], "unused indices or arguments in 'nanoduration' subsetting")
expect_identical(dd[c(F,F,F,NA)], NA_nanoduration_)

##test_subset_character <- function() {
dd <- c(x=as.nanoduration(1), y=as.nanoduration(2))
expect_identical(dd["x"], c(x=as.nanoduration(1)))
expect_identical(dd["y"], c(y=as.nanoduration(2)))
expect_warning(dd["x", "y"], "unused indices or arguments in 'nanoduration' subsetting")

## also check out of bound subset
res <- as.nanoduration(as.integer64(NA))
names(res) <- NA
expect_identical(dd["a"], res)
expect_warning(dd["a", "b"], "unused indices or arguments in 'nanoduration' subsetting")

## subset incorrect index type
expect_error(dd[as.integer64(1)], "']' not defined on 'nanoduration' for index of type 'ANY'")

## subassign:
##test_subassign_logical <- function() {
x <- as.nanoduration(1:10)
x[c(T,T,T,F,F,F,F,F,F,F)] <- as.nanoduration(2:4)
expect_identical(x, as.nanoduration(c(2:4, 4:10)))

##test_subassign_numeric <- function() {
x <- as.nanoduration(1:10)
x[1:3] <- as.nanoduration(2:4)
x[4:10] <- as.nanoduration(5:11)
expect_identical(x, as.nanoduration(2:11))

##test_subsassign_character <- function() {
dd <- c(a=as.nanoduration(1), b=as.nanoduration(2), c=as.nanoduration(3), d=as.nanoduration(4))
dd[c("b", "c")] <- as.nanoduration(20:21)
expected <- c(a=as.nanoduration(1), b=as.nanoduration(20), c=as.nanoduration(21), d=as.nanoduration(4))
expect_identical(dd, expected)


##test_square_bracket <- function() {
dd <- c(a=as.nanoduration(1), b=as.nanoduration(2), c=as.nanoduration(3), d=as.nanoduration(4))
dd_nonames <- as.nanoduration(1:4)
expect_identical(dd_nonames[1], dd[[1]])
expect_identical(dd_nonames[2], dd[[2]])
expect_identical(dd_nonames[3], dd[[3]])


## test names
##test_get_names <- function() {
dd <- c(a=as.nanoduration(1), b=as.nanoduration(2), c=as.nanoduration(3), d=as.nanoduration(4))
expect_identical(names(dd), c("a","b","c","d"))

##test_set_names <- function() {
names <- c("a","b","c","d")
dd <- as.nanoduration(1:4)
names(dd) <- names
expect_identical(names(dd), names)
names(dd)[1] <- "x"
expect_identical(names(dd), c("x","b","c","d"))


## test scalar/vector mix LLL

## ops
## -
##test_nanoduration_minus_nanoduration <- function() {
expect_identical(as.nanoduration("1:00:00") - as.nanoduration("00:01:00"), as.nanoduration("00:59:00"))
expect_identical(as.nanoduration("-1:00:00") - as.nanoduration("00:01:00"), as.nanoduration("-01:01:00"))
expect_identical(as.nanoduration("1:00:00.1") - as.nanoduration("1:00:00"), as.nanoduration("0.1"))
expect_identical(as.nanoduration(1:10) - as.nanoduration(0:9), as.nanoduration(rep(1, 10)))
expect_identical(as.nanoduration(1:10) - as.nanoduration(1), as.nanoduration(0:9))
expect_identical(c(a=as.nanoduration(1), b=as.nanoduration(1)) - as.nanoduration(1),
                 as.nanoduration(c(a=0, b=0)))

##test_nanoduration_minus_numeric <- function() {
expect_identical(as.nanoduration("1:00:00") - minute, as.nanoduration("00:59:00"))
expect_identical(as.nanoduration(1:10) - 0.0:9, as.nanoduration(rep(1, 10)))
expect_identical(as.nanoduration(1:10) - 1, as.nanoduration(0:9))
expect_identical(c(a=as.nanoduration(1), b=as.nanoduration(1)) - 1, as.nanoduration(c(a=0, b=0)))

##test_nanoduration_minus_integer <- function() {
expect_identical(as.nanoduration("1:00:00") - as.integer(milli), as.nanoduration("00:59:59.999"))
expect_identical(as.nanoduration(1:10) - 0:9, as.nanoduration(rep(1, 10)))
expect_identical(as.nanoduration(1:10) - as.integer(1), as.nanoduration(0:9))
expect_identical(c(a=as.nanoduration(1), b=as.nanoduration(1)) - as.integer(1),
                 as.nanoduration(c(a=0, b=0)))

##test_nanoduration_minus_integer64 <- function() {
expect_identical(as.nanoduration("1:00:00") - as.integer64(minute), as.nanoduration("00:59:00"))
expect_identical(as.nanoduration(1:10) - as.integer64(0:9), as.nanoduration(rep(1, 10)))
expect_identical(as.nanoduration(1:10) - as.integer64(1), as.nanoduration(0:9))
expect_identical(c(a=as.nanoduration(1), b=as.nanoduration(1)) - c(c=as.integer64(1)),
                 as.nanoduration(c(a=0, b=0)))

##test_integer64_minus_nanoduration <- function() {
expect_identical(as.integer64(hour) - as.nanoduration("1:00:00"), as.nanoduration("0:00:00"))

##test_nanoduration_minus_integer64 <- function() {
expect_identical(as.nanoduration("1:00:00") - as.integer64(hour), as.nanoduration("0:00:00"))

##test_numeric_minus_nanoduration <- function() {
expect_identical(hour - as.nanoduration("1:00:00"), as.nanoduration("0:00:00"))

##test_integer_minus_nanoduration <- function() {
expect_identical(as.integer(1e9) - as.nanoduration("1:00:00"), as.nanoduration("-00:59:59"))

## LLL name issue here due do to underlying issues with 'bit64'
## ##test_numeric_minus_nanoduration <- function() {
##     expect_identical(2 - c(a=as.nanoduration(1), b=as.nanoduration(2)),
##                    c(a=as.nanoduration(1), b=as.nanoduration(0)))
##     expect_identical(2 - c(a=as.integer64(1), b=as.integer64(2)),
##                    c(a=as.integer64(1), b=as.integer64(0)))

## 
##test_any_minus_nanoduration <- function() {
expect_error("a" - as.nanoduration("1:00:00"), "invalid operand types")

##test_nanoduration_minus_any <- function() {
expect_error(as.nanoduration("1:00:00") - "a", "invalid operand types")


## +
##test_nanoduration_plus_nanoduration <- function() {
expect_identical(as.nanoduration("1:00:00") + as.nanoduration("00:01:00"), as.nanoduration("01:01:00"))
expect_identical(as.nanoduration("-1:00:00") + as.nanoduration("00:01:00"), as.nanoduration("-00:59:00"))
expect_identical(as.nanoduration("-1:00:00.1") + as.nanoduration("1:00:00"), as.nanoduration("-0.1"))
expect_identical(-as.nanoduration(1:10) + as.nanoduration(0:9), as.nanoduration(rep(-1, 10)))
expect_identical(as.nanoduration(1:10) + as.nanoduration(1), as.nanoduration(2:11))
expect_identical(c(a=as.nanoduration(1), b=as.nanoduration(1)) + as.nanoduration(1),
                 as.nanoduration(c(a=2, b=2)))

##test_integer64_plus_nanoduration <- function() {
expect_identical(as.integer64(hour) + as.nanoduration("1:00:00"), as.nanoduration("2:00:00"))

##test_nanoduration_plus_integer64 <- function() {
expect_identical(as.nanoduration("1:00:00") + as.integer64(hour), as.nanoduration("2:00:00"))

##test_nanoduration_plus_numeric <- function() {
expect_identical(as.nanoduration("1:00:00") + hour, as.nanoduration("2:00:00"))

##test_numeric_plus_nanoduration <- function() {
expect_identical(hour + as.nanoduration("1:00:00"), as.nanoduration("2:00:00"))

##test_character_plus_nanoduration <- function() {
expect_error("hello" + as.nanoduration("1:00:00"), "invalid operand types")

##test_any_plus_nanoduration <- function() {
expect_error("a" + as.nanoduration("1:00:00"), "invalid operand types")

##test_nanoduration_plus_any <- function() {
expect_error(as.nanoduration("1:00:00") + "a", "invalid operand types")


## *
##test_nanoduration_times_numeric <- function() {
expect_identical(as.nanoduration("00:01:00") * 3, as.nanoduration("00:03:00"))

##test_numeric_times_nanoduration <- function() {
expect_identical(3 * as.nanoduration("00:01:00"), as.nanoduration("00:03:00"))

##test_nanoduration_times_integer64 <- function() {
expect_identical(as.nanoduration("00:01:00") * as.integer64(3), as.nanoduration("00:03:00"))

##test_nanoduration_times_nanoduration <- function() {
expect_error(as.nanoduration(1) * as.nanoduration(2), "invalid operand types")

## LLL have to fix this, dependent also on 'integer64' fixes...
## ##test_numeric_times_nanoduration <- function() {
##     expect_identical(1.5 * as.nanoduration("00:01:00"), as.nanoduration("00:01:30"))
## 
##test_numeric_times_nanoduration <- function() {
expect_identical(3 * as.nanoduration("00:01:00"), as.nanoduration("00:03:00"))

##test_integer64_times_nanoduration <- function() {
expect_identical(as.integer64(3) * as.nanoduration("00:01:00"), as.nanoduration("00:03:00"))

##test_any_times_nanoduration <- function() {
expect_error("hello" * as.nanoduration("00:01:00"), "invalid operand types")

##test_nanoduration_times_character <- function() {
expect_error(as.nanoduration("00:01:00") *  "hello", "invalid operand types")

## /
##test_nanoduration_div_numeric <- function() {
expect_identical(as.nanoduration("00:03:00") / 3, as.nanoduration("00:01:00"))

##test_nanoduration_div_integer64 <- function() {
expect_identical(as.nanoduration("00:03:00") / as.integer64(3), as.nanoduration("00:01:00"))

##test_nanoduration_div_integer <- function() {
expect_identical(as.nanoduration("00:03:00") / as.integer(3), as.nanoduration("00:01:00"))

##test_numeric_div_nanoduration <- function() {
expect_error(1.5 / as.nanoduration("00:01:00"), "invalid operand types")

##test_nanoduration_div_any <- function() {
expect_error(as.nanoduration("1:00:00") / "a", "invalid operand types")

##test_nanoduration_div_nanoduration <- function() {
expect_identical(as.nanoduration(6) / as.nanoduration(2), 3.0)

##test_nanoduration_div_nanoduration <- function() {
expect_identical(as.nanoduration(7) / as.nanoduration(2), 3.5)


## unary
##test_unary_minus <- function() {
expect_identical(-as.nanoduration("00:01:00"), as.nanoduration("-00:01:00"))


##test_unary_plus <- function() {
expect_identical(+as.nanoduration("00:01:00"), as.nanoduration("00:01:00"))


## Summary
##test_sum <- function() {
## error in underlying bit64: LLL
## expect_identical(sum(as.nanoduration(1), as.nanoduration(2), as.nanoduration(3)),
##                as.nanoduration(6))
expect_identical(sum(c(as.nanoduration(1), as.nanoduration(2), as.nanoduration(3))),
                 as.nanoduration(6))                

##test_min <- function() {
## error in underlying bit64: LLL
## expect_identical(min(as.nanoduration(1), as.nanoduration(2), as.nanoduration(3)),
##                as.nanoduration(1))
expect_identical(min(c(as.nanoduration(1), as.nanoduration(2), as.nanoduration(3))),
                 as.nanoduration(1))                

##test_max <- function() {
## error in underlying bit64: LLL
## expect_identical(max(as.nanoduration(1), as.nanoduration(2), as.nanoduration(3)),
##                as.nanoduration(3))
expect_identical(max(c(as.nanoduration(1), as.nanoduration(2), as.nanoduration(3))),
                 as.nanoduration(3))                

##test_range <- function() {
## error in underlying bit64: LLL
## expect_identical(range(as.nanoduration(1), as.nanoduration(2), as.nanoduration(3)),
##                c(as.nanoduration(1), as.nanoduration(3)))
expect_identical(range(c(as.nanoduration(1), as.nanoduration(2), as.nanoduration(3))),
                 c(as.nanoduration(1), as.nanoduration(3)))            

##test_prod <- function() {
expect_error(prod(c(as.nanoduration(1), as.nanoduration(2), as.nanoduration(3))),
             "invalid 'type' \\(nanoduration\\) of argument")


## Complex
##test_Complex <- function() {
expect_error(Arg(as.nanoduration(1)), "non-numeric argument to function")


## Logic
##test_Logic <- function() {
expect_error(as.nanoduration(1) | as.nanoduration(1),
             "operations are possible only for numeric, logical or complex types")
expect_error(as.nanoduration(1) | "a",
             "operations are possible only for numeric, logical or complex types")
expect_error("a" & as.nanoduration(1),
             "operations are possible only for numeric, logical or complex types")


## Math
##test_abs <- function() {
expect_identical(abs(as.nanoduration(-1)), as.nanoduration(1))

##test_sign <- function() {
expect_identical(sign(as.nanoduration(-1)), as.integer64(-1))
expect_identical(sign(as.nanoduration(1)), as.integer64(1))

##test_Math <- function() {
expect_error(sqrt(as.nanoduration(1)), "non-numeric argument to mathematical function")


## Math2

##test_Math2 <- function() {
expect_error(round(as.nanoduration(1)), "non-numeric argument to mathematical function")


## Compare
##test_Compare_nanoduration_ANY <- function() {
expect_error(as.nanoduration(1) < "a", "cannot parse nanoduration")
## not quite clear in R:
## > 1 < list(1)
## [1] FALSE
## > 1 > list(1)
## [1] FALSE
## > 1 == list(1)
## [1] TRUE  

##test_Compare_ANY_nanoduration <- function() {
expect_error("a" < as.nanoduration(1), "cannot parse nanoduration")

##test_Compare_character_nanoduration <- function() {
expect_true("12:13:14.151617" == as.nanoduration("12:13:14.151617"))
expect_false("12:13:14.151617" == as.nanoduration("12:13:14.151617001"))
expect_true("12:13:14.151617" != as.nanoduration("12:13:14.151617001"))
expect_false("12:13:14.151617" != as.nanoduration("12:13:14.151617"))
expect_true("12:13:14.151617" < as.nanoduration("12:13:14.151617001"))
expect_false("12:13:14.151617001" < as.nanoduration("12:13:14.151617"))
expect_true("12:13:14.151617" <= as.nanoduration("12:13:14.151617"))
expect_false("12:13:14.151617001" <= as.nanoduration("12:13:14.151617"))
expect_true("12:13:14.151617001" > as.nanoduration("12:13:14.151617"))
expect_false("12:13:14.151617" > as.nanoduration("12:13:14.151617001"))
expect_true("12:13:14.151617" >= as.nanoduration("12:13:14.151617"))
expect_false("12:13:14.151617" >= as.nanoduration("12:13:14.151617001"))
 
##test_Compare_nanoduration_character <- function() {
expect_true(as.nanoduration("12:13:14.151617") == "12:13:14.151617")
expect_false(as.nanoduration("12:13:14.151617") == "12:13:14.151617001")
expect_true(as.nanoduration("12:13:14.151617") != "12:13:14.151617001")
expect_false(as.nanoduration("12:13:14.151617") != "12:13:14.151617")
expect_true(as.nanoduration("12:13:14.151617") < "12:13:14.151617001")
expect_false(as.nanoduration("12:13:14.151617001") < "12:13:14.151617")
expect_true(as.nanoduration("12:13:14.151617") <= "12:13:14.151617")
expect_false(as.nanoduration("12:13:14.151617001") <= "12:13:14.151617")
expect_true(as.nanoduration("12:13:14.151617001") > "12:13:14.151617")
expect_false(as.nanoduration("12:13:14.151617") > "12:13:14.151617001")
expect_true(as.nanoduration("12:13:14.151617") >= "12:13:14.151617")
expect_false(as.nanoduration("12:13:14.151617") >= "12:13:14.151617001")

## Arith
##test_Arith <- function() {
expect_identical(as.nanoduration(4) %% 3, as.integer64(1))


## NA stuff
expect_true(is.na(as.nanoduration(as.integer(NA))))
expect_true(is.na(as.nanoduration(as.integer(NaN))))
d <- as.nanoduration(1:10)
is.na(d) <- 1:3
expect_true(all(is.na(d[1:3])))
expect_true(!any(is.na(d[4:10])))
expect_true(is.na(NA_nanoduration_))
expect_identical(is.na(c(a=NA_nanoduration_)), c(a=TRUE))

## test S4 conversions:
expect_identical(nanoduration(1,1,1,1), as("01:01:01.000_000_001", "nanoduration"))
expect_identical(as.nanoduration(as.integer64(hour)), as(hour, "nanoduration"))
expect_identical(as.nanoduration(hour), as(hour, "nanoduration"))

## test seq:
expect_identical(seq(as.nanoduration("00:00:00"), by=as.nanoduration("00:00:01"), length.out=10),
                 as.nanoduration(seq(0, by=1e9, length.out=10)))
expect_identical(seq(as.nanoduration("00:00:00"), to=as.nanoduration("00:00:09"), length.out=10),
                 as.nanoduration(seq(0, by=1e9, length.out=10)))
expect_identical(seq(as.nanoduration("00:00:00"), to=as.nanoduration("00:00:09"), along.with=1:10),
                 as.nanoduration(seq(0, by=1e9, along.with=1:10)))

## all.equal
##test_all.equal_nanotime_any <- function() {
expect_true(!isTRUE(all.equal(nanotime(1), "a")))

##test_all.equal_any_nanotime <- function() {
expect_true(!isTRUE(all.equal("a", nanotime(1))))

## all.equal
expect_true(isTRUE(all.equal(as.nanoduration(1), as.nanoduration(1))))
expect_true(!isTRUE(all.equal(as.nanoduration(1), as.nanoduration(2))))  
expect_identical(all.equal(as.nanoduration(1), as.nanoduration(2)), "Mean relative difference: 1")  
expect_identical(all.equal(as.nanoduration(c(1L,NA)), as.nanoduration(1:2)), "'is.NA' value mismatch: 0 in current 1 in target")

expect_error(all.equal(as.nanoduration(1), as.nanoduration(1), tolerance="1"), "'tolerance' should be numeric")
expect_error(all.equal(as.nanoduration(1), as.nanoduration(1), scale="a"), "'scale' should be numeric or NULL")
expect_error(all.equal(as.nanoduration(1), as.nanoduration(1), check.attributes="a"), "'check.attributes' must be logical")
expect_error(all.equal(as.nanoduration(1), as.nanoduration(2), scale=-1), "all\\(scale > 0\\) is not TRUE")

expect_false(isTRUE(all.equal(as.nanoduration(1), as.nanoperiod("1d"))))
expect_identical(all.equal(as.nanoduration(1), as.nanoduration(1:2)), "Numeric: lengths (1, 2) differ")
expect_identical(all.equal(as.nanoduration(c(1,2,3)), as.nanoduration(c(1,1,2)), countEQ=TRUE), "Mean relative difference: 0.3333333")
expect_false(isTRUE(all.equal(as.nanoduration(1), 1i)))
expect_identical(all.equal(as.nanoduration(1), as.nanoduration(3), tolerance=1), "Mean absolute difference: 2")
expect_identical(all.equal(as.nanoduration(1), as.nanoduration(2e9), scale=1e9), "Mean scaled difference: 2")
expect_identical(all.equal(as.nanoduration(1), as.nanoduration(2e9), scale=1), "Mean absolute difference: 2e+09")

## test rounding functions:

## nano_ceiling:
## hours:
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:00:00 UTC"), as.nanoduration("06:00:00")),
                 as.nanotime("2010-10-10T12:00:00+00:00"))
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:00:00 UTC"), as.nanoduration("01:00:00")),
                 as.nanotime("2010-10-10T12:00:00+00:00"))
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoduration("06:00:00")),
                 as.nanotime("2010-10-10T18:00:00+00:00"))
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoduration("07:00:00")),
                 as.nanotime("2010-10-10T19:00:00+00:00"))
expect_identical(nano_floor(as.nanotime("1940-12-12 00:12:23 UTC"), as.nanoduration("01:00:00")),
                 as.nanotime("1940-12-12T00:00:00+00:00"))
expect_identical(nano_ceiling(as.nanotime("1970-01-01 UTC"), as.nanoduration("06:00:00")),
                 as.nanotime("1970-01-01T00:00:00+00:00"))
expect_identical(nano_ceiling(as.nanotime("1970-01-01 UTC"), as.nanoduration("01:30:00")),
                 as.nanotime("1970-01-01T00:00:00+00:00"))

expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23 UTC"),
                              as.nanoduration("07:00:00"),
                              origin = as.nanotime("2010-10-10 10:23:12 UTC")),
                 as.nanotime("2010-10-10T17:23:12+00:00"))

expect_error(nano_ceiling(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoduration("07:00:00"), origin = as.nanotime(1:10)),
             "'origin' must be scalar")

                 
expect_error(nano_ceiling(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoduration("-06:00:00")),
             "'precision' must be strictly positive")

expect_error(nano_ceiling(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoduration("-06:00:00"), origin="wrong type"),
             "'origin' must be of class 'nanotime'")

## minutes:
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoduration("00:05:00")),
                 as.nanotime("2010-10-10T12:25:00+00:00"))
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoduration("00:01:00")),
                 as.nanotime("2010-10-10T12:24:00+00:00"))

expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23 UTC"),
                              as.nanoduration("00:07:00"),
                              origin = as.nanotime("2010-10-10 12:22:12 UTC")),
                 as.nanotime("2010-10-10T12:29:12+00:00"))

## seconds:
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoduration("00:00:05")),
                 as.nanotime("2010-10-10T12:23:25+00:00"))
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23.123 UTC"), as.nanoduration("00:00:01")),
                 as.nanotime("2010-10-10T12:23:24+00:00"))

## milliseconds
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23.123456789 UTC"), as.nanoduration("00:00:00.001")),
                 as.nanotime("2010-10-10T12:23:23.124+00:00"))
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23.123456789 UTC"), as.nanoduration("00:00:00.010")),
                 as.nanotime("2010-10-10T12:23:23.13+00:00"))

## microseconds
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23.123456789 UTC"), as.nanoduration("00:00:00.000001")),
                 as.nanotime("2010-10-10T12:23:23.123457+00:00"))
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23.123456789 UTC"), as.nanoduration("00:00:00.000010")),
                 as.nanotime("2010-10-10T12:23:23.12346+00:00"))

## nanoseconds
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23.123456789 UTC"), as.nanoduration("00:00:00.000000001")),
                 as.nanotime("2010-10-10T12:23:23.123456789+00:00"))
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23.123456789 UTC"), as.nanoduration("00:00:00.000000010")),
                 as.nanotime("2010-10-10T12:23:23.123456790+00:00"))
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23.123456789 UTC"), as.nanoduration("00:00:00.000000033")),
                 as.nanotime("2010-10-10T12:23:23.123456814+00:00"))


## nano_floor

## hours:
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoduration("06:00:00")),
                 as.nanotime("2010-10-10T12:00:00+00:00"))
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoduration("07:00:00")),
                 as.nanotime("2010-10-10T12:00:00+00:00"))
expect_identical(nano_floor(as.nanotime("1970-01-01 UTC"), as.nanoduration("06:00:00")),
                 as.nanotime("1970-01-01T00:00:00+00:00"))
expect_identical(nano_floor(as.nanotime("1970-01-01 UTC"), as.nanoduration("01:30:00")),
                 as.nanotime("1970-01-01T00:00:00+00:00"))
expect_identical(nano_floor(as.nanotime("1970-01-01 06:00:00 UTC"), as.nanoduration("06:00:00")),
                 as.nanotime("1970-01-01T06:00:00+00:00"))
expect_identical(nano_floor(as.nanotime("1970-01-01 07:00:00 UTC"), as.nanoduration("07:00:00")),
                 as.nanotime("1970-01-01T07:00:00+00:00"))

expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23 UTC"),
                            as.nanoduration("07:00:00"),
                            origin = as.nanotime("2010-10-10 10:23:12 UTC")),
                 as.nanotime("2010-10-10T10:23:12+00:00"))

expect_error(nano_floor(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoduration("07:00:00"), origin = as.nanotime(1:10)),
             "'origin' must be scalar")

expect_error(nano_floor(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoduration("-06:00:00")),
             "'precision' must be strictly positive")

expect_error(nano_floor(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoduration("-06:00:00"), origin="wrong type"),
             "'origin' must be of class 'nanotime'")

## minutes:
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoduration("00:05:00")),
                 as.nanotime("2010-10-10T12:20:00+00:00"))
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoduration("00:01:00")),
                 as.nanotime("2010-10-10T12:23:00+00:00"))

## seconds:
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoduration("00:00:05")),
                 as.nanotime("2010-10-10T12:23:20+00:00"))
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23.123 UTC"), as.nanoduration("00:00:01")),
                 as.nanotime("2010-10-10T12:23:23+00:00"))

## milliseconds
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23.123456789 UTC"), as.nanoduration("00:00:00.001")),
                 as.nanotime("2010-10-10T12:23:23.123+00:00"))
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23.123456789 UTC"), as.nanoduration("00:00:00.010")),
                 as.nanotime("2010-10-10T12:23:23.12+00:00"))

## microseconds
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23.123456789 UTC"), as.nanoduration("00:00:00.000001")),
                 as.nanotime("2010-10-10T12:23:23.123456+00:00"))
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23.123456789 UTC"), as.nanoduration("00:00:00.000010")),
                 as.nanotime("2010-10-10T12:23:23.12345+00:00"))

## nanoseconds
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23.123456789 UTC"), as.nanoduration("00:00:00.000000001")),
                 as.nanotime("2010-10-10T12:23:23.123456789+00:00"))
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23.123456789 UTC"), as.nanoduration("00:00:00.000000010")),
                 as.nanotime("2010-10-10T12:23:23.12345678+00:00"))
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23.123456789 UTC"), as.nanoduration("00:00:00.000000033")),
                 as.nanotime("2010-10-10T12:23:23.123456781+00:00"))

## rep
expect_identical(rep(as.nanoduration(1), 2), as.nanoduration(rep(1,2)))
expect_identical(rep(as.nanoduration(1:2), each=2), as.nanoduration(rep(1:2, each=2)))

## difftime
expect_identical(as.nanoduration(as.difftime(2, units = "secs")),
                 nanoduration(seconds = 2))
expect_identical(as.nanoduration(as.difftime(2.5, units = "secs")),
                 nanoduration(seconds = 2, nanoseconds = 5e8))
expect_identical(as.nanoduration(as.difftime(2, units = "hours")),
                 nanoduration(hours = 2))
                 
dt_1sec <- as.difftime(1, units = "secs")
nd_1sec <- nanoduration(seconds = 1L)
expect_identical(dt_1sec + nd_1sec, nanoduration(seconds = 2L))
expect_identical(nd_1sec + dt_1sec, nanoduration(seconds = 2L))
expect_identical(dt_1sec - nd_1sec, nanoduration(seconds = 0L))
expect_identical(nd_1sec - dt_1sec, nanoduration(seconds = 0L))
		 
nt <- as.nanotime("2010-10-10 12:23:23.123456789 UTC")
expect_identical(nt + dt_1sec, as.nanotime("2010-10-10 12:23:24.123456789 UTC"))
expect_identical(dt_1sec + nt, as.nanotime("2010-10-10 12:23:24.123456789 UTC"))
expect_identical(nt - dt_1sec, as.nanotime("2010-10-10 12:23:22.123456789 UTC"))
expect_identical(nd_1sec + nt, as.nanotime("2010-10-10 12:23:24.123456789 UTC"))

savedFormat <- options()$nanotimeFormat
options(nanotimeFormat="%Y-%m-%d %H:%M:%S")
ni <- as.nanoival("+2013-01-04 15:00:00 -> 2013-01-04 17:00:00-")
expect_identical(ni + dt_1sec, as.nanoival("+2013-01-04 15:00:01 -> 2013-01-04 17:00:01-"))
expect_identical(dt_1sec + ni, as.nanoival("+2013-01-04 15:00:01 -> 2013-01-04 17:00:01-"))
expect_identical(nd_1sec + ni, as.nanoival("+2013-01-04 15:00:01 -> 2013-01-04 17:00:01-"))
expect_identical(ni - dt_1sec, as.nanoival("+2013-01-04 14:59:59 -> 2013-01-04 16:59:59-"))
expect_identical(ni - nd_1sec, as.nanoival("+2013-01-04 14:59:59 -> 2013-01-04 16:59:59-"))
options(nanotimeFormat=savedFormat)

suppressMessages({
    library(bit64)
    library(nanotime)
})

## constructors
##test_as.nanoperiod_character <- function() {
p1 <- as.nanoperiod("1m1d")
expect_identical(nanoperiod.day(p1), 1)
expect_identical(nanoperiod.month(p1), 1)
expect_identical(nanoperiod.nanoduration(p1), as.nanoduration(0))

expect_identical(as.nanoperiod("1y"), as.nanoperiod("12m"))
expect_identical(as.nanoperiod("2y"), as.nanoperiod("24m"))
expect_identical(as.nanoperiod("1w"), as.nanoperiod("7d"))
expect_identical(as.nanoperiod("2w"), as.nanoperiod("14d"))

p2 <- as.nanoperiod("2m1d/00:01:01.1")
expect_identical(nanoperiod.month(p2), 2)
expect_identical(nanoperiod.day(p2), 1)
expect_identical(nanoperiod.nanoduration(p2), as.nanoduration("00:01:01.1"))

p3 <- as.nanoperiod(c("1d","2d","3d"))
expect_identical(nanoperiod.day(p3[1]), 1)
expect_identical(nanoperiod.day(p3[2]), 2)
expect_identical(nanoperiod.day(p3[3]), 3)    

expect_identical(nanoperiod(), as.nanoperiod(NULL))
expect_identical(nanoperiod(), as.nanoperiod())
expect_identical(length(nanoperiod()), 0L)
expect_identical(length(as.nanoperiod(NULL)), 0L)

## check year and week
expect_identical(as.character(as.nanoperiod("1y/00:01:01")), "12m0d/00:01:01")
expect_identical(as.character(as.nanoperiod("1w/00:01:01")), "0m7d/00:01:01")
expect_identical(names(as.nanoperiod(c(a="1m"))), "a")

## check negative duration
expect_identical(as.nanoperiod("-00:00:01"), as.nanoperiod(as.nanoduration("-00:00:01")))
expect_identical(as.nanoperiod("-0:00:01"), as.nanoperiod(as.nanoduration("-00:00:01")))

##test_as.nanoperiod_character_error <- function() {
expect_error(as.nanoperiod("1wm00:01:01"), "cannot parse nanoperiod")
expect_error(as.nanoperiod("1ym00:01:01"), "cannot parse nanoperiod")
expect_error(as.nanoperiod("1dm00:01:01"), "cannot parse nanoperiod")

##test_as.nanoperiod_integer64 <- function() {
p1 <- as.nanoperiod(as.integer64(1:10))
expect_identical(nanoperiod.nanoduration(p1), as.nanoduration(1:10))
p1 <- as.nanoperiod(c(a=as.integer64(1)))
expect_identical(nanoperiod.nanoduration(p1), as.nanoduration(c(a=1)))

##test_as.nanoperiod_integer <- function() {
p1 <- as.nanoperiod(1:10)
expect_identical(nanoperiod.nanoduration(p1), as.nanoduration(1:10))
p1 <- as.nanoperiod(c(a=1L))
expect_identical(nanoperiod.nanoduration(p1), as.nanoduration(c(a=1)))

##test_as.nanoperiod_numeric <- function() {
p1 <- as.nanoperiod(as.numeric(1:10))
expect_identical(nanoperiod.nanoduration(p1), as.nanoduration(1:10))
p1 <- as.nanoperiod(c(a=1))
expect_identical(nanoperiod.nanoduration(p1), c(a=as.nanoduration(1)))

##test_as.nanoperiod_nanoduration <- function() {
p1 <- as.nanoperiod(c(a=as.nanoduration(1)))
expect_identical(p1, c(a=as.nanoperiod("00:00:00.000_000_001")))

##test_nanoperiod <- function() {
expect_identical(nanoperiod(0,0,1), as.nanoperiod(1))
expect_identical(nanoperiod(0,0,1:10), as.nanoperiod(1:10))
expect_identical(nanoperiod(1,1,0:9), as.nanoperiod(paste0("1m1d/00:00:00.000_000_00", 0:9)))

expect_error(nanoperiod("a"), "argument 'months' must be numeric")
expect_error(nanoperiod(1, "a"), "argument 'days' must be numeric")


## accessors:
##test_nanoperiod.day <- function() {
p1 <- as.nanoperiod(paste0(1:10, "d"))
names(p1) <- 1:10
expected  <- as.numeric(1:10)
names(expected)  <- 1:10
expect_identical(nanoperiod.day(p1), expected)

##test_nanoperiod.month <- function() {
p1 <- as.nanoperiod(paste0(1:10, "m"))
names(p1) <- 1:10
expected  <- as.numeric(1:10)
names(expected)  <- 1:10
expect_identical(nanoperiod.month(p1), expected)

##test_nanoperiod.nanoduration <- function() {
p1 <- as.nanoperiod(1:10)
names(p1) <- 1:10
expected  <- as.numeric(1:10)
names(expected)  <- 1:10
expect_identical(nanoperiod.nanoduration(p1), as.nanoduration(expected))


## show/print/as.character/format
##test_show <- function() {
p1 <- format(as.nanoperiod("1m1d/1:00:00.1"))
expect_identical(p1, "1m1d/01:00:00.100")
expect_stdout(show(as.nanoperiod("1m1d/1:00:00.1")))

##test_print <- function() {
p1 <- format(as.nanoperiod("2m2d/2:02:02.2"))
expect_identical(p1, "2m2d/02:02:02.200")
expect_stdout(print(nanoperiod()))

##test_as.character <- function() {
p1 <- as.character(as.nanoperiod("2m2d/2:02:02.20001"))
expect_identical(p1, "2m2d/02:02:02.200_010")
expect_identical(as.character(NA_nanoperiod_), NA_character_)

##test_as.character_named <- function() {
p1 <- as.character(c(a=as.nanoperiod("2m2d/2:02:02.20001"),
                     b=as.nanoperiod("2m2d/2:02:02.20002")))
expect_identical(p1, c(a="2m2d/02:02:02.200_010", b="2m2d/02:02:02.200_020"))

##test_format <- function() {
p1 <- as.nanoperiod("2m2d/2:02:02.20001")
expect_identical(format(p1), "2m2d/02:02:02.200_010")
expect_identical(as.character(p1), "2m2d/02:02:02.200_010")


## subset:
##test_subset_int <- function() {
p1 <- as.nanoperiod("1m1d/00:00:01")
p2 <- as.nanoperiod("2m2d/00:00:02") 
p3 <- as.nanoperiod("3m3d/00:00:03") 
p4 <- as.nanoperiod("4m4d/00:00:04") 
pp <- c(p1, p2, p3, p4)
expect_identical(pp[1], p1)
expect_identical(pp[2], p2)
expect_identical(pp[3], p3)
expect_identical(pp[4], p4)
expect_identical(pp[1:4], pp)
expect_identical(pp[2:3], c(p2,p3))
expect_identical(pp[-1:-2], c(p3,p4))
expect_warning(pp[1, 2], "unused indices or arguments in 'nanoperiod' subsetting")

##test_subset_logical <- function() {
p1 <- as.nanoperiod("1m1d/00:00:01")
p2 <- as.nanoperiod("2m2d/00:00:02") 
p3 <- as.nanoperiod("3m3d/00:00:03") 
p4 <- as.nanoperiod("4m4d/00:00:04") 
pp <- c(p1, p2, p3, p4)
expect_identical(pp[c(T,F,F,F)], p1)
expect_identical(pp[c(F,T,F,F)], p2)
expect_identical(pp[c(F,F,T,F)], p3)
expect_identical(pp[c(F,F,F,T)], p4)
expect_identical(pp[TRUE], pp)
expect_identical(pp[c(F,T,T,F)], c(p2,p3))
expect_identical(pp[c(NA, F, F, F)], as.nanoperiod(NA_integer_))
expect_identical(pp[c(NA, F, F, F)], NA_nanoperiod_)
expect_warning(pp[T, F], "unused indices or arguments in 'nanoperiod' subsetting")

##test_subset_character <- function() {
pp <- c(x=as.nanoperiod(1), y=as.nanoperiod(2))
expect_identical(pp["x"], c(x=as.nanoperiod(1)))
expect_identical(pp["y"], c(y=as.nanoperiod(2)))
res  <- NA_nanoperiod_
names(res)  <- NA_character_
expect_identical(pp["a"], res)
expect_warning(pp["a", "b"], "unused indices or arguments in 'nanoperiod' subsetting")

## test subset incorrect type
expect_error(pp[as.integer64(1)], "']' not defined on 'nanoperiod' for index of type 'ANY'")

## subassign
##test_subassign_logical <- function() {
x <- as.nanoperiod(1:10)
x[c(T,T,T,F,F,F,F,F,F,F)] <- as.nanoperiod(2:4)
expect_identical(x, as.nanoperiod(c(2:4, 4:10)))
x <- as.nanoperiod(paste0(1:10, "d"))
x[c(T,T,T,F,F,F,F,F,F,F)] <- as.nanoperiod(paste0(2:4, "d"))
expect_identical(x, as.nanoperiod(paste0(c(2:4, 4:10), "d")))

##test_subassign_numeric <- function() {
x <- as.nanoperiod(1:10)
x[1:3] <- as.nanoperiod(2:4)
x[4:10] <- as.nanoperiod(5:11)
expect_identical(x, as.nanoperiod(2:11))
x <- as.nanoperiod(paste0(1:10, "d"))
x[1:3] <- as.nanoperiod(paste0(2:4, "d"))
x[4:10] <- as.nanoperiod(paste0(5:11, "d"))
expect_identical(x, as.nanoperiod(paste0(2:11, "d")))

##test_subsassign_character <- function() {
pp <- c(a=as.nanoperiod(1), b=as.nanoperiod(2), c=as.nanoperiod(3), d=as.nanoperiod(4))
pp[c("b", "c")] <- as.nanoperiod(20:21)
expected <- c(a=as.nanoperiod(1), b=as.nanoperiod(20), c=as.nanoperiod(21), d=as.nanoperiod(4))
expect_identical(pp, expected)

##test_square_bracket <- function() {
pp <- c(a=as.nanoperiod(1), b=as.nanoperiod(2), c=as.nanoperiod(3), d=as.nanoperiod(4))
pp_nonames <- as.nanoperiod(1:4)
expect_identical(pp_nonames[1], pp[[1]])
expect_identical(pp_nonames[2], pp[[2]])
expect_identical(pp_nonames[3], pp[[3]])


## ops
## -
##test_nanoperiod_minus_nanoperiod <- function() {
expect_identical(-as.nanoperiod("2m2d/00:00:02"), as.nanoperiod("-2m-2d/-00:00:02"))
expect_identical(as.nanoperiod("2m2d") - as.nanoperiod("1m1d"), as.nanoperiod("1m1d"))
expect_identical(as.nanoperiod("-1m-1d/-00:00:01") - as.nanoperiod("1m1d/00:00:01"),
               as.nanoperiod("-2m-2d/-00:00:02"))

##test_nanoperiod_minus_nanoperiod_vector <- function() {
expect_identical(as.nanoperiod("1d") - as.nanoperiod(paste0(1:10, "d")),
               as.nanoperiod(paste0(0:-9,"d")))
expect_identical(as.nanoperiod(paste0(1:10, "d")) - as.nanoperiod("1d"),
               as.nanoperiod(paste0(0:9,"d")))
expect_identical(as.nanoperiod(paste0(1:10, "d")) - as.nanoperiod(paste0(0:9, "d")),
               as.nanoperiod(paste0(rep(1,10),"d")))

##test_nanoperiod_minus_numeric <- function() {
expect_identical(as.nanoperiod("2m2d") - 1,
               as.nanoperiod("2m2d/-00:00:00.000_000_001"))
expect_identical(as.nanoperiod(paste0(1:10,"m2d")) - 1,
               as.nanoperiod(paste0(1:10, "m2d/-00:00:00.000_000_001")))
expect_identical(as.nanoperiod("12m2d") - 1:9.0,
               as.nanoperiod(paste0("12m2d/-00:00:00.000_000_00", 1:9)))

##test_nanoperiod_minus_nanoduration <- function() {
expect_identical(as.nanoperiod("2m2d") - as.nanoduration(1), as.nanoperiod("2m2d/-00:00:00.000_000_001"))
expect_identical(as.nanoperiod(paste0(1:10,"m2d")) - as.nanoduration(1),
               as.nanoperiod(paste0(1:10, "m2d/-00:00:00.000_000_001")))
expect_identical(as.nanoperiod("12m2d") - as.nanoduration(1:9),
               as.nanoperiod(paste0("12m2d/-00:00:00.000_000_00", 1:9)))

##test_nanoperiod_minus_integer <- function() {
expect_identical(as.nanoperiod("2m2d") - as.integer(1), as.nanoperiod("2m2d/-00:00:00.000_000_001"))
expect_identical(as.nanoperiod(paste0(1:10,"m2d")) - as.integer(1),
               as.nanoperiod(paste0(1:10, "m2d/-00:00:00.000_000_001")))
expect_identical(as.nanoperiod("12m2d") - as.integer(1:9),
               as.nanoperiod(paste0("12m2d/-00:00:00.000_000_00", 1:9)))

##test_nanoperiod_minus_integer64 <- function() {
expect_identical(as.nanoperiod("2m2d") - as.integer64(1), as.nanoperiod("2m2d/-00:00:00.000_000_001"))
expect_identical(as.nanoperiod(paste0(1:10,"m2d")) - as.integer64(1),
               as.nanoperiod(paste0(1:10, "m2d/-00:00:00.000_000_001")))
expect_identical(as.nanoperiod("12m2d") - as.integer64(1:9),
               as.nanoperiod(paste0("12m2d/-00:00:00.000_000_00", 1:9)))

##test_numeric_minus_nanoperiod <- function() {
expect_identical(1 - as.nanoperiod("1m1d"), as.nanoperiod("-1m-1d/00:00:00.000_000_001"))
expect_identical(1:10 - as.nanoperiod("1m1d"), nanoperiod(-1,-1,1:10))
expect_identical(1 - nanoperiod(1:10,1,1), nanoperiod(-1:-10, -1, 0))

##test_integer64_minus_nanoperiod <- function() {
expect_identical(as.integer64(1) - as.nanoperiod("1m1d"),
               as.nanoperiod("-1m-1d/00:00:00.000_000_001"))
expect_identical(as.integer64(1:10) - as.nanoperiod("1m1d"), nanoperiod(-1,-1,1:10))
expect_identical(as.integer64(1) - nanoperiod(1:10,1,1), nanoperiod(-1:-10, -1, 0))

##test_nanoduration_minus_nanoperiod <- function() {
expect_identical(as.nanoduration(1) - as.nanoperiod("1m1d"),
               as.nanoperiod("-1m-1d/00:00:00.000_000_001"))
expect_identical(as.nanoduration(1:10) - as.nanoperiod("1m1d"), nanoperiod(-1,-1,1:10))
expect_identical(as.nanoduration(1) - nanoperiod(1:10,1,1), nanoperiod(-1:-10, -1, 0))

##test_nanoperiod_minus_any <- function() {
expect_error(as.nanoperiod(1) - "a", "invalid operand types")

##test_any_minus_nanoperiod <- function() {
expect_error("a" - as.nanoperiod(1), "invalid operand types")
expect_error(as.nanoperiod(1) - nanotime(1), "invalid operand types")

## +
##test_nanoperiod_plus_nanoperiod <- function() {
expect_identical(+as.nanoperiod("2m"), as.nanoperiod("2m"))
expect_identical(as.nanoperiod("2m2d") + as.nanoperiod("1m1d"), as.nanoperiod("3m3d"))
expect_identical(as.nanoperiod("-1m-1d/00:00:01") + as.nanoperiod("1m1d/00:00:01"),
               as.nanoperiod("0m0d/00:00:02"))
expect_identical(nanoperiod(1,1,1:10) + nanoperiod(1,1,0), nanoperiod(2,2,1:10))
expect_identical(nanoperiod(1,1,0) + nanoperiod(1,1,1:10), nanoperiod(2,2,1:10))

##test_nanoperiod_plus_nanoperiod_names <- function() {
non_scalar <- as.nanoperiod(c(a="2m2d", b="3m3d"))
scalar <- as.nanoperiod(c(c="1m1d"))
expected <- as.nanoperiod(c(a="3m3d", b="4m4d"))
expect_identical(non_scalar + scalar, expected)
expect_identical(scalar + non_scalar, expected)

##test_integer64_plus_nanoperiod <- function() {
expect_identical(as.integer64(1) + as.nanoperiod(1), as.nanoperiod(2))
expect_identical(as.integer64(0:9) + as.nanoperiod(1), as.nanoperiod(1:10))
expect_identical(as.integer64(1) + nanoperiod(1:10,1,0), nanoperiod(1:10,1,1))

##test_nanoperiod_plus_integer64 <- function() {
expect_identical(as.nanoperiod(1) + as.integer64(1), as.nanoperiod(2))
expect_identical(as.nanoperiod("2m2d") + as.integer64(1), as.nanoperiod("2m2d/00:00:00.000_000_001"))
expect_identical(as.nanoperiod(1) + as.integer64(0:9), as.nanoperiod(1:10))
expect_identical(nanoperiod(1:10,1,0) + as.integer64(1), nanoperiod(1:10,1,1))

##test_nanoperiod_plus_nanoduration <- function() {
expect_identical(as.nanoperiod(1) + as.nanoduration(1), as.nanoperiod(2))
expect_identical(as.nanoperiod("2m2d") + as.nanoduration(1), as.nanoperiod("2m2d/00:00:00.000_000_001"))
expect_identical(as.nanoperiod(1) + as.nanoduration(0:9), as.nanoperiod(1:10))
expect_identical(nanoperiod(1:10,1,0) + as.nanoduration(1), nanoperiod(1:10,1,1))

##test_nanoduration_plus_nanoperiod <- function() {
expect_identical(as.nanoduration(1) + as.nanoperiod(1), as.nanoperiod(2))
expect_identical(as.nanoduration(1) + as.nanoperiod("2m2d"), as.nanoperiod("2m2d/00:00:00.000_000_001"))
expect_identical(as.nanoduration(0:9) + as.nanoperiod(1), as.nanoperiod(1:10))
expect_identical(as.nanoduration(1) + nanoperiod(1:10,1,0), nanoperiod(1:10,1,1))

##test_numeric_plus_nanoperiod <- function() {
expect_identical(as.nanoperiod(1) + 1, as.nanoperiod(2))
expect_identical(as.nanoperiod("2m2d") + 1, as.nanoperiod("2m2d/00:00:00.000_000_001"))
expect_identical(0.0:9.0 + as.nanoperiod(1), as.nanoperiod(1:10))
expect_identical(1.0 + nanoperiod(1:10,1,0), nanoperiod(1:10,1,1))

##test_any_plus_nanoperiod <- function() {
expect_error("a" + as.nanoperiod(1), "invalid operand types")

##test_nanoperiod_plus_any <- function() {
expect_error(as.nanoperiod(1) + any, "invalid operand types")


## *
##test_nanoperiod_times_numeric <- function() {
expect_identical(as.nanoperiod(1) * 3, as.nanoperiod(3))
expect_identical(as.nanoperiod("1m1d") * 3, as.nanoperiod("3m3d"))
expect_identical(nanoperiod(1,1,1) * 1:10, nanoperiod(1:10,1:10,1:10))
expect_identical(nanoperiod(1:10,1,1) * 3, nanoperiod(1:10 * 3, 3, 3))

##test_nanoperiod_times_integer64 <- function() {
expect_identical(as.nanoperiod(1) * as.integer64(3), as.nanoperiod(3))
expect_identical(nanoperiod(1,1,1) * as.integer64(1:10), nanoperiod(1:10,1:10,1:10))
expect_identical(nanoperiod(1:10,1,1) * as.integer64(3), nanoperiod(1:10 * 3, 3, 3))

##test_numeric_times_nanoperiod <- function() {
expect_identical(3 * as.nanoperiod(1), as.nanoperiod(3))
expect_identical(4.5 * as.nanoperiod("10d"), as.nanoperiod("45d"))
expect_identical(1:10.0 * as.nanoperiod(1), as.nanoperiod(1:10))
expect_identical(1 * as.nanoperiod(1:10), as.nanoperiod(1:10))

##test_integer64_times_nanoperiod <- function() {
expect_identical(as.integer64(3) * as.nanoperiod(1), as.nanoperiod(3))
expect_identical(as.integer64(3) * as.nanoperiod("1m1d"), as.nanoperiod("3m3d"))
expect_identical(as.integer64(1:10) * as.nanoperiod(1), as.nanoperiod(1:10))
expect_identical(as.integer64(1) * as.nanoperiod(1:10), as.nanoperiod(1:10))

##test_character_times_nanoperiod <- function() {
expect_error("a"   * as.nanoperiod(1), "invalid operand types")
expect_error("123" * as.nanoperiod(1), "invalid operand types")

##test_nanoperiod_times_character <- function() {
expect_error(as.nanoperiod(1) * "a",   "invalid operand types")
expect_error(as.nanoperiod(1) * "123", "invalid operand types")


## /
##test_nanoperiod_div_numeric <- function() {
expect_identical(as.nanoperiod(4) / 3, as.nanoperiod(1))
expect_identical(as.nanoperiod("5m5d") / 2.5, as.nanoperiod("2m2d"))
expect_identical(as.nanoperiod(4) / c(4,2,1), as.nanoperiod(c(1,2,4)))
expect_identical(as.nanoperiod(4:2) / c(4,2,1), as.nanoperiod(c(1,1,2)))
expect_error(as.nanoperiod("2m") / 0, "divide by zero")

##test_nanoperiod_div_integer64 <- function() {
expect_identical(as.nanoperiod(4) / as.integer64(3), as.nanoperiod(1))
expect_identical(as.nanoperiod("5m5d") / as.integer64(2), as.nanoperiod("2m2d"))
expect_identical(as.nanoperiod(4) / as.integer64(c(4,2,1)), as.nanoperiod(c(1,2,4)))
expect_identical(as.nanoperiod(4:2) / as.integer64(c(4,2,1)), as.nanoperiod(c(1,1,2)))

##test_nanoperiod_div_integer <- function() {
expect_identical(as.nanoperiod(4) / as.integer(3), as.nanoperiod(1))
expect_identical(as.nanoperiod("5m5d") / as.integer(2), as.nanoperiod("2m2d"))
expect_identical(as.nanoperiod(4) / as.integer(c(4,2,1)), as.nanoperiod(c(1,2,4)))
expect_identical(as.nanoperiod(4:2) / as.integer(c(4,2,1)), as.nanoperiod(c(1,1,2)))

##test_nanoperiod_div_any <- function() {
expect_error(as.nanoperiod(1) / "a", "invalid operand types")

##test_any_div_nanoperiod <- function() {
expect_error("a" / as.nanoperiod(1), "invalid operand types")

##test_Logic_nanoperiod_any <- function() {
expect_error(as.nanoperiod(1) | "a", "operation not defined for 'nanoperiod' objects")

##test_Logic_any_nanoperiod <- function() {
expect_error("a" | as.nanoperiod(1), "operation not defined for 'nanoperiod' objects")


## Math/Math2/Summary/Complex
##test_nanoperiod_Math <- function() {
## is that right? LLL
expect_error(abs(as.nanoperiod(1)), "operation not defined for 'nanoperiod' objects")  

##test_nanoperiod_Math2 <- function() {
expect_error(round(as.nanoperiod(1)), "operation not defined for 'nanoperiod' objects")  

##test_nanoperiod_Summary  <- function() {
expect_error(min(as.nanoperiod(1)), "invalid 'type' \\(nanoperiod\\) of argument")
expect_error(max(as.nanoperiod(1)), "invalid 'type' \\(nanoperiod\\) of argument")

##test_nanoperiod_Complex  <- function() {
expect_error(Arg(as.nanoperiod(1)), "operation not defined for 'nanoperiod' objects")  

##test_binary_plus_nanoperiod_nanotime <- function() {
expect_error(as.nanoperiod(1) + nanotime(1),
             "binary '\\+' is not defined for 'nanoperiod' and 'nanotime' objects; instead use 'plus\\(e1, e2, tz\\)'")

##test_binary_plus_nanotime_nanoperiod <- function() {
expect_error(nanotime(1) + as.nanoperiod(1),
             "binary '\\+' is not defined for 'nanotime' and 'nanoperiod' objects; instead use 'plus\\(e1, e2, tz\\)'")

##test_binary_plus_nanoperiod_nanoival <- function() {
expect_error(as.nanoperiod(1) + nanoival(nanotime(1), nanotime(2)),
             "binary '\\+' is not defined for 'nanoperiod' and 'nanoival' objects; instead use 'plus\\(e1, e2, tz\\)'")

##test_binary_plus_nanoival_nanoperiod <- function() {
expect_error(nanoival(nanotime(1), nanotime(2)) + as.nanoperiod(1),
             "binary '\\+' is not defined for 'nanoival' and 'nanoperiod' objects; instead use 'plus\\(e1, e2, tz\\)'")


## Compare
## ----------

##test_nanoperiod_eq_nanoperiod <- function() {
expect_true(as.nanoperiod(1) == as.nanoperiod(1))
expect_true(as.nanoperiod("1d") == as.nanoperiod("1d"))
expect_true(!(as.nanoperiod(1) == as.nanoperiod(2)))
expect_identical(as.nanoperiod(1:10) == as.nanoperiod(1:10), rep(TRUE, 10))

##test_nanoperiod_ne_nanoperiod <- function() {
expect_true(as.nanoperiod(1) != as.nanoperiod(2))
expect_true(as.nanoperiod("1d") != as.nanoperiod("2d"))
expect_true(!(as.nanoperiod(1) != as.nanoperiod(1)))
expect_identical(as.nanoperiod(1:10) != as.nanoperiod(1:10), rep(FALSE, 10))

##test_nanoperiod_eq_any <- function() {
expect_error(as.nanoperiod(1) == "a", "operation not defined for 'nanoperiod' objects")  

##test_any_eq_nanoperiod <- function() {
expect_error("a" == as.nanoperiod(1), "operation not defined for 'nanoperiod' objects")  

##test_all.equal <- function() {
expect_true(all.equal(as.nanoperiod(1), as.nanoperiod(1)))
expect_true(all.equal(as.nanoperiod(1:10), as.nanoperiod(1:10)))
expect_true(all.equal(as.nanoperiod("1m1d"), as.nanoperiod("1d") + as.nanoperiod("1m")))


## names (in general)
##test_nanoperiod_get_names <- function() {
a <- as.nanoperiod(1:10)
names(a) <- "b"
expect_identical(names(a), c("b", rep(as.character(NA), 9)))
    
##test_nanoperiod_set_names <- function() {
names <- c("a","b","c","d")
pp <- as.nanoperiod(1:4)
names(pp) <- names
expect_identical(names(pp), names)
names(pp)[1] <- "x"
expect_identical(names(pp), c("x","b","c","d"))
    

##test_nanoperiod_c <- function() {
pp <- c(x=as.nanoperiod(1), y=as.nanoperiod(2))
expect_identical(names(pp), c("x","y"))
expect_identical(pp[1], c(x=as.nanoperiod(1)))
expect_identical(pp[2], c(y=as.nanoperiod(2)))

pp <- c(as.nanoperiod(1:10), as.nanoperiod(11:20))
expect_identical(pp, as.nanoperiod(1:20))


## plus/minus with 'nanotime':

##test_plus_nanotime_nanoperiod <- function() {
nt <- nanotime("2018-01-01T05:00:00.000000000+00")
p  <- c(p=as.nanoperiod("4m"))
tz <- "America/New_York"
expected <- c(p=nanotime("2018-05-01T00:00:00.000000000-04:00"))
expect_identical(plus(nt, p, tz), expected)

##test_plus_nanotime_nanoperiod_vector1 <- function() {
nt <- nanotime("2018-01-01T05:00:00.000000000+00") + 1:10
p  <- as.nanoperiod("4m")
tz <- "America/New_York"
expected <- nanotime("2018-05-01T00:00:00.000000000-04:00") + 1:10
expect_identical(plus(nt, p, tz), expected)

##test_plus_nanotime_nanoperiod_vector2 <- function() {
nt <- nanotime("2018-01-01T05:00:00.000000000+00")
p  <- as.nanoperiod("4m") + 1:10
tz <- "America/New_York"
expected <- nanotime("2018-05-01T00:00:00.000000000-04:00") + 1:10
expect_identical(plus(nt, p, tz), expected)

##test_plus_nanotime_nanoperiod_vector3 <- function() {
nt <- nanotime("2018-01-01T05:00:00.000000000+00")
p  <- as.nanoperiod("4m") + 1:10
tz <- rep("America/New_York", 10)
expected <- nanotime("2018-05-01T00:00:00.000000000-04:00") + 1:10
expect_identical(plus(nt, p, tz), expected)

##test_plus_nanoperiod_nanotime <- function() {
nt <- nanotime("2018-01-01T05:00:00.000000000+00")
p  <- c(p=as.nanoperiod("4m"))
tz <- "America/New_York"
expected <- c(p=nanotime("2018-05-01T00:00:00.000000000-04:00"))
expect_identical(plus(p, nt, tz), expected)

##test_minus_nanotime_nanoperiod <- function() {
nt <- c(p1=nanotime("2018-05-01T00:00:00.000000000-04:00"))
p  <- c(p2=as.nanoperiod("4m"))
tz <- "America/New_York"
expected <- c(p1=nanotime("2018-01-01T00:00:00.000000000-05:00"))
expect_identical(minus(nt, p, tz), expected)

##test_minus_nanoperiod_nanotime <- function() {
nt <- nanotime("2018-05-01T00:00:00.000000000-04:00")
p  <- as.nanoperiod("4m")
tz <- "America/New_York"
expect_error(minus(p, nt, tz), "operation not defined for 'nanoperiod' objects")

## test the crossing of daylight saving time in both directions:

## adding/subtracting a nanoperiod should not realign if doing so
## crosses again a DST boundary:

## look at cases over the Spring boundary in America:
nt <- as.nanotime("2020-03-08 01:45:30 America/New_York")
p <- as.nanoperiod("00:30:00")
tz <- "America/New_York"
expected <- as.nanotime("2020-03-08 03:15:30 America/New_York")
expect_identical(plus(nt, p, tz), expected)

nt <- as.nanotime("2020-03-08 01:00:00 America/New_York")
p <- as.nanoperiod("01:00:00")
tz <- "America/New_York"
expected <- as.nanotime("2020-03-08 03:00:00 America/New_York")
expect_identical(plus(nt, p, tz), expected)

nt <- as.nanotime("2020-03-08 03:15:30 America/New_York")
p <- as.nanoperiod("00:30:00")
tz <- "America/New_York"
expected <- as.nanotime("2020-03-08 01:45:30 America/New_York")
expect_identical(minus(nt, p, tz), expected)

nt <- as.nanotime("2020-03-08 03:00:00 America/New_York")
p <- as.nanoperiod("01:00:00")
tz <- "America/New_York"
expected <- as.nanotime("2020-03-08 01:00:00 America/New_York")
expect_identical(minus(nt, p, tz), expected)

## look at the cases over the Autumn boundary in America:
nt <- as.nanotime("2020-11-01 01:45:30 America/New_York")
p <- as.nanoperiod("00:30:00")
tz <- "America/New_York"
expected <- as.nanotime("2020-11-01 02:15:30 America/New_York")
expect_identical(plus(nt, p, tz), expected)

nt <- as.nanotime("2020-11-01 01:00:00 America/New_York")
p <- as.nanoperiod("01:00:00")
tz <- "America/New_York"
expected <- as.nanotime("2020-11-01 02:00:00 America/New_York")
expect_identical(plus(nt, p, tz), expected)

nt <- as.nanotime("2020-11-01 02:15:30 America/New_York")
p <- as.nanoperiod("00:30:00")
tz <- "America/New_York"
##expected <- as.nanotime("2020-11-01 01:45:30 America/New_York")  # ambiguous
expected <- as.nanotime("2020-11-01 06:45:30+00:00")
expect_identical(minus(nt, p, tz), expected)

nt <- as.nanotime("2020-11-01 02:00:00 America/New_York")
p <- as.nanoperiod("01:00:00")
tz <- "America/New_York"
## expected <- as.nanotime("2020-11-01 01:00:00 America/New_York")    # ambiguous
expected <- as.nanotime("2020-11-01 06:00:00+00:00")
expect_identical(minus(nt, p, tz), expected)


## adding/subtracting a nanoperiod should realign if doing so does
## not cross again a DST boundary:

## look at cases over the Spring boundary in America:
nt <- as.nanotime("2020-03-08 01:00:00 America/New_York")
p <- as.nanoperiod("02:00:01")
tz <- "America/New_York"
expected <- as.nanotime("2020-03-08 03:00:01 America/New_York")
expect_identical(plus(nt, p, tz), expected)

nt <- as.nanotime("2020-03-08 03:00:00 America/New_York")
p <- as.nanoperiod("02:00:01")
tz <- "America/New_York"
expected <- as.nanotime("2020-03-08 00:59:59 America/New_York")
expect_identical(minus(nt, p, tz), expected)

## look at the cases over the Autumn boundary in America:
nt <- as.nanotime("2020-11-01 01:00:00 America/New_York")
p <- as.nanoperiod("02:00:01")
tz <- "America/New_York"
expected <- as.nanotime("2020-11-01 03:00:01 America/New_York")
expect_identical(plus(nt, p, tz), expected)

nt <- as.nanotime("2020-11-01 03:00:00 America/New_York")
p <- as.nanoperiod("02:00:01")
tz <- "America/New_York"
expected <- as.nanotime("2020-11-01 00:59:59 America/New_York")
expect_identical(minus(nt, p, tz), expected)


## plus/minus with 'nanoival':

##test_plus_nanoival_nanoperiod <- function() {
start <- nanotime("2018-01-01T05:00:00.000000000+00")
end <- nanotime("2018-01-01T23:00:00.000000000+00")
ni <- nanoival(start, end)
p  <- as.nanoperiod("4m")
tz <- "America/New_York"
expected <- as.nanoival("+2018-05-01T00:00:00.000000000-04:00 -> 2018-05-01T18:00:00.000000000-04:00-")
expect_identical(plus(ni, p, tz), expected)

##test_plus_nanoival_nanoperiod_pre_1970 <- function() {
start <- nanotime("1969-01-01T05:00:00.000000000+00")
end <- nanotime("1969-01-01T23:00:00.000000000+00")
ni <- nanoival(start, end)
p  <- as.nanoperiod("4m")
tz <- "America/New_York"
expected <- as.nanoival("+1969-05-01T00:00:00.000000000-04:00 -> 1969-05-01T18:00:00.000000000-04:00-")
expect_identical(plus(ni, p, tz), expected)

##test_plus_nanoperiod_nanoival <- function() {
start <- nanotime("2018-01-01T05:00:00.000000000+00")
end <- nanotime("2018-01-01T23:00:00.000000000+00")
ni <- nanoival(start, end)
p  <- c(a=as.nanoperiod("4m"))
tz <- "America/New_York"
expected <- c(a=as.nanoival("+2018-05-01T00:00:00.000000000-04:00 -> 2018-05-01T18:00:00.000000000-04:00-"))
expect_identical(plus(p, ni, tz), expected)

##test_minus_nanoival_nanoperiod <- function() {
start <- nanotime("2018-05-01T05:00:00.000000000-04")
end <- nanotime("2018-05-01T23:00:00.000000000-04")
ni <- c(a=nanoival(start, end))
p  <- as.nanoperiod("4m")
tz <- "America/New_York"
expected <- c(a=as.nanoival("+2018-01-01T05:00:00.000000000-05:00 -> 2018-01-01T23:00:00.000000000-05:00-"))
expect_identical(minus(ni, p, tz), expected)

##test_minus_nanoperiod_nanoival <- function() {
start <- nanotime("2018-05-01T05:00:00.000000000-04")
end <- nanotime("2018-05-01T23:00:00.000000000-04")
ni <- nanoival(start, end)
p  <- as.nanoperiod("4m")
tz <- "America/New_York"
expect_error(minus(p, ni, tz), "operation not defined for 'nanoperiod' objects")


## NA stuff
expect_true(is.na(as.nanoperiod(NA_integer_)))
expect_true(is.na(as.nanoperiod(NA_integer64_)))
expect_true(is.na(as.nanoperiod(NA_real_)))
expect_true(is.na(as.nanoperiod(as.integer(NaN))))
p <- as.nanoperiod(1:10)
is.na(p) <- 1:3
expect_true(all(is.na(p[1:3])))
expect_true(!any(is.na(p[4:10])))
expect_true(is.na(NA_nanoperiod_))
expect_true(is.na(nanoperiod.nanoduration(NA_nanoperiod_)))
expect_true(is.na(nanoperiod.month(NA_nanoperiod_)))
expect_true(is.na(nanoperiod.day(NA_nanoperiod_)))

## test S4 conversions:
expect_identical(nanoperiod(1,1,1), as("1m1d/00:00:00.000_000_001", "nanoperiod"))
hour <- 3600*1e9
expect_identical(as.nanoperiod(as.integer64(hour)), as(hour, "nanoperiod"))
expect_identical(as.nanoperiod(hour), as(hour, "nanoperiod"))
expect_identical(as.nanoperiod(hour), as(as.nanoduration(hour), "nanoperiod"))

## 0-length ops:
## ------------

## constructor:

expect_identical(nanoperiod(integer(), integer(), nanoduration()), nanoperiod())
expect_identical(nanoperiod(1, 1, nanoduration()), nanoperiod())
expect_identical(nanoperiod(1,integer(), as.nanoduration(1)), nanoperiod())
expect_identical(nanoperiod(numeric(), integer(), as.nanoduration(1)), nanoperiod())

## Comp:
expect_identical(as.nanoperiod() == as.nanoperiod(), logical())
expect_identical(as.nanoperiod(1) == as.nanoperiod(), logical())
expect_identical(as.nanoperiod(1) != as.nanoperiod(), logical())


## ops
expect_identical(as.nanoperiod() + as.nanoperiod(), as.nanoperiod())
expect_identical(nanoperiod(1)   + as.nanoperiod(), as.nanoperiod())
expect_identical(nanoperiod()    + as.nanoperiod(1:10), as.nanoperiod())
expect_identical(nanoperiod()    + as.nanoduration(1:10), as.nanoperiod())
expect_identical(nanoperiod()    + as.integer64(1:10), as.nanoperiod())
expect_identical(nanoperiod(1)   + as.integer(), as.nanoperiod())
expect_identical(as.nanoperiod() - as.nanoperiod(), as.nanoperiod())
expect_identical(nanoperiod(1)   - as.nanoperiod(), as.nanoperiod())
expect_identical(nanoperiod()    - as.nanoperiod(1:10), as.nanoperiod())
expect_identical(nanoperiod()    * 3, as.nanoperiod())
expect_identical(nanoperiod(1)   * integer(), as.nanoperiod())

## accessors
expect_identical(nanoperiod.month(nanoperiod()), numeric())
expect_identical(nanoperiod.day(nanoperiod()), numeric())
expect_identical(nanoperiod.nanoduration(nanoperiod()), as.nanoduration())

## all.equal:
expect_identical(all.equal(as.nanoperiod("1m"), as.nanoperiod("1m")), TRUE)
expect_false(isTRUE(all.equal(as.nanoperiod("1d"), "A")))
expect_identical(all.equal(as.nanoperiod("1d"), NA_nanoperiod_), "'is.NA' value mismatch: 1 in current 0 in target")


## test rounding functions:

## nano_ceiling:

## years:
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:00:00 America/New_York"), as.nanoperiod("12m"), tz="America/New_York"),
                 as.nanotime("2011-01-01T00:00:00-05:00"))
expect_identical(nano_ceiling(as.nanotime("1970-01-01 America/New_York"), as.nanoperiod("12m"), tz="America/New_York"),
                 as.nanotime("1970-01-01 America/New_York"))
expect_identical(nano_ceiling(as.nanotime("1970-01-01 06:00:00 America/New_York"), as.nanoperiod("12m"), tz="America/New_York"),
                 as.nanotime("1971-01-01 America/New_York"))

expect_error(nano_ceiling(c(as.nanotime("2010-10-10 12:00:00 America/New_York"), as.nanotime("2010-10-10 11:00:00 America/New_York")),
                          as.nanoperiod("12m"), tz="America/New_York"),
             "'x' must be sorted")

expect_error(nano_ceiling(as.nanotime("2010-10-10 12:00:00 America/New_York"), as.nanoperiod("12m"), tz=c("America/New_York", "UTC")),
             "'tz' must be scalar")

expect_identical(nano_ceiling(as.nanotime("1940-10-10 12:00:00 America/New_York"), as.nanoperiod("12m"), tz="America/New_York"),
                 as.nanotime("1941-01-01T00:00:00-05:00"))

expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:00:00 America/New_York"),
                              as.nanoperiod("12m"),
                              origin=as.nanotime("2010-10-10 00:00:01 America/New_York"),
                              tz="America/New_York"),
                 as.nanotime("2011-10-10T04:00:01+00:00"))

expect_error(nano_ceiling(as.nanotime("2010-10-10 12:00:00 America/New_York"),
                          as.nanoperiod("12m"),
                          origin=as.nanotime(1:10),
                          tz="America/New_York"),
             "'origin' must be scalar")

expect_identical(nano_ceiling(as.nanotime("1904-10-10 12:00:00 America/New_York"),
                              as.nanoperiod("12m"),
                              origin=as.nanotime("1904-10-10 00:00:01 America/New_York"),
                              tz="America/New_York"),
                 as.nanotime("1905-10-10T05:00:01+00:00"))

expect_error(nano_ceiling(as.nanotime("2010-10-10 12:00:00 America/New_York"),
                          as.nanoperiod("12m"),
                          origin=as.nanotime("2004-10-10 00:00:01 America/New_York"),
                          tz="America/New_York"),
             "when specifying 'origin', the first interval must contain at least one observation")

expect_error(nano_ceiling(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoperiod("-12m"), tz="America/New_York"),
             "'precision' must be strictly positive")

expect_error(nano_ceiling(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoperiod("-12m"), tz="America/New_York", origin="wrong type"),
             "'origin' must be of class 'nanotime'")

expect_error(nano_ceiling(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoperiod("-12m"), tz=12),
             "'tz' must be of type 'character'")

## months:
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:00:00 America/New_York"), as.nanoperiod("13m"), tz="America/New_York"),
                 as.nanotime("2011-11-01T00:00:00-04:00"))
expect_identical(nano_ceiling(as.nanotime("2010-08-10 12:00:00 America/New_York"), as.nanoperiod("3m"), tz="America/New_York"),
                 as.nanotime("2010-10-01T00:00:00-04:00"))
expect_identical(nano_ceiling(as.nanotime("2010-08-10 12:00:00 America/New_York"), as.nanoperiod("1m"), tz="America/New_York"),
                 as.nanotime("2010-09-01T00:00:00-04:00"))
expect_identical(nano_ceiling(as.nanotime("1940-08-10 12:00:00 America/New_York"), as.nanoperiod("1m"), tz="America/New_York"),
                 as.nanotime("1940-09-01T00:00:00-04:00"))

## days:
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:00:00 America/New_York"), as.nanoperiod("1d"), tz="America/New_York"),
                 as.nanotime("2010-10-11T00:00:00-04:00"))
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:00:00 America/New_York"), as.nanoperiod("2d"), tz="America/New_York"),
                 as.nanotime("2010-10-12T00:00:00-04:00"))
expect_identical(nano_ceiling(as.nanotime("1940-10-10 12:00:00 America/New_York"), as.nanoperiod("2d"), tz="America/New_York"),
                 as.nanotime("1940-10-12T00:00:00-05:00"))

## hours:
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:00:00 America/New_York"), as.nanoperiod("06:00:00"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:00:00-04:00"))
expect_identical(nano_ceiling(as.nanotime("1927-10-10 12:00:00 America/New_York"), as.nanoperiod("06:00:00"), tz="America/New_York"),
                 as.nanotime("1927-10-10T12:00:00-05:00"))
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:00:00 America/New_York"), as.nanoperiod("01:00:00"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:00:00-04:00"))
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23 America/New_York"), as.nanoperiod("06:00:00"), tz="America/New_York"),
                 as.nanotime("2010-10-10T18:00:00-04:00"))
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23 America/New_York"), as.nanoperiod("07:00:00"), tz="America/New_York"),
                 as.nanotime("2010-10-10T19:00:00-04:00"))
expect_identical(nano_ceiling(as.nanotime("1899-10-10 12:23:23 America/New_York"), as.nanoperiod("07:00:00"), tz="America/New_York"),
                 as.nanotime("1899-10-10T19:00:00-05:00"))

## minutes:
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23 America/New_York"), as.nanoperiod("00:05:00"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:25:00-04:00"))
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:25:00 America/New_York"), as.nanoperiod("00:05:00"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:25:00-04:00"))
expect_identical(nano_ceiling(as.nanotime("1922-10-10 12:23:23 America/New_York"), as.nanoperiod("00:05:00"), tz="America/New_York"),
                 as.nanotime("1922-10-10T12:25:00-05:00"))
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23 America/New_York"), as.nanoperiod("00:01:00"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:24:00-04:00"))
expect_identical(nano_ceiling(as.nanotime("1922-10-10 12:25:00 America/New_York"), as.nanoperiod("00:05:00"), tz="America/New_York"),
                 as.nanotime("1922-10-10T12:25:00-05:00"))

## seconds:
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23 America/New_York"), as.nanoperiod("00:00:05"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:23:25-04:00"))
expect_identical(nano_ceiling(as.nanotime("1969-10-10 12:23:23 America/New_York"), as.nanoperiod("00:00:05"), tz="America/New_York"),
                 as.nanotime("1969-10-10T12:23:25-04:00"))
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23.123 America/New_York"), as.nanoperiod("00:00:01"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:23:24-04:00"))
expect_identical(nano_ceiling(as.nanotime("1888-10-10 12:23:23.123 America/New_York"), as.nanoperiod("00:00:11"), tz="America/New_York"),
                 as.nanotime("1888-10-10T12:23:34-05:00"))

## milliseconds
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23.123456789 America/New_York"), as.nanoperiod("00:00:00.001"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:23:23.124-04:00"))
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23.123456789 America/New_York"), as.nanoperiod("00:00:00.010"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:23:23.13-04:00"))

## microseconds
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23.123456789 America/New_York"), as.nanoperiod("00:00:00.000001"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:23:23.123457-04:00"))
expect_identical(nano_ceiling(as.nanotime("1933-10-10 12:23:23.123456789 America/New_York"), as.nanoperiod("00:00:00.000001"), tz="America/New_York"),
                 as.nanotime("1933-10-10T12:23:23.123457-05:00"))
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23.123456789 America/New_York"), as.nanoperiod("00:00:00.000010"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:23:23.12346-04:00"))

## nanoseconds
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23.123456789 America/New_York"), as.nanoperiod("00:00:00.000000001"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:23:23.123456789-04:00"))
expect_identical(nano_ceiling(as.nanotime("1931-10-10 12:23:23.123456789 America/New_York"), as.nanoperiod("00:00:00.000000001"), tz="America/New_York"),
                 as.nanotime("1931-10-10T12:23:23.123456789-05:00"))
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23.123456789 America/New_York"), as.nanoperiod("00:00:00.000000010"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:23:23.123456790-04:00"))
expect_identical(nano_ceiling(as.nanotime("2010-10-10 12:23:23.123456789 America/New_York"), as.nanoperiod("00:00:00.000000033"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:23:23.123456789-04:00"))


## nano_floor


## years:
expect_identical(nano_floor(as.nanotime("2010-10-10 12:00:00 America/New_York"), as.nanoperiod("12m"), tz="America/New_York"),
                 as.nanotime("2010-01-01T05:00:00-00:00"))
expect_identical(nano_floor(as.nanotime("1970-01-01 06:00:00 America/New_York"), as.nanoperiod("12m"), tz="America/New_York"),
                 as.nanotime("1970-01-01 America/New_York"))

expect_error(nano_floor(c(as.nanotime("2010-10-10 12:00:00 America/New_York"), as.nanotime("2010-10-10 11:00:00 America/New_York")),
                        as.nanoperiod("12m"), tz="America/New_York"),
             "'x' must be sorted")

expect_error(nano_floor(as.nanotime("2010-10-10 12:00:00 America/New_York"), as.nanoperiod("12m"), tz=c("America/New_York", "UTC")),
             "'tz' must be scalar")

expect_identical(nano_floor(as.nanotime("2010-10-10 12:00:00 America/New_York"),
                              as.nanoperiod("12m"),
                              origin=as.nanotime("2010-10-10 00:00:01 America/New_York"),
                              tz="America/New_York"),
                 as.nanotime("2010-10-10T04:00:01+00:00"))

expect_error(nano_floor(as.nanotime("2010-10-10 12:00:00 America/New_York"),
                          as.nanoperiod("12m"),
                          origin=as.nanotime("2004-10-10 00:00:01 America/New_York"),
                          tz="America/New_York"),
             "when specifying 'origin', the first interval must contain at least one observation")


expect_error(nano_floor(as.nanotime("2010-10-10 12:00:00 America/New_York"),
                          as.nanoperiod("12m"),
                          origin=as.nanotime(1:10),
                          tz="America/New_York"),
             "'origin' must be scalar")

expect_error(nano_floor(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoperiod("-12m"), tz="America/New_York"),
             "'precision' must be strictly positive")

expect_error(nano_floor(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoperiod("-12m"), tz="America/New_York", origin="wrong type"),
             "'origin' must be of class 'nanotime'")

expect_error(nano_floor(as.nanotime("2010-10-10 12:23:23 UTC"), as.nanoperiod("-12m"), tz=12),
             "'tz' must be of type 'character'")



## hours:
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23 America/New_York"), as.nanoperiod("06:00:00"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:00:00-04:00"))
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23 America/New_York"), as.nanoperiod("07:00:00"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:00:00-04:00"))
## minutes:
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23 America/New_York"), as.nanoperiod("00:05:00"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:20:00-04:00"))
expect_identical(nano_floor(as.nanotime("2010-10-10 12:20:00 America/New_York"), as.nanoperiod("00:05:00"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:20:00-04:00"))
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23 America/New_York"), as.nanoperiod("00:01:00"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:23:00-04:00"))
expect_identical(nano_floor(as.nanotime("1911-10-10 12:20:00 America/New_York"), as.nanoperiod("00:01:00"), tz="America/New_York"),
                 as.nanotime("1911-10-10T12:20:00-05:00"))

## seconds:
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23 America/New_York"), as.nanoperiod("00:00:05"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:23:20-04:00"))
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23.123 America/New_York"), as.nanoperiod("00:00:01"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:23:23-04:00"))

## milliseconds
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23.123456789 America/New_York"), as.nanoperiod("00:00:00.001"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:23:23.123-04:00"))
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23.123456789 America/New_York"), as.nanoperiod("00:00:00.010"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:23:23.12-04:00"))

## microseconds
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23.123456789 America/New_York"), as.nanoperiod("00:00:00.000001"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:23:23.123456-04:00"))
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23.123456789 America/New_York"), as.nanoperiod("00:00:00.000010"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:23:23.12345-04:00"))

## nanoseconds
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23.123456789 America/New_York"), as.nanoperiod("00:00:00.000000001"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:23:23.123456789-04:00"))
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23.123456789 America/New_York"), as.nanoperiod("00:00:00.000000010"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:23:23.12345678-04:00"))
expect_identical(nano_floor(as.nanotime("2010-10-10 12:23:23.123456789 America/New_York"), as.nanoperiod("00:00:00.000000033"), tz="America/New_York"),
                 as.nanotime("2010-10-10T12:23:23.123456789-04:00"))
expect_identical(nano_floor(as.nanotime("1965-10-10 12:23:23.123456789 America/New_York"), as.nanoperiod("00:00:00.000000033"), tz="America/New_York"),
                 as.nanotime("1965-10-10T12:23:23.123456789-04:00"))

## rep
expect_identical(rep(as.nanoperiod(1), 2), as.nanoperiod(rep(1,2)))
expect_identical(rep(as.nanoperiod(1:2), each=2), as.nanoperiod(rep(1:2, each=2)))

library(nanotime)

if (!requireNamespace("data.table", quietly=TRUE))
    exit_file("Package 'data.table' missing")

library(data.table)

## nanotime
##test_nanotime_data_table_constructor <- function() {
## simpler data.table, inserts 'formatted'
set.seed(42)
N <- 300
shine <- nanotime(Sys.time()) + cumsum(10*rpois(n=N+1, lambda=4))
rain  <- nanotime(Sys.time()) + cumsum(10*rpois(n=N+1, lambda=4) + round(runif(N+1)*25))
newdf <- data.table(rain=rain, shine=shine)
expect_identical(newdf$rain,  rain)
expect_identical(newdf$shine, shine)

##test_nanotime_data_table_rbind <- function() {
t1 <- nanotime(1:10)
d1 <- 1:10
t2 <- nanotime(11:20)
d2 <- 11:20
df1 <- data.table(t = t1, d = d1)
df2 <- data.table(t = t2, d = d2)
df <- rbind(df1, df2)
expect_identical(df, data.table(t = c(t1, t2), d = c(d1, d2)))

##test_nanotime_data_table_cbind <- function() {
t1 <- nanotime(1:10)
d1 <- 1:10
t2 <- nanotime(11:20)
d2 <- 11:20
df1 <- data.table(t1 = t1, d1 = d1)
df2 <- data.table(t2 = t2, d2 = d2)
df <- cbind(df1, df2)
expect_identical(df, data.table(t1 = t1, d1 = d1, t2=t2, d2=d2))


## nanoduration
##test_nanoduration_data_table_constructor <- function() {
## simpler data.table, inserts 'formatted'
set.seed(42)
N <- 300
shine <- as.nanoduration(1000) + cumsum(10*rpois(n=N+1, lambda=4))
rain  <- as.nanoduration(2000) + cumsum(10*rpois(n=N+1, lambda=4) + round(runif(N+1)*25))
newdf <- data.table(rain=rain, shine=shine)
expect_identical(newdf$rain,  rain)
expect_identical(newdf$shine, shine)

##test_nanoduration_data_table_rbind <- function() {
t1 <- as.nanoduration(1:10)
d1 <- 1:10
t2 <- as.nanoduration(11:20)
d2 <- 11:20
df1 <- data.table(t = t1, d = d1)
df2 <- data.table(t = t2, d = d2)
df <- rbind(df1, df2)
expect_identical(df, data.table(t = c(t1, t2), d = c(d1, d2)))


##test_nanoduration_data_table_cbind <- function() {
t1 <- as.nanoduration(1:10)
d1 <- 1:10
t2 <- as.nanoduration(11:20)
d2 <- 11:20
df1 <- data.table(t1 = t1, d1 = d1)
df2 <- data.table(t2 = t2, d2 = d2)
df <- cbind(df1, df2)
expect_identical(df, data.table(t1 = t1, d1 = d1, t2=t2, d2=d2))


## nanoperiod
##test_nanoperiod_data.table_constructor  <- function() {
## simpler data.table, inserts 'formatted'
set.seed(42)
N <- 300
shine <- as.nanoperiod(1000) + cumsum(10*rpois(n=N+1, lambda=4))
rain  <- as.nanoperiod(2000) + cumsum(10*rpois(n=N+1, lambda=4) + round(runif(N+1)*25))
newdf <- data.table(rain=rain, shine=shine)
expect_identical(newdf$rain,  rain)
expect_identical(newdf$shine, shine)

##test_nanoperiod_data_table_rbind <- function() {
t1 <- as.nanoperiod(1:10)
d1 <- 1:10
t2 <- as.nanoperiod(11:20)
d2 <- 11:20
df1 <- data.table(t = t1, d = d1)
df2 <- data.table(t = t2, d = d2)
df <- rbind(df1, df2)
expect_identical(df, data.table(t = c(t1, t2), d = c(d1, d2)))

##test_nanoperiod_data_table_cbind <- function() {
t1 <- as.nanoperiod(1:10)
d1 <- 1:10
t2 <- as.nanoperiod(11:20)
d2 <- 11:20
df1 <- data.table(t1 = t1, d1 = d1)
df2 <- data.table(t2 = t2, d2 = d2)
df <- cbind(df1, df2)
expect_identical(df, data.table(t1 = t1, d1 = d1, t2=t2, d2=d2))


## nanoival
##test_nanoival_data.table_constructor  <- function() {
## simpler data.table, inserts 'formatted'
set.seed(42)
N <- 300
shine_start <- nanotime(1000) + cumsum(10*rpois(n=N+1, lambda=4))
shine_end   <- shine_start + as.nanoduration("01:00:00")
rain_start  <- nanotime(2000) + cumsum(10*rpois(n=N+1, lambda=4) + round(runif(N+1)*25))
rain_end    <- shine_end + as.nanoduration("02:00:00")
shine <- nanoival(shine_start, shine_end)
rain  <- nanoival(rain_start, rain_end)
newdf <- data.table(rain=rain, shine=shine)
expect_identical(newdf$rain,  rain)
expect_identical(newdf$shine, shine)

##test_nanoival_data_table_rbind <- function() {
t1 <- nanoival(nanotime(1:10), nanotime(2:11))
d1 <- 1:10
t2 <- nanoival(nanotime(11:20), nanotime(12:21))
d2 <- 11:20
df1 <- data.table(t = t1, d = d1)
df2 <- data.table(t = t2, d = d2)
df <- rbind(df1, df2)
expect_identical(df, data.table(t = c(t1, t2), d = c(d1, d2)))

##test_nanoival_data_table_cbind <- function() {
t1 <- nanoival(nanotime(1:10), nanotime(2:11))
d1 <- 1:10
t2 <- nanoival(nanotime(11:20), nanotime(12:21))
d2 <- 11:20
df1 <- data.table(t1 = t1, d1 = d1)
df2 <- data.table(t2 = t2, d2 = d2)
df <- cbind(df1, df2)
expect_identical(df, data.table(t1 = t1, d1 = d1, t2=t2, d2=d2))

suppressMessages({
    library(bit64)
    library(nanotime)
})

## ------------ `-`
#"test_nanotime-nanotime" <- function() {
expect_identical(nanotime(2)  - nanotime(1),  as.nanoduration(1))
expect_identical(nanotime(-1) - nanotime(-2), as.nanoduration(1))

#"test_nanotime-integer64" <- function() {
expect_identical(nanotime(2)  - as.integer64(1),  nanotime(1))
expect_identical(nanotime(-1) - as.integer64(-2), nanotime(1))

#"test_nanotime-integer" <- function() {
expect_identical(nanotime(2)  - 1L,  nanotime(1))
expect_identical(nanotime(-1) - -2L, nanotime(1))

#"test_nanotime-character" <- function() {
expect_error(nanotime(0) - "A", "invalid operand types")

#"test_character-nanotime" <- function() {
expect_error("A" - nanotime(0), "invalid operand types")

#"test_numeric-nanotime" <- function() {
expect_error(1 - nanotime(0), "invalid operand types")



## ----------- `+`
#"test_nanotime+numeric" <- function() {
expect_identical(nanotime(0) +  1, nanotime(1))
expect_identical(nanotime(0) + -1, nanotime(-1))


#"test_nanotime+integer" <- function() {
expect_identical(nanotime(0) +  1L, nanotime(1))
expect_identical(nanotime(0) + -1L, nanotime(-1))

#"test_nanotime+integer64" <- function() {
expect_identical(nanotime(0) + as.integer64(1), nanotime(1))
expect_identical(nanotime(0) + as.integer64(-1), nanotime(-1))

#"test_nanotime+character" <- function() {
expect_error(nanotime(0) + "A", "invalid operand types")

#"test_character+nanotime" <- function() {
expect_error("A" + nanotime(0), "invalid operand types")

#"test_nanotime+nanotime" <- function() {
expect_error(nanotime(1) + nanotime(0), "invalid operand types")

#"test_nanotime+integer64" <- function() {
expect_identical(nanotime(0) + as.integer64(1), nanotime(1))
expect_identical(nanotime(0) + as.integer64(-1), nanotime(-1))

#"test_numeric+nanotime" <- function() {
expect_identical(nanotime(0) +  1, nanotime( 1))
expect_identical(nanotime(0) + -1, nanotime(-1))

#"test_integer+nanotime" <- function() {
expect_identical( 1L + nanotime(0), nanotime(1))
expect_identical(-1L + nanotime(0), nanotime(-1))



## ---------- other ops
#test_compare_nanotime_ANY <- function() {
expect_true(nanotime(1) == nanotime(1))
expect_true(nanotime(1) == as.integer64(1))
expect_true(nanotime(1) == 1L)
expect_true(nanotime(1) == 1)
expect_error(is.na(nanotime(1) == "a"))  # now passes on and gets parse error

expect_true(!(nanotime(1) < nanotime(1)))
expect_true(!(nanotime(1) < as.integer64(1)))
expect_true(!(nanotime(1) < 1L))
expect_true(!(nanotime(1) < 1))

#test_compare_ANY_nanotime <- function() {
expect_true(as.integer64(1) == nanotime(1))
expect_true(1L == nanotime(1))
expect_true(1 == nanotime(1))
expect_true(!("a" == is.na(nanotime(1))))

expect_true(!(as.integer64(1) < nanotime(1)))
expect_true(!(1L < nanotime(1)))
expect_true(!(1 < nanotime(1)))

#test_compare_character_nanotime <- function() {
expect_true(isFALSE("2018-12-28T16:34:59.649943448+00:00" < nanotime("2018-12-28T16:34:59.000000000+00:00")))
expect_true("2018-12-28T16:34:59.649943448+00:00" >  nanotime("2018-12-28T16:34:59.000000000+00:00"))
expect_true("2018-12-28T16:34:59.649943448+00:00" == nanotime("2018-12-28T16:34:59.649943448+00:00"))

#test_compare_nanotime_character <- function() {
expect_true(isFALSE(nanotime("2018-12-28T16:34:59.649943448+00:00") < "2018-12-28T16:34:59.000000000+00:00"))
expect_true(nanotime("2018-12-28T16:34:59.649943448+00:00") >  "2018-12-28T16:34:59.000000000+00:00")
expect_true(nanotime("2018-12-28T16:34:59.649943448+00:00") == "2018-12-28T16:34:59.649943448+00:00")

#test_Logic <- function() {
exc <- "operations are possible only for numeric, logical or complex types"
#expect_error(nanotime(1) & nanotime(1), exc)
expect_error(nanotime(1) & as.integer64(1), exc)
expect_error(nanotime(1) & TRUE, exc)

expect_error(as.integer64(1) & nanotime(1), exc)
expect_error(1L & nanotime(1), exc)
expect_error(FALSE & nanotime(1), exc)

#test_Math <- function() {
expect_error(abs(nanotime(1)), "non-numeric argument to mathematical function")

#test_Math2 <- function() {
expect_error(round(nanotime(1), 2), "non-numeric argument to mathematical function")

#test_Summary <- function() {
expect_error(sum(nanotime(1)))#, "invalid 'type' (nanotime) of argument")

expect_identical(min(nanotime(c(1,8,4,2,0,3,10))), nanotime(0))
expect_identical(max(nanotime(c(1,8,4,2,0,3,10))), nanotime(10))
expect_identical(range(nanotime(c(1,8,4,2,0,3,10))), nanotime(c(0, 10)))

#test_Complex <- function() {
expect_error(Arg(nanotime(1)), "non-numeric argument to function")

library(nanotime)

## nanotime
##test_nanotime_data_frame_constructor <- function() {
## simpler data.frame, inserts 'formatted'
set.seed(42)
N <- 300
shine <- nanotime(Sys.time()) + cumsum(10*rpois(n=N+1, lambda=4))
rain  <- nanotime(Sys.time()) + cumsum(10*rpois(n=N+1, lambda=4) + round(runif(N+1)*25))
newdf <- data.frame(rain=rain, shine=shine)
expect_identical(newdf$rain,  rain)
expect_identical(newdf$shine, shine)

##test_nanotime_data_frame_rbind <- function() {
t1 <- nanotime(1:10)
d1 <- 1:10
t2 <- nanotime(11:20)
d2 <- 11:20
df1 <- data.frame(t = t1, d = d1)
df2 <- data.frame(t = t2, d = d2)
df <- rbind(df1, df2)
expect_identical(df, data.frame(t = c(t1, t2), d = c(d1, d2)))

##test_nanotime_data_frame_cbind <- function() {
t1 <- nanotime(1:10)
d1 <- 1:10
t2 <- nanotime(11:20)
d2 <- 11:20
df1 <- data.frame(t1 = t1, d1 = d1)
df2 <- data.frame(t2 = t2, d2 = d2)
df <- cbind(df1, df2)
expect_identical(df, data.frame(t1 = t1, d1 = d1, t2=t2, d2=d2))


## nanoduration
##test_nanoduration_data_frame_constructor <- function() {
## simpler data.frame, inserts 'formatted'
set.seed(42)
N <- 300
shine <- as.nanoduration(1000) + cumsum(10*rpois(n=N+1, lambda=4))
rain  <- as.nanoduration(2000) + cumsum(10*rpois(n=N+1, lambda=4) + round(runif(N+1)*25))
newdf <- data.frame(rain=rain, shine=shine)
expect_identical(newdf$rain,  rain)
expect_identical(newdf$shine, shine)

##test_nanoduration_data_frame_rbind <- function() {
t1 <- as.nanoduration(1:10)
d1 <- 1:10
t2 <- as.nanoduration(11:20)
d2 <- 11:20
df1 <- data.frame(t = t1, d = d1)
df2 <- data.frame(t = t2, d = d2)
df <- rbind(df1, df2)
expect_identical(df, data.frame(t = c(t1, t2), d = c(d1, d2)))


##test_nanoduration_data_frame_cbind <- function() {
t1 <- as.nanoduration(1:10)
d1 <- 1:10
t2 <- as.nanoduration(11:20)
d2 <- 11:20
df1 <- data.frame(t1 = t1, d1 = d1)
df2 <- data.frame(t2 = t2, d2 = d2)
df <- cbind(df1, df2)
expect_identical(df, data.frame(t1 = t1, d1 = d1, t2=t2, d2=d2))


## nanoperiod
##test_nanoperiod_data.frame_constructor  <- function() {
## simpler data.frame, inserts 'formatted'
set.seed(42)
N <- 300
shine <- as.nanoperiod(1000) + cumsum(10*rpois(n=N+1, lambda=4))
rain  <- as.nanoperiod(2000) + cumsum(10*rpois(n=N+1, lambda=4) + round(runif(N+1)*25))
newdf <- data.frame(rain=rain, shine=shine)
expect_identical(newdf$rain,  rain)
expect_identical(newdf$shine, shine)

##test_nanoperiod_data_frame_rbind <- function() {
t1 <- as.nanoperiod(1:10)
d1 <- 1:10
t2 <- as.nanoperiod(11:20)
d2 <- 11:20
df1 <- data.frame(t = t1, d = d1)
df2 <- data.frame(t = t2, d = d2)
df <- rbind(df1, df2)
expect_identical(df, data.frame(t = c(t1, t2), d = c(d1, d2)))

##test_nanoperiod_data_frame_cbind <- function() {
t1 <- as.nanoperiod(1:10)
d1 <- 1:10
t2 <- as.nanoperiod(11:20)
d2 <- 11:20
df1 <- data.frame(t1 = t1, d1 = d1)
df2 <- data.frame(t2 = t2, d2 = d2)
df <- cbind(df1, df2)
expect_identical(df, data.frame(t1 = t1, d1 = d1, t2=t2, d2=d2))


## nanoival
##test_nanoival_data.frame_constructor  <- function() {
## simpler data.frame, inserts 'formatted'
set.seed(42)
N <- 300
shine_start <- nanotime(1000) + cumsum(10*rpois(n=N+1, lambda=4))
shine_end   <- shine_start + as.nanoduration("01:00:00")
rain_start  <- nanotime(2000) + cumsum(10*rpois(n=N+1, lambda=4) + round(runif(N+1)*25))
rain_end    <- shine_end + as.nanoduration("02:00:00")
shine <- nanoival(shine_start, shine_end)
rain  <- nanoival(rain_start, rain_end) 
newdf <- data.frame(rain=rain, shine=shine)
expect_identical(newdf$rain,  rain)
expect_identical(newdf$shine, shine)

##test_nanoival_data_frame_rbind <- function() {
t1 <- nanoival(nanotime(1:10), nanotime(2:11))
d1 <- 1:10
t2 <- nanoival(nanotime(11:20), nanotime(12:21))
d2 <- 11:20
df1 <- data.frame(t = t1, d = d1)
df2 <- data.frame(t = t2, d = d2)
df <- rbind(df1, df2)
expect_identical(df, data.frame(t = c(t1, t2), d = c(d1, d2)))

##test_nanoival_data_frame_cbind <- function() {
t1 <- nanoival(nanotime(1:10), nanotime(2:11))
d1 <- 1:10
t2 <- nanoival(nanotime(11:20), nanotime(12:21))
d2 <- 11:20
df1 <- data.frame(t1 = t1, d1 = d1)
df2 <- data.frame(t2 = t2, d2 = d2)
df <- cbind(df1, df2)
expect_identical(df, data.frame(t1 = t1, d1 = d1, t2=t2, d2=d2))

##' @rdname nanoperiod
setClass("nanoperiod", contains = "complex")

##' Period type with nanosecond precision
##'
##' \code{nanoperiod} is a length of time type (implemented as an S4
##' class) with nanosecond precision. It differs from
##' \code{nanoduration} because it is capable of representing calendar
##' months and days. It can thus represent years (12 months) and weeks
##' (7 days). A period is a somewhat abstract representation of time:
##' it is only when anchored to a point in time and in a specific time
##' zone that it is possible to convert it to a specific
##' duration. This means that many of the operations involving periods
##' need the additional argument \code{tz}.
##'
##' @section Constructors:
##'
##' The true constructor is
##'
##' @section Output Format:
##'
##' A \code{nanoperiod} is displayed as months, days, and \code{nanoduration}
##' like this: \code{10m2d/10:12:34.123_453_000}.
##'
##' @section Details:
##'
##'
##'
##' Adding or subtracting \code{nanoperiod} and \code{nanotime}
##' require a timezone as third argument. For this reason it is not
##' possible to use the binary operator `\code{+}`. Instead the
##' functions `\code{plus}` and `\code{minus}` are defined. These
##' functions attempt to keep the same offset within a day in the
##' specified timezone: this means for instance that adding a day when
##' that day crosses a time zone adjustment such as a daylight saving
##' time, results in a true time increment of less or more than 24
##' hours to preserve the offset. Preserving the offset works for
##' increments that are smaller than a day too, provided the increment
##' results in a datetime where the timezone adjustment is valid. When
##' this is not the case, adding a `nanoperiod` behaves in the same
##' way as adding a `nanoduration`.
##'
##'
##' @param x,value An object of class \code{nanoperiod}
##' @param ... further arguments
##' @param e1 Operand of class \code{nanoperiod}
##' @param e2 Operand of class \code{nanoperiod}
##' @param object argument for method \code{show}
##' @param i index specifying elements to extract or replace.
##' @param j Required for \code{[} signature but ignored here
##' @param drop Required for \code{[} signature but ignored here
##' @param months Used in the constructor to indicate the number of
##'     months of the \code{nanoperiod}
##' @param days Used in the constructor to indicate the number of
##'     days of the \code{nanoperiod}
##' @param duration Used in the constructor to indicate the duration
##'     component of the \code{nanoperiod}
##' @param quote indicates if the output of \code{print} should be
##'     quoted
##' @param tz \code{character} indicating a timezone
##' @author Dirk Eddelbuettel
##' @author Leonardo Silvestri
##' @examples
##' \dontrun{
##' p <- nanoperiod(months=12, days=7, duration="01:00:00")
##' print(p)
##'
##' #  when adding a \code{nanoperiod} to a \code{nanotime} or to a
##' # \code{nanoival}, a time zone must be specified:
##' y <- nanotime("1970-01-01T00:00:00+00:00")
##' plus(y, p, tz="America/Chicago")
##' }
##'
##' @seealso \code{\link{nanotime}}, \code{\link{nanoduration}},
##'     \code{\link{nanoival}}, \code{\link{nanoperiod.month,nanoperiod-method}}
##'
##' @aliases Compare,ANY,nanoperiod-method Compare,nanoperiod,ANY-method
##' @aliases -,ANY,nanoperiod-method -,nanoperiod,nanotime-method
##' @aliases -,nanotime,nanoperiod-method
##' @aliases /,ANY,nanoperiod-method /,nanoperiod,ANY-method
##' @aliases Complex,nanoperiod-method Math,nanoperiod-method Math2,nanoperiod-method
##' @aliases Summary,nanoperiod-method
##' @aliases minus,nanoperiod,nanoival,character-method
##'
##' @rdname nanoperiod
nanoperiod <- function(months=0, days=0, duration=as.nanoduration(0)) {
    if (nargs() == 0) {
        as.nanoperiod(NULL)
    } else {
        if (!is.numeric(months)) {
            stop("argument 'months' must be numeric")
        }
        if (!is.numeric(days)) {
            stop("argument 'days' must be numeric")
        }        
        period_from_parts_impl(as.integer(months), as.integer(days), as.nanoduration(duration))
    }
}

##' @noRd
setGeneric("nanoperiod")


##' @noRd
setGeneric("as.nanoperiod", function(x) standardGeneric("as.nanoperiod"))

##' @rdname nanoperiod
##' @aliases as.nanoperiod
setMethod("as.nanoperiod",
          "character",
          function(x) {
              period_from_string_impl(x)
          })

setAs("character", "nanoperiod", function(from) as.nanoperiod(from))

##' @rdname nanoperiod
setMethod("as.nanoperiod",
          "integer64",
          function(x) {
              period_from_integer64_impl(x)
          })

setAs("integer64", "nanoperiod", function(from) as.nanoperiod(from))

##' @rdname nanoperiod
setMethod("as.nanoperiod",
          "numeric",
          function(x) {
              period_from_double_impl(x)
          })

setAs("numeric", "nanoperiod", function(from) as.nanoperiod(from))

##' @rdname nanoperiod
setMethod("as.nanoperiod",
          "integer",
          function(x) {
              period_from_integer_impl(x)
          })

setAs("integer", "nanoperiod", function(from) as.nanoperiod(from))

##' @rdname nanoperiod
setMethod("as.nanoperiod",
          "nanoduration",
          function(x) {
              period_from_integer64_impl(x)
          })

setAs("nanoduration", "nanoperiod", function(from) as.nanoperiod(from))

##' @rdname nanoperiod
setMethod("as.nanoperiod",
          "NULL",
          function(x) {
              period_from_string_impl(character())
          })

##' @rdname nanoperiod
setMethod("as.nanoperiod",
          "missing",
          function(x) {
              period_from_string_impl(character())
          })


##' @rdname nanoperiod
setMethod("show",
          signature("nanoperiod"),
          function(object) print(object))

##' @rdname nanoperiod
setMethod("print",
          "nanoperiod",
          function(x, quote=FALSE, ...) {
              if (length(x)==0) {
                  print("nanoperiod(0)", quote=quote)
              } else {
                  print(period_to_string_impl(x), quote=quote)
              }
          })

##' @rdname nanoperiod
format.nanoperiod <- function(x, ...) {
  period_to_string_impl(x)
}

##' @rdname nanoperiod
setMethod("as.character",
          signature("nanoperiod"),
          function(x) {
              period_to_string_impl(x)
          })

##' @rdname nanoperiod
setMethod("is.na",
          "nanoperiod",
          function(x) {
              period_isna_impl(x)
          })

##' @rdname nanoperiod
setMethod("is.na<-",
          "nanoperiod",
          function(x, value) {
              x[value] <- NA_nanoperiod_
              x
          })

## accessors

## ----------- non ops

##' @rdname nanoperiod
setMethod("[[",
          signature("nanoperiod", "ANY"),
          function (x, i, j, ..., drop=FALSE) {
              new("nanoperiod", callNextMethod())
          })

##' @rdname nanoperiod
setMethod("[",
          signature("nanoperiod", "numeric"),
          function (x, i, j, ..., drop=FALSE) {
              if (!missing(j) || length(list(...)) > 0) {
                  warning("unused indices or arguments in 'nanoperiod' subsetting")
              }
              if (isTRUE(any(i < 0))) {
                  new("nanoperiod", unclass(x)[i])
              } else {
                  period_subset_numeric_impl(x, i)
              }
          })

##' @rdname nanoperiod
setMethod("[",
          signature("nanoperiod", "logical"),
          function (x, i, j, ..., drop=FALSE) {
              if (!missing(j) || length(list(...)) > 0) {
                  warning("unused indices or arguments in 'nanoperiod' subsetting")
              }
              period_subset_logical_impl(x, i)
          })

##' @rdname nanoperiod
setMethod("[",
          signature("nanoperiod", "character"),
          function (x, i, j, ..., drop=FALSE) {
              ## we don't implement 'period_subset_character_impl' but
              ## do the gymnastic of finding the NAs here at R level;
              ## it's not as efficient, but using 'character' indexing
              ## is by itself inefficient, so the overhead should have
              ## no practical consequences.
              if (!missing(j) || length(list(...)) > 0) {
                  warning("unused indices or arguments in 'nanoperiod' subsetting")
              }
              na_index <- !(i %in% names(x)) | is.na(i)
              res <- new("nanoperiod", unclass(x)[i])
              res[na_index] <- NA_nanoperiod_
              res
          })

##' @rdname nanoperiod
setMethod("[",
          signature("nanoperiod", "ANY"),
          function (x, i, j, ..., drop=FALSE) {
              stop("']' not defined on 'nanoperiod' for index of type 'ANY'")
          })

##' @rdname nanoperiod
setMethod("[<-",
          signature("nanoperiod", "ANY", "ANY", "ANY"),
          function (x, i, j, ..., value) {
              new("nanoperiod", callNextMethod())
          })

##' @rdname nanoperiod
c.nanoperiod <- function(...) {
    args <- list(...)
    s3args <- lapply(args, function (x) S3Part(x, strictS3=TRUE))
    res <- do.call(c, s3args)
    names(res) <- names(args)
    new("nanoperiod", res)
}

##' @noRd
setGeneric("nanoperiod.month", function(x) standardGeneric("nanoperiod.month"))

##' Nanoperiod accessors
##'
##' These functions allow access to the components of a \code{nanoperiod}
##'
##' @rdname nanoperiod.month
##' @aliases nanoperiod.month nanoperiod.day nanoperiod.nanoduration
##' @param x A \code{nanoperiod}
##' @return \code{nanoperiod.month} and \code{nanoperiod.day} return
##'     an \code{integer64} whereas \code{nanoperiod.nanoduration}
##'     returns a \code{nanoduration}
##' @examples
##' p <- as.nanoperiod("2y1m1d/12:00:00")
##' nanoperiod.month(p)
##' nanoperiod.day(p)
##' nanoperiod.nanoduration(p)
##' @author Dirk Eddelbuettel
##' @author Leonardo Silvestri
##' @seealso \code{\link{nanoduration}}
setMethod("nanoperiod.month",
          "nanoperiod",
          function(x) {
              period_month_impl(x)
          })

##' @noRd
setGeneric("nanoperiod.day", function(x) standardGeneric("nanoperiod.day"))
##' @rdname nanoperiod.month
setMethod("nanoperiod.day",
          "nanoperiod",
          function(x) {
              period_day_impl(x)
          })

##' @noRd
setGeneric("nanoperiod.nanoduration", function(x) standardGeneric("nanoperiod.nanoduration"))
##' @rdname nanoperiod.month
setMethod("nanoperiod.nanoduration",
          "nanoperiod",
          function(x) {
              period_duration_impl(x)
          })

##' @rdname nanoperiod
setMethod("names",
          signature("nanoperiod"),
          function(x) {
              callNextMethod()
          })

##' @rdname nanoperiod
setMethod("names<-",
          signature("nanoperiod"),
          function(x, value) {
              names(S3Part(x, strictS3=TRUE)) <- value
              new("nanoperiod", x)
          })


## ----------- make sure ops that don't make sense error out
##' @noRd
setMethod("Ops", c("nanoperiod", "ANY"),
          function(e1, e2) {
              stop("operation not defined for 'nanoperiod' objects")
          })
##' @noRd
setMethod("Ops", c("ANY", "nanoperiod"),
          function(e1, e2) {
              stop("operation not defined for 'nanoperiod' objects")
          })

##' @noRd
setMethod("Math", c("nanoperiod"),
          function(x) {
              stop("operation not defined for 'nanoperiod' objects")
          })

##' @noRd
setMethod("Math2", c("nanoperiod"),
          function(x, digits) {
              stop("operation not defined for 'nanoperiod' objects")
          })

##' @noRd
setMethod("Summary", c("nanoperiod"),
          function(x, ..., na.rm = FALSE) {
              stop("invalid 'type' (nanoperiod) of argument")
          })

##' @noRd
setMethod("Complex", c("nanoperiod"),
          function(z) {
              stop("operation not defined for 'nanoperiod' objects")
          })

## ------------ `-`
##' @rdname nanoperiod
setMethod("-", c("nanoperiod", "ANY"),
          function(e1, e2) {
              if (missing(e2)) {
                  minus_period_impl(e1)
              }
              else {
                  stop("invalid operand types")
              }
          })

##' @rdname nanoperiod
setMethod("-", c("nanoperiod", "nanoperiod"),
          function(e1, e2) {
              minus_period_period_impl(e1, e2)
          })

## --

##' @rdname nanoperiod
##' @aliases -,nanoperiod,ANY-method
setMethod("-", c("nanoperiod", "nanoduration"),
          function(e1, e2) {
              minus_period_integer64_impl(e1, e2)
          })

##' @rdname nanoperiod
setMethod("-", c("nanoperiod", "integer64"),
          function(e1, e2) {
              minus_period_integer64_impl(e1, e2)
          })

##' @noRd
setMethod("-", c("nanotime", "nanoperiod"),
          function(e1, e2) {
              stop(paste0("binary '-' is not defined for 'nanotime' and 'nanoperiod' objects; instead use 'minus(e1, e2, tz)'"))
          })

##' @noRd
setMethod("-", c("nanoperiod", "nanotime"),
          function(e1, e2) {
              stop("invalid operand types")
          })

##' @rdname nanoperiod
setMethod("-", c("nanoperiod", "numeric"),
          function(e1, e2) {
              minus_period_integer64_impl(e1, as.integer64(e2))
          })

## --
##' @noRd
setMethod("-", c("ANY", "nanoperiod"),
          function(e1, e2) {
              stop("invalid operand types")
          })

## --
##' @rdname nanoperiod
setMethod("-", c("nanoduration", "nanoperiod"),
          function(e1, e2) {
              minus_integer64_period_impl(e1, e2)
          })

## --
##' @rdname nanoperiod
setMethod("-", c("integer64", "nanoperiod"),
          function(e1, e2) {
              minus_integer64_period_impl(as.integer64(e1), e2)
          })

## --
##' @rdname nanoperiod
setMethod("-", c("numeric", "nanoperiod"),
          function(e1, e2) {
              minus_integer64_period_impl(as.integer64(e1), e2)
          })


## ----------- `+`

##' @rdname nanoperiod
##' @aliases +,ANY,nanoperiod-method
setMethod("+", c("nanoperiod", "ANY"),
          function(e1, e2) {
              if (missing(e2)) {
                  e1
              }
              else {
                  stop("invalid operand types")
              }
          })

##' @rdname nanoperiod
setMethod("+", c("nanoperiod", "nanoperiod"),
          function(e1, e2) {
              plus_period_period_impl(e1, e2)
          })

## --
##' @noRd
setMethod("+", c("ANY", "nanoperiod"),
          function(e1, e2) {
              stop("invalid operand types")
          })

##' @rdname nanoperiod
setMethod("+", c("nanoperiod", "nanoduration"),
          function(e1, e2) {
              plus_period_integer64_impl(e1, S3Part(e2, strictS3=TRUE))
          })

##' @rdname nanoperiod
setMethod("+", c("nanoperiod", "integer64"),
          function(e1, e2) {
              plus_period_integer64_impl(e1, e2)
          })

##' @rdname nanoperiod
setMethod("+", c("nanoperiod", "nanotime"),
          function(e1, e2) {
              stop(paste0("binary '+' is not defined for 'nanoperiod' and 'nanotime' objects; instead use 'plus(e1, e2, tz)'"))
          })

##' @rdname nanoperiod
setMethod("+", c("nanoival", "nanoperiod"),
          function(e1, e2) {
              stop(paste0("binary '+' is not defined for 'nanoival' and 'nanoperiod' objects; instead use 'plus(e1, e2, tz)'"))
          })

##' @rdname nanoperiod
setMethod("+", c("nanoperiod", "nanoival"),
          function(e1, e2) {
              stop(paste0("binary '+' is not defined for 'nanoperiod' and 'nanoival' objects; instead use 'plus(e1, e2, tz)'"))
          })

##' @rdname nanoperiod
setMethod("+", c("nanotime", "nanoperiod"),
          function(e1, e2) {
              stop(paste0("binary '+' is not defined for 'nanotime' and 'nanoperiod' objects; instead use 'plus(e1, e2, tz)'"))
          })

##' @rdname nanoperiod
setMethod("+", c("nanoperiod", "numeric"),
          function(e1, e2) {
              plus_period_integer64_impl(e1, as.integer64(e2))
          })

## --
##' @rdname nanoperiod
setMethod("+", c("nanoduration", "nanoperiod"),
          function(e1, e2) {
              plus_period_integer64_impl(e2, e1)
          })

##' @rdname nanoperiod
setMethod("+", c("integer64", "nanoperiod"),
          function(e1, e2) {
              plus_period_integer64_impl(e2, e1)
          })

##' @rdname nanoperiod
setMethod("+", c("numeric", "nanoperiod"),
          function(e1, e2) {
              plus_period_integer64_impl(e2, as.integer64(e1))
          })

## ----------- `*`

##' @noRd
setMethod("*", c("ANY", "nanoperiod"),
          function(e1, e2) {
              stop("invalid operand types")
          })

##' @noRd
setMethod("*", c("nanoperiod", "ANY"),
          function(e1, e2) {
              stop("invalid operand types")
          })

##' @rdname nanoperiod
##' @aliases *,nanoperiod,ANY-method *,ANY,nanoperiod-method
setMethod("*", c("nanoperiod", "integer64"),
          function(e1, e2) {
              multiplies_period_integer64_impl(e1, e2)
          })

##' @rdname nanoperiod
setMethod("*", c("nanoperiod", "numeric"),
          function(e1, e2) {
              multiplies_period_double_impl(e1, e2)
          })

##' @rdname nanoperiod
setMethod("*", c("integer64", "nanoperiod"),
          function(e1, e2) {
              multiplies_period_integer64_impl(e2, e1)
          })

##' @rdname nanoperiod
setMethod("*", c("numeric", "nanoperiod"),
          function(e1, e2) {
              multiplies_period_double_impl(e2, e1)
          })

## ----------- `/`

##' @rdname nanoperiod
##' @noRd
setMethod("/", c("ANY", "nanoperiod"),
          function(e1, e2) {
              stop("invalid operand types")
          })

##' @rdname nanoperiod
##' @noRd
setMethod("/", c("nanoperiod", "ANY"),
          function(e1, e2) {
              stop("invalid operand types")
          })

##' @rdname nanoperiod
setMethod("/", c("nanoperiod", "integer64"),
          function(e1, e2) {
              divides_period_integer64_impl(e1, e2)
          })

##' @rdname nanoperiod
setMethod("/", c("nanoperiod", "numeric"),
          function(e1, e2) {
              divides_period_double_impl(e1, e2)
          })

## Compare
## -------

##' @noRd
setMethod("Compare", c("ANY", "nanoperiod"),
          function(e1, e2) {
              stop("operation not defined for 'nanoperiod' objects")
          })
##' @noRd
setMethod("Compare", c("nanoperiod", "ANY"),
          function(e1, e2) {
              stop("operation not defined for 'nanoperiod' objects")
          })

##' @rdname nanoperiod
setMethod("==", c("nanoperiod", "nanoperiod"), function(e1, e2) eq_period_period_impl(e1, e2))

##' @rdname nanoperiod
setMethod("!=", c("nanoperiod", "nanoperiod"), function(e1, e2) ne_period_period_impl(e1, e2))


## ---------- plus/minus ops with nanotime and nanoperiod (which require 'tz')

##' @noRd
setGeneric("plus",  function(e1, e2, tz) standardGeneric("plus"))
##' @noRd
setGeneric("minus", function(e1, e2, tz) standardGeneric("minus"))


##' @rdname nanoperiod
##' @aliases plus
setMethod("plus", c("nanotime", "nanoperiod", "character"),
          function(e1, e2, tz) {
            plus_nanotime_period_impl(e1, e2, tz)
          })

##' @rdname nanoperiod
setMethod("plus", c("nanoperiod", "nanotime", "character"),
          function(e1, e2, tz) {
            plus_nanotime_period_impl(e2, e1, tz)
          })


##' @rdname nanoperiod
##' @aliases minus
setMethod("minus", c("nanotime", "nanoperiod", "character"),
          function(e1, e2, tz) {
            minus_nanotime_period_impl(e1, e2, tz)
          })

##' @rdname nanoperiod
setMethod("minus", c("nanoperiod", "nanotime", "character"),
          function(e1, e2, tz) {
            stop("operation not defined for 'nanoperiod' objects")
          })


## ---------- plus/minus ops with nanoival and nanoperiod (which require 'tz')


##' @rdname nanoperiod
setMethod("plus", c("nanoival", "nanoperiod", "character"),
          function(e1, e2, tz) {
            plus_nanoival_period_impl(e1, e2, tz)
          })

##' @rdname nanoperiod
setMethod("plus", c("nanoperiod", "nanoival", "character"),
          function(e1, e2, tz) {
            plus_nanoival_period_impl(e2, e1, tz)
          })

##' @rdname nanoperiod
setMethod("minus", c("nanoival", "nanoperiod", "character"),
          function(e1, e2, tz) {
            minus_nanoival_period_impl(e1, e2, tz)
          })

##' @rdname nanoperiod
##' @noRd
setMethod("minus", c("nanoperiod", "nanoival", "character"),
          function(e1, e2, tz) {
            stop("operation not defined for 'nanoperiod' objects")
          })


##' Test if Two Objects are (Nearly) Equal
##'
##' Compare \code{target} and \code{current} testing \sQuote{near
##' equality}.  If they are different, comparison is still made to
##' some extent, and a report of the differences is returned.  Do not
##' use \code{all.equal} directly in \code{if} expressions---either
##' use \code{isTRUE(all.equal(....))} or \code{\link{identical}} if
##' appropriate.
##'
##' @param target,current \code{nanoperiod} arguments to be compared
##' @param ... further arguments for different methods
##' @param check.attributes logical indicating if the
##'     \code{attributes} of \code{target} and \code{current} (other than
##'     the names) should be compared.
##'
##' @seealso \code{\link{identical}}, \code{\link{isTRUE}},
##'     \code{\link{==}}, and \code{\link{all}} for exact equality
##'     testing.
##'
##' @method all.equal nanoperiod
##'
all.equal.nanoperiod <-
    function(target, current, ..., check.attributes = TRUE) all.equal.raw(target, current, ..., check.attributes)


##' @rdname all.equal.nanoperiod
setMethod("all.equal", c(target = "nanoperiod", current="ANY"), all.equal.nanoperiod)


##' @rdname nanoperiod
NA_nanoperiod_ <- new("nanoperiod", complex(1, -1.0609978954826362e-314, 0))


##' @rdname rounding
##' @param origin a \code{nanotime} scalar indicating the origin at which the rounding is considered
##' @param tz a \code{character} scalar indicating the time zone in which to conduct the rounding
setMethod("nano_ceiling", c(x="nanotime", precision="nanoperiod"),
          function(x, precision, origin=nanotime(), tz) {
              if (!inherits(origin, "nanotime")) {
                  stop("'origin' must be of class 'nanotime'")
              }
              if (!is.character(tz)) {
                  stop("'tz' must be of type 'character'")
              }
              if (is.unsorted(x)) {
                  stop("'x' must be sorted")
              }
              ceiling_tz_impl(x, precision, origin, tz)
          })

##' @rdname rounding
setMethod("nano_floor",   c(x="nanotime", precision="nanoperiod"),
          function(x, precision, origin=nanotime(), tz) {
              if (!inherits(origin, "nanotime")) {
                  stop("'origin' must be of class 'nanotime'")
              }
              if (!is.character(tz)) {
                  stop("'tz' must be of type 'character'")
              }
              if (is.unsorted(x)) {
                  stop("'x' must be sorted")
              }
              floor_tz_impl(x, precision, origin, tz)
          })


##' Replicate Elements
##'
##' Replicates the values in 'x' similarly to the default method.
##'
##' @param x a vector of \code{nanoperiod}
##' @param ... further arguments:
##' 
##'        'times' an integer-valued vector giving the (non-negative)
##'            number of times to repeat each element if of length
##'            'length(x)', or to repeat the whole vector if of length
##'            1. Negative or 'NA' values are an error. A 'double'
##'            vector is accepted, other inputs being coerced to an
##'            integer or double vector.
##' 
##'        'length.out' non-negative integer. The desired length of the
##'            output vector. Other inputs will be coerced to a double
##'            vector and the first element taken. Ignored if 'NA' or
##'            invalid.
##' 
##'        'each' non-negative integer. Each element of 'x' is repeated
##'            'each' times.  Other inputs will be coerced to an integer
##'            or double vector and the first element taken. Treated as
##'            '1' if 'NA' or invalid.
setMethod("rep", c(x = "nanoperiod"), function(x, ...) {
    new("nanoperiod", callNextMethod())
})

## TODO:
## - prevent matrix, array, cbind, rbind, compared with non-interval
##



##' @rdname nanoival
setClass("nanoival", contains="complex")

##' Interval type with nanosecond precision
##'
##' \code{nanoival} is a time interval type (an S4 class) with
##' nanosecond precision. One of its purposes is to allow quick
##' subsetting of a \code{nanotime} vector. \code{nanoival} is
##' composed of a \code{nanotime} pair which defines the start and end
##' of the time interval. Additionally, it has a pair of logical
##' values which determine if the start and end of the time interval
##' are open (true) or closed (false).
##'
##' An interval object can be constructed with the constructor
##' \code{nanoival} which takes as arguments two \code{nanotime}
##' objects that define the start and the end of the interval,
##' together with two \code{logical} arguments that define if the
##' start and the end of the interval are open (true) or closed
##' (false) (note that these objects can all be vector, and therefore
##' the interval object is not necessarily scalar). Alternatively, an
##' interval can be constructed with a \code{character}: the format
##' follows that of \code{nanotime}; the start time is preceeded by
##' either \code{-} or \code{+} indicating if the interval start is
##' open (-) or closed (+); the start and end times are separated by
##' an arrow \code{->}; the end is folloed by either \code{-} or
##' \code{+} which have the same semantics as the start time.
##'
##' The most important set of methods defined for \code{interval} are
##' set functions \code{intersect}, \code{union} and \code{setdiff}.
##'
##' Additionally, \code{interval} allows the subsetting into a
##' \code{nanotime} vector. Note that subsetting is allowed only if
##' the \code{nanotime} vector is sorted.
##'
##' Finally, accessors are provided to get the interval start
##' (\code{start}), the end (\code{end}), the open/close status of the
##' start (\code{sopen}) and the open/close status of the end
##' (\code{eopen}). The former return a \code{nanotime} while the
##' latter return a \code{logical}.
##'
##' @section Output Format:
##'
##' Formatting and character conversion for \code{nanoival} objects is
##' identical to \code{nanotime} objects. The default format is
##' ISO3339 compliant: \code{\%Y-\%m-\%dT\%H:\%M:\%E9S\%Ez}. It
##' specifies a standard ISO 8601 part for date and time --- as well
##' as nine digits of precision for fractional seconds (down to
##' nanoseconds) and on offset (typically zero as we default to UTC).
##' It can be overriden by using \code{options()} with the key of
##' \code{nanotimeFormat} and a suitable value. Similarly,
##' \code{nanotimeTz} can be used to select a different timezone.
##'
##' @param x,from a \code{nanoival} object
##' @param tz \code{character} indicating a timezone
##' @param ... further arguments passed to or from methods.
##' @param e1 Operand of class \code{nanoival}
##' @param e2 Operand of class \code{nanoival}
##' @param format A character string. Can also be set via
##'     \code{options("nanotimeFormat")} and uses
##'     \sQuote{\%Y-\%m-\%dT\%H:\%M:\%E9S\%Ez} as a default and
##'     fallback
##' @param object argument for method \code{show}
##' @param i index specifying elements to extract or replace.
##' @param j Required for \code{[} signature but ignored here
##' @param drop Required for \code{[} signature but ignored here
##' @param value argument for \code{nanoival-class}
##' @param start \code{nanotime} start of interval
##' @param end \code{nanotime} end of interval
##' @param sopen logical indicating if the start of the interval is open
##' @param eopen logical indicating if the end of the interval is open
##' @param quote indicates if the output of \code{print} should be
##'     quoted
##' @return A nanoival object
##' @author Dirk Eddelbuettel
##' @author Leonardo Silvestri
##' @examples
##' \dontrun{
##' ## creating a \code{nanoival}, with the start time included ('+') and the end
##' ## time excluded ('-')
##' as.nanoival("+2012-03-01T21:21:00.000000001+00:00->2015-01-01T21:22:00.000000999+04:00-")
##'
##' ## a \code{nanoival} can also be created with a pair of \code{nanotime} objects, a start
##' ## and an end, and optionally two logicals determining if the interval start(end) are open
##' ## or closed; by default the start is closed and end is open:
##' start <- nanotime("2012-03-01T21:21:00.000000001+00:00")
##' end <- nanotime("2013-03-01T21:21:00.000000001+00:00")
##' nanoival(start, end)
##'
##' ## a vector of 'nanotime' can be subsetted by a 'nanoival':
##' one_second <- 1e9
##' a <- seq(nanotime("2012-12-12 12:12:12+00:00"), length.out=10, by=one_second)
##' idx <- c(as.nanoival("-2012-12-12 12:12:10+00:00 -> 2012-12-12 12:12:14+00:00-"),
##'          as.nanoival("+2012-12-12 12:12:18+00:00 -> 2012-12-12 12:12:20+00:00+"))
##' a[idx]
##' }
##' @aliases +,ANY,nanoival-method
##' @aliases +,nanoival,ANY-method
##' @aliases +,nanoival,nanoival-method
##' @aliases -,ANY,nanoival-method
##' @aliases -,nanoival,ANY-method
##' @aliases -,nanoival,nanoival-method
##' @aliases Arith,nanoival,ANY-method
##' @aliases Compare,nanoival,ANY-method
##' @aliases Complex,nanoival-method
##' @aliases Logic,ANY,nanoival-method
##' @aliases Logic,nanoival,ANY-method
##' @aliases Logic,nanoival,nanoival-method
##' @aliases Math2,nanoival-method
##' @aliases Math,nanoival-method
##' @aliases Summary,nanoival-method
##'
##' @seealso \code{\link{intersect.idx}}, \code{\link{setdiff.idx}},
##' @rdname nanoival
nanoival <- function(start, end, sopen=FALSE, eopen=TRUE) {
    if (nargs() == 0) {
        new("nanoival", as.complex(NULL))
    } else {
        nanoival_new_impl(as.integer64(start), as.integer64(end),
                          as.logical(sopen), as.logical(eopen))
    }
}

##' @noRd
setGeneric("nanoival.start", function(x) standardGeneric("nanoival.start"))

##' @rdname nanoival
##' @aliases nanoival.start
setMethod("nanoival.start",
          "nanoival",
          function(x) {
            res <- nanoival_get_start_impl(x)
            oldClass(res)  <- "integer64"
            new("nanotime", res)
          })

##' @noRd
setGeneric("nanoival.end", function(x) standardGeneric("nanoival.end"))

##' @rdname nanoival
##' @aliases nanoival.end
setMethod("nanoival.end",
          "nanoival",
          function(x) {
            res <- nanoival_get_end_impl(x)
            oldClass(res)  <- "integer64"
            new("nanotime", res)
          })

##' @noRd
setGeneric("nanoival.sopen", function(x) standardGeneric("nanoival.sopen"))

##' @rdname nanoival
##' @aliases nanoival.sopen
setMethod("nanoival.sopen",
          "nanoival",
          function(x) { nanoival_get_sopen_impl(x) })

##' @noRd
setGeneric("nanoival.eopen", function(x) standardGeneric("nanoival.eopen"))

##' @rdname nanoival
##' @aliases nanoival.eopen
setMethod("nanoival.eopen",
          "nanoival",
          function(x) { nanoival_get_eopen_impl(x) })

##' @rdname nanoival
format.nanoival <-
  function(x, ...) {
    if (length(x) == 0) {
      "nanoival(0)"
    } else {
      s  <- paste0(ifelse(nanoival.sopen(x), "-", "+"),
                   format(nanoival.start(x), ...), " -> ",
                   format(nanoival.end(x), ...),
                   ifelse(nanoival.eopen(x), "-", "+"))
      if (!is.null(attr(x, "names", exact=TRUE))) {
        names(s) <- names(x)
      }
      s[nanoival_isna_impl(x)] = NA_character_
      s
    }
  }

##' @rdname nanoival
setMethod("print",
          "nanoival",
          function(x, quote=FALSE, ...) {
              s <- format(x, ...)
              max.print <- options()$max.print
              if (length(x) > max.print) {                   		## #nocov start
                  f <- head(x, max.print)
                  print(f, quote=quote)
                  cat(paste(' [ reached getOption("max.print") -- omitted',
                            length(x) - max.print, "entries ]\n"))
              }                                              		## #nocov end
              else {
                  print(s, quote=quote)
              }
              invisible(s)
          })

##' @rdname nanoival
setMethod("show",
          signature("nanoival"),
          function(object) print(object))


.secondaryNanoivalParse <- function(x, format="", tz="") {
  format <- .getFormat(format)
  tz <- .getTz(x, tz)
  ## parse the +-, split on -> and process the two
  s1 = substr(x, 1, 1)
  if (any(s1 != '-' & s1 != '+')) {
    stop("`nanoival` must start with '+' or '-'")
  }
  sopen <- s1 == "-"
  xlen <- nchar(x)
  e1 <- substr(x, xlen, xlen)
  if (any(e1 != '-' & e1 != '+')) {
    stop("`nanoival` must end with '+' or '-'")
  }
  eopen <- e1 == "-"
  start_stop <- strsplit(substr(x, 2, xlen-1), "->")
  start <- sapply(start_stop, function(x) head(x, 1))
  end <- sapply(start_stop, function(x) tail(x, 1))
  res <- nanoival(nanotime(start, format, tz),
                  nanotime(end,   format, tz),
                  sopen,
                  eopen)
  names(res) <- names(x)
  res
}


##' @noRd
setGeneric("as.nanoival", function(from, format="", tz="") standardGeneric("as.nanoival"))

##' @rdname nanoival
##' @aliases as.nanoival
setMethod("as.nanoival",
          "character",
          function(from, format="", tz="") {
              if (!is.character(tz)) {
                  stop("argument 'tz' must be of type 'character'")
              }
              tryCatch(nanoival_make_impl(from, tz), error=function(e) {
                  if (grepl("Cannot retrieve timezone", e$message)) {
                      stop(e$message)
                  } else {
                      .secondaryNanoivalParse(from, format, tz)
                  }
              })
          })

setAs("character", "nanoival", function(from) as.nanoival(from))


##' @rdname nanoival
setMethod("as.nanoival",
          "NULL",
          function(from, format="", tz="") {
              new("nanoival", as.complex(NULL))
          })

##' @rdname nanoival
setMethod("as.nanoival",
          "missing",
          function(from, format="", tz="") {
              new("nanoival", as.complex(NULL))
          })

##' @rdname nanoival
setMethod("is.na",
          "nanoival",
          function(x) {
              nanoival_isna_impl(x)
          })


##' @rdname nanoival
setMethod("is.na<-",
          "nanoival",
          function(x, value) {
              x[value] <- NA_nanoival_
              x
          })

## ------------ logical comp

##' @rdname nanoival
setMethod("<", c("nanoival", "nanoival"),
          function(e1, e2) {
              nanoival_lt_impl(e1, e2)
          })

##' @rdname nanoival
setMethod("<=", c("nanoival", "nanoival"),
          function(e1, e2) {
              nanoival_le_impl(e1, e2)
          })

##' @rdname nanoival
setMethod(">", c("nanoival", "nanoival"),
          function(e1, e2) {
              nanoival_gt_impl(e1, e2)
          })

##' @rdname nanoival
setMethod(">=", c("nanoival", "nanoival"),
          function(e1, e2) {
              nanoival_ge_impl(e1, e2)
          })

##' @rdname nanoival
setMethod("==", c("nanoival", "nanoival"),
          function(e1, e2) {
              nanoival_eq_impl(e1, e2)
          })

##' @rdname nanoival
setMethod("!=", c("nanoival", "nanoival"),
          function(e1, e2) {
              nanoival_ne_impl(e1, e2)
          })


## ------------ `-`
##' @noRd
setMethod("-", c("nanoival", "ANY"),
          function(e1, e2) {
              stop("invalid operand types")
          })


##' @rdname nanoival
setMethod("-", c("nanoival", "integer64"),
          function(e1, e2) {
              new("nanoival", nanoival_minus_impl(e1, e2))
          })

##' @rdname nanoival
setMethod("-", c("nanoival", "numeric"),
          function(e1, e2) {
              new("nanoival", nanoival_minus_impl(e1, as.integer64(e2)))
          })

##' @noRd
setMethod("-", c("ANY", "nanoival"),
          function(e1, e2) {
              stop("invalid operand types")
          })

##' @noRd
setMethod("-", c("nanoival", "nanoival"),
          function(e1, e2) {
              stop("invalid operand types")
          })


## ----------- `+`
##' @noRd
setMethod("+", c("nanoival", "nanoival"),
          function(e1, e2) {
              stop("invalid operand types")
          })

##' @noRd
setMethod("+", c("nanoival", "ANY"),
          function(e1, e2) {
              stop("invalid operand types")
          })

##' @rdname nanoival
setMethod("+", c("nanoival", "integer64"),
          function(e1, e2) {
              new("nanoival", nanoival_plus_impl(e1, e2))
          })

##' @rdname nanoival
setMethod("+", c("nanoival", "numeric"),
          function(e1, e2) {
              new("nanoival", nanoival_plus_impl(e1, as.integer64(e2)))
          })

##' @noRd
setMethod("+", c("ANY", "nanoival"),
          function(e1, e2) {
              stop("invalid operand types")
          })

##' @rdname nanoival
setMethod("+", c("integer64", "nanoival"),
          function(e1, e2) {
              new("nanoival", nanoival_plus_impl(e2, e1))
          })

##' @rdname nanoival
setMethod("+", c("numeric", "nanoival"),
          function(e1, e2) {
              new("nanoival", nanoival_plus_impl(e2, as.integer64(e1)))
          })

##' @noRd
setMethod("+", c("nanoival", "nanoival"),
          function(e1, e2) {
              stop("invalid operand types")
          })


## ---------- other ops

##' @noRd
setMethod("Arith", c("nanoival", "ANY"),
          function(e1, e2) {
              stop("operation not defined for \"nanoival\" objects")
          })

##' @noRd
setMethod("Compare", c("nanoival", "ANY"),
          function(e1, e2) {
              stop("invalid operand types")
          })


##' @noRd
setMethod("Logic", c("nanoival", "nanoival"),
          function(e1, e2) {
              ## this is the same error message that R gives for "A" | "A"
              stop("operations are possible only for numeric, logical or complex types")
          })

##' @noRd
setMethod("Logic", c("nanoival", "ANY"),
          function(e1, e2) {
              ## this is the same error message that R gives for "A" | "A"
              stop("operations are possible only for numeric, logical or complex types")
          })

##' @noRd
setMethod("Logic", c("ANY", "nanoival"),
          function(e1, e2) {
              ## this is the same error message that R gives for "A" | "A"
              stop("operations are possible only for numeric, logical or complex types")
          })

##' @noRd
setMethod("Math", c("nanoival"),
          function(x) {
              ## this is the same error message that R gives for abs("A")
              stop("non-numeric argument to mathematical function")
          })

##' @noRd
setMethod("Math2", c("nanoival"),
          function(x, digits) {
              ## this is the same error message that R gives for round("A")
              stop("non-numeric argument to mathematical function")
          })

##' @noRd
setMethod("Summary", c("nanoival"),
          function(x, ..., na.rm = FALSE) {
              ## this is the same error message that R gives for sum("A")
              stop("invalid 'type' (nanoival) of argument")
          })

##' @noRd
setMethod("Complex", c("nanoival"),
          function(z) {
              ## this is the same error message that R gives for Arg("A")
              stop("non-numeric argument to function")
          })

## ----------- non ops

##' @rdname nanoival
setMethod("[[",
          signature("nanoival"),
          function (x, i, j, ..., drop=FALSE) {
              new("nanoival", callNextMethod())
          })

##' @rdname nanoival
setMethod("[",
          signature("nanoival", "logical"),
          function (x, i, j, ..., drop=FALSE) {
              if (!missing(j) || length(list(...)) > 0) {
                  warning("unused indices or arguments in 'nanoival' subsetting")
              }
              nanoival_subset_logical_impl(x, i)
         })

##' @rdname nanoival
setMethod("[",
          signature("nanoival", "numeric"),
          function (x, i, j, ..., drop=FALSE) {
              if (!missing(j) || length(list(...)) > 0) {
                  warning("unused indices or arguments in 'nanoival' subsetting")
              }
              if (isTRUE(any(i < 0))) {
                  new("nanoival", unclass(x)[i])
              } else {
                  nanoival_subset_numeric_impl(x, i)
              }
          })

##' @rdname nanoival
setMethod("[",
          signature("nanoival", "character"),
          function (x, i, j, ..., drop=FALSE) {
              ## we don't implement 'period_subset_character_impl' but
              ## do the gymnastic of finding the NAs here at R level;
              ## it's not as efficient, but using 'character' indexing
              ## is by itself inefficient, so the overhead should have
              ## no practical consequences.
              if (!missing(j) || length(list(...)) > 0) {
                  warning("unused indices or arguments in 'nanoival' subsetting")
              }
              na_index <- !(i %in% names(x)) | is.na(i)
              res <- new("nanoival", unclass(x)[i])
              res[na_index] <- NA_nanoival_
              res
          })

##' @rdname nanoival
setMethod("[",
          signature("nanoival", "ANY"),
          function (x, i, j, ..., drop=FALSE) {
              stop("']' not defined for on 'nanoival' for index of type 'ANY'")
          })

##' @rdname nanoival
setMethod("[<-",
          signature("nanoival", "logical", "ANY", "nanoival"),
          function (x, i, j, ..., value) {
              new("nanoival", callNextMethod())
          })

##' @rdname nanoival
c.nanoival <- function(...) {
    args <- list(...)
    s3args <- lapply(args, function (x) S3Part(x, strictS3=TRUE))
    res <- do.call(c, s3args)
    names(res) <- names(args)
    new("nanoival", res)
}

## S4 'c' doesn't handle names correctly, and couldn't find a way to rectify that:

## ##' @rdname nanoival
## ## setMethod("c",
##           signature("nanoival"),
##           function(x, ...) {
##               print("method ccc")
##               args <- list(...)
##               res <- callNextMethod()
##               names(res) <- names(args)
##               print(names(args))
##               new("nanoival", res)
##           })

##' @rdname nanoival
setMethod("t", c("nanoival"),
          function(x) {
              ## identity, like POSIXct, because nanoival doesn't support arrays
              x
          })


## ##' @rdname nanoival
## ## setMethod("cbind2",
##           signature("nanoival", "nanoival"),
##           function (x, y, ...) {
##               print(dimnames(x))
##               print("calling next method")
##               res <- callNextMethod()
##               print(attributes(res))
##               new("nanoival", res)
##           })

## ##' @rdname nanoival
## ## setMethod("cbind2",
##           signature("nanoival", "nanoival"),
##           function (x, y, ...) {
##               x <- t(x)
##               y <- t(y)
##               print(dimnames(x))
##               print("calling next method")
##               res <- callNextMethod()
##               print(attributes(res))
##               new("nanoival", res)
##           })


## ##' @rdname nanoival
## ## setMethod("rbind2",
##           signature("nanoival", "nanoival"),
##           function (x, y, ...) {
##               print("rbind2")
##               dim(x)
##               x <- t(x)
##               y <- t(y)
##               print(dim(x))
##               print(dim(y))
##               print("rbind2 dimnames")
##               print(dimnames(x))
##               new("nanoival", callNextMethod())
##           })

## set functions
## -------------

##' Set operations
##'
##' Performs set intersection, union and difference between vectors of
##' temporal types from the \code{nanotime} package.
##'
##' Set operations between \code{nanoival} operands allow the
##' construction of complex interval vectors (i.e. a \code{nanoival}
##' vector can specify any number of inclusions and exclusions of
##' time). Set operations between \code{nanotime} and \code{nanoival}
##' allow to subset time vectors with interval vectors. In addition to
##' the generic set functions, the function \code{intersect.idx} is
##' defined which returns the indices of the intersection, and the
##' operator \code{\%in\%} is overloaded for \code{nanotime-nanoival}
##' which returns a logical vector that indicates which elements
##' belong to the interval vector.
##'
##' 
##' @param x,y a temporal type
##' @param table \code{nanoival}: used in \code{\%in\%}
##' @return \code{intersect}, \code{union}, \code{setdiff} return
##'     temporal types that are the result of the intersection. For
##'     instance, set operations on two \code{nanoival} return a
##'     \code{nanoival}, whereas intersection between a
##'     \code{nanoival} and a \code{nanotime} returns a
##'     \code{nanotime}. \code{intersect.idx} return a list of vectors
##'     representing the element indices that intersect and
##'     \code{setdiff.idx} returns a vector representing the element
##'     indices to be removed.
##' @examples
##' \dontrun{
##' ## a vector of 'nanotime' can be subsetted by a 'nanoival' which is equivalent to 'intersect':
##' one_second <- 1e9
##' a <- seq(nanotime("2012-12-12 12:12:12+00:00"), length.out=10, by=one_second)
##' idx <- c(as.nanoival("-2012-12-12 12:12:10+00:00 -> 2012-12-12 12:12:14+00:00-"),
##'          as.nanoival("+2012-12-12 12:12:18+00:00 -> 2012-12-12 12:12:20+00:00+"))
##' a[idx]
##' intersect(a, idx)
##'
##' ## 'nanoival' also has the set operations 'union', 'intersect', 'setdiff':
##' a <- seq(nanotime("2012-12-12 12:12:12+00:00"), length.out=10, by=one_second)
##' i <- as.nanoival("-2012-12-12 12:12:14+00:00 -> 2012-12-12 12:12:18+00:00-")
##' setdiff(a, i)
##'
##' i1 <- as.nanoival("+2012-12-12 12:12:14+00:00 -> 2012-12-12 12:12:17+00:00-")
##' i2 <- as.nanoival("+2012-12-12 12:12:16+00:00 -> 2012-12-12 12:12:18+00:00-")
##' union(i1, i2)
##'
##' ## 'intersect.idx' returns the indices of the intersection:
##' a <- seq(nanotime("2012-12-12 12:12:12+00:00"), length.out=10, by=one_second)
##' idx <- as.nanoival("+2012-12-12 12:12:14+00:00 -> 2012-12-12 12:12:19+00:00+")
##' idx_intersect <- intersect.idx(a, idx)
##'
##' ## Intersection can be performed using these indices:
##' a[idx_intersect$x]
##'
##' ## which is equivalent to:
##' a[idx]
##'
##' ## The logical vector indicating intersection can be obtained like this:
##' a %in% idx
##' }
##' @rdname set_operations
##'
setMethod("intersect",
          c("nanoival", "nanoival"),
          function(x, y) {
              x <- sort(x)
              y <- sort(y)
              res <- nanoival_intersect_impl(x, y)
              new("nanoival", res)
          })

##' @rdname set_operations
setMethod("union",
          c("nanoival", "nanoival"),
          function(x, y) {
              x <- sort(x)
              y <- sort(y)
              res <- nanoival_union_impl(x, y)
              new("nanoival", res)
          })

##' @rdname set_operations
setMethod("setdiff",
          c("nanoival", "nanoival"),
          function(x, y) {
              x <- sort(x)
              y <- sort(y)
              res <- nanoival_setdiff_impl(x, y)
              new("nanoival", res)
          })


## nanotime and nanoival:
## ---------------------

##' @rdname nanoival
setMethod("[",
          signature("nanotime", "nanoival"),
          function (x, i, ..., drop=FALSE) {
              if (is.unsorted(x)) stop("x must be sorted")
              i <- sort(i)
              nanoival_intersect_time_interval_impl(x, i)
          })


##' @noRd
setGeneric("intersect.idx", function(x, y) standardGeneric("intersect.idx"))


##' @rdname set_operations
##' @aliases intersect.idx
setMethod("intersect.idx",
          c("nanotime", "nanoival"),
          function(x, y) {
              if (is.unsorted(x)) stop("x must be sorted")
              y <- sort(y)
              nanoival_intersect_idx_time_interval_impl(x, y)
          })


##' @rdname set_operations
##' @method %in% nanotime
`%in%.nanotime` <- function(x, table) {
    if (inherits(table, "nanoival")) {
        if (is.unsorted(x)) stop("x must be sorted")
        table <- sort(table)
        nanoival_intersect_idx_time_interval_logical_impl(x, table)
    } else {
        NextMethod()
    }
}

##' @rdname set_operations
setMethod("%in%",
          c("nanotime", "nanoival"),
          function(x, table) {
              if (is.unsorted(x)) stop("x must be sorted")   ## #nocov
              table <- sort(table)                           ## #nocov
              nanoival_intersect_idx_time_interval_logical_impl(x, table)  ## #nocov
          })

##' @rdname set_operations
setMethod("intersect",
          c("nanotime", "nanoival"),
          function(x, y) {
              x <- sort(x)
              y <- sort(y)
              nanoival_intersect_time_interval_impl(x, y)
          })

##' @rdname set_operations
setMethod("setdiff",
          c("nanotime", "nanoival"),
          function(x, y) {
              x <- sort(x)
              y <- sort(y)
              res <- nanoival_setdiff_time_interval_impl(x, y)
              oldClass(res) <- "integer64"
              new("nanotime", res)
          })

##' @noRd
setGeneric("setdiff.idx", function(x, y) standardGeneric("setdiff.idx"))

##' @rdname set_operations
##' @aliases setdiff.idx
setMethod("setdiff.idx",
          c("nanotime", "nanoival"),
          function(x, y) {
              if (is.unsorted(x)) stop("x must be sorted")
              y <- sort(y)
              nanoival_setdiff_idx_time_interval_impl(x, y)
          })


## provide 'nanotime'-'nanotime' set operations and document here
## ---------------------

##' @rdname set_operations
setMethod("intersect",
          c("nanotime", "nanotime"),
          function(x, y) {
              res <- callNextMethod()
              oldClass(res) <- "integer64"
              new("nanotime", res)
          })

##' @rdname set_operations
setMethod("union",
          c("nanotime", "nanotime"),
          function(x, y) {
              res <- callNextMethod()
              oldClass(res) <- "integer64"
              new("nanotime", res)
          })

##' @rdname set_operations
setMethod("setdiff",
          c("nanotime", "nanotime"),
          function(x, y) {
              res <- if (getRversion() >= "4.5.0") setdiff(as.integer64(x), as.integer64(y)) else callNextMethod()
              oldClass(res) <- "integer64"
              new("nanotime", res)
          })


## misc functions:
## --------------

##' Test if Two Objects are (Nearly) Equal
##'
##' Compare \code{target} and \code{current} testing \sQuote{near
##' equality}.  If they are different, comparison is still made to
##' some extent, and a report of the differences is returned.  Do not
##' use \code{all.equal} directly in \code{if} expressions---either
##' use \code{isTRUE(all.equal(....))} or \code{\link{identical}} if
##' appropriate.
##'
##' @param target,current \code{nanoival} arguments to be compared
##' @param ... further arguments for different methods
##' @param check.attributes logical indicating if the
##'     \code{attributes} of \code{target} and \code{current} (other than
##'     the names) should be compared.
##'
##' @seealso \code{\link{identical}}, \code{\link{isTRUE}},
##'     \code{\link{==}}, and \code{\link{all}} for exact equality
##'     testing.
##'
##' @method all.equal nanoival
##'
all.equal.nanoival <-
    function(target, current, ..., check.attributes = TRUE) all.equal.raw(target, current, ..., check.attributes)


##' @rdname all.equal.nanoival
setMethod("all.equal", c(target = "nanoival", current="ANY"), all.equal.nanoival)


##' Test if a \code{nanoival} vector is Not Sorted
##'
##' Test if an object is not sorted (in increasing order), without the
##' cost of sorting it.
##'
##' @param x a \code{nanoival} vector
##' @param na.rm logical.  Should missing values be removed before
##'     checking?
##' @param strictly logical indicating if the check should be for
##'     _strictly_ increasing values.
##'
##' @seealso \code{\link{sort}}
##'
setMethod("is.unsorted", "nanoival",
          function(x, na.rm=FALSE, strictly=FALSE) {
              if (!is.logical(strictly)) {
                  stop("argument 'strictly' must be a logical")
              }
              if (na.rm == TRUE) {
                  nanoival_is_unsorted_impl(x[!is.na(x)], strictly) # we could code this more efficiently!
              } else {
                  if (any(is.na(x))) {  # same here...
                      NA_nanoival_
                  } else {
                      nanoival_is_unsorted_impl(x, strictly)
                  }
              }
          })


##' Sorting or Ordering Vectors
##'
##' Sort (or _order_) a vector of \code{nanoival} into ascending or
##' descending order
##'
##' @param x a vector of \code{nanoival}
##' @param decreasing logical.  Should the sort be increasing or
##'     decreasing?
##' @seealso \code{\link{is.unsorted}}
##'
setMethod("sort", c("nanoival"),
          function(x, decreasing=FALSE) {
              if (!is.logical(decreasing)) {
                  stop("argument 'decreasing' must be logical")
              }
              new("nanoival", nanoival_sort_impl(x, decreasing))
          })


##' Sequence Generation
##'
##' Generate a sequence of \code{nanoival}
##'
##' @param ... arguments passed to or from methods; the only
##'     interesting additional argument is \code{tz} where the
##'     \code{to} argument is of type \code{nanoperiod}
##' @param from,to the starting and (maximal) end values of the
##'     sequence
##' @param by \code{nanoduration} or \code{nanoperiod} increment of
##'     the sequence; note that if the class is \code{nanoperiod} the
##'     additional argument \code{tz} must be speficied and is of
##'     \code{character} type indicating a timezone
##' @param length.out an integer desired length of the sequence
##' @param along.with take the length from the length of this argument.
##' @examples
##' \dontrun{
##' from <- as.nanoival("-2018-01-14T13:00:00+00:00 -> 2018-01-14T15:00:00+00:00+")
##' seq(from, by=as.nanoperiod("1m"), length.out=5, tz="America/New_York")
##' }
setMethod("seq", c("nanoival"),
          function(from, to = NULL, by = NULL, length.out = NULL, along.with = NULL, ...)
          {
              if (is.null(to)) {
                  start <- seq(nanoival.start(from), by=by, length.out=length.out, along.with=along.with, ...)
              } else {
                  start  <- seq(nanoival.start(from), nanoival.start(to), by, length.out, along.with, ...)
              }
              if (is.null(by)) {
                  end <- seq(nanoival.end(from), by=start[2]-start[1], length.out=length(start), ...)
              } else {
                  end <- seq(nanoival.end(from), by=by, length.out=length(start), ...)
              }
              nanoival(start, end, nanoival.sopen(from), nanoival.eopen(from))
          })


##' @rdname nanoival
NA_nanoival_ <- new("nanoival", complex(1, -4.9406564584124654418e-324, -4.9406564584124654418e-324))


##' Replicate Elements
##'
##' Replicates the values in 'x' similarly to the default method.
##'
##' @param x a vector of \code{nanoival}
##' @param ... further arguments:
##' 
##'        'times' an integer-valued vector giving the (non-negative)
##'            number of times to repeat each element if of length
##'            'length(x)', or to repeat the whole vector if of length
##'            1. Negative or 'NA' values are an error. A 'double'
##'            vector is accepted, other inputs being coerced to an
##'            integer or double vector.
##' 
##'        'length.out' non-negative integer. The desired length of the
##'            output vector. Other inputs will be coerced to a double
##'            vector and the first element taken. Ignored if 'NA' or
##'            invalid.
##' 
##'        'each' non-negative integer. Each element of 'x' is repeated
##'            'each' times.  Other inputs will be coerced to an integer
##'            or double vector and the first element taken. Treated as
##'            '1' if 'NA' or invalid.
setMethod("rep", c(x = "nanoival"), function(x, ...) {
    new("nanoival", callNextMethod())
})

##' @rdname nanoival
as.character.nanoival <- function(x, ...) {
    format(x, ...)
}

library(nanotime)

z <- RcppCCTZ:::parseDouble("1970-01-01T00:00:00.000000001+00:00")
cat("z is: ")
print(z)

x <- nanotime("1970-01-01T00:00:00.000000001+00:00")
cat("x is: ")
print(x)
format(x)

cat("x+1 is: ")
x <- x + 1
print(x)
format(x)

cat("y is: ")
y <- nanotime(z)
print(y)
#print(class(y))
format(y)

cat("y+1 is: ")
y <- y + 1
print(y)
format(y)

print(x == y)

od <- getOption("digits.secs")
options("digits.secs"=6)
as.POSIXct(x)
as.POSIXct(x+1000)
as.POSIXlt(x)
as.POSIXlt(x+1000)
as.Date(x)
as.Date(x, tz="UTC")
options("digits.secs"=od)


y <- nanotime(1L)  # integer, may dispatch via nanotime.numeric
print(y)

y <- nanotime(1)   # numeric
print(y)

## v <- nanotime:::nanotime.default(1)     # forced call, gets imprecise value
## print(v)


options("nanotimeFormat"="%Y-%m-%d %H:%M:%S")
format(x <- nanotime("1970-01-01 00:00:00"))

options("nanotimeFormat"="%Y-%m-%d %H:%M:%E*S")
format(x <- nanotime("1970-01-01 00:00:00.123456789"))

options("nanotimeFormat"="%Y-%m-%d %H:%M:%E*S%Ez")   # default

cat("Done\n")

library(nanotime)

format(nanotime(Sys.time()),
       lcltzstr="America/Chicago", tgttzstr="America/Chicago")

format(nanotime(as.POSIXlt(Sys.time())),
       lcltzstr="America/Chicago", tgttzstr="America/Chicago")
       
format(nanotime(Sys.Date()),
       lcltzstr="America/Chicago", tgttzstr="America/Chicago")

rver <- getRversion()
si <- Sys.info()
if (rver < "4.3.0" || (si["sysname"] == "Darwin" && si["machine"] == "x86_64")) {
    ## for R versions prior to 4.3.0 we need to set the standard, and also on
    ## the r-release-macos-x86_64 machine which is not as C++20-ready as the rest
    cxxstd <- "CXX_STD = CXX17"
} else {
    ## for R 4.3.0 C++17 is standard so no need to set it
    cxxstd <- "#CXX_STD = CXX17"
}
win <- if (si["sysname"] == "Windows") ".win" else ""
infile <- file.path("src", paste0("Makevars", win, ".in"))
outfile <- file.path("src", paste0("Makevars", win))
lines <- readLines(infile)
lines <- gsub("@CXXSTD@", cxxstd, lines)
writeLines(lines, outfile)

library(nanotime)

set.seed(42)
N <- 300
shine <- nanotime(Sys.time()) + cumsum(10*rpois(n=N+1, lambda=4))
rain <- nanotime(Sys.time()) + cumsum(10*rpois(n=N+1, lambda=4) + round(runif(N+1)*25))

if (requireNamespace("data.table", quietly=TRUE)) {
    suppressMessages(library(data.table))
    suppressMessages(library(bit64))
    raw <- data.table(shine=shine, rain=rain)

    df <- data.frame(val=c(rain,shine), key=rep(c("rain", "shine"), each=N+1))
    head(df)

    ## now on differences
    ddf <- data.frame(val=as.numeric(c(diff(rain),diff(shine))), # need to cast to numeric after diffs
                      key=rep(c("rain", "shine"), each=N))
    head(ddf)

    ## simpler data.frame, inserts 'formatted'
    newdf <- data.frame(rain=rain, shine=shine)
    head(newdf)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

duration_from_string_impl <- function(str) {
    .Call(`_nanotime_duration_from_string_impl`, str)
}

duration_to_string_impl <- function(dur) {
    .Call(`_nanotime_duration_to_string_impl`, dur)
}

duration_is_na_impl <- function(dur) {
    .Call(`_nanotime_duration_is_na_impl`, dur)
}

make_duration_impl <- function(h_nv, m_nv, s_nv, n_nv) {
    .Call(`_nanotime_make_duration_impl`, h_nv, m_nv, s_nv, n_nv)
}

nanoduration_subset_numeric_impl <- function(v, idx) {
    .Call(`_nanotime_nanoduration_subset_numeric_impl`, v, idx)
}

nanoduration_subset_logical_impl <- function(v, idx_p) {
    .Call(`_nanotime_nanoduration_subset_logical_impl`, v, idx_p)
}

nanoival_intersect_idx_time_interval_impl <- function(nv1, nv2) {
    .Call(`_nanotime_nanoival_intersect_idx_time_interval_impl`, nv1, nv2)
}

nanoival_intersect_idx_time_interval_logical_impl <- function(nv1, nv2) {
    .Call(`_nanotime_nanoival_intersect_idx_time_interval_logical_impl`, nv1, nv2)
}

nanoival_intersect_time_interval_impl <- function(nv1, nv2) {
    .Call(`_nanotime_nanoival_intersect_time_interval_impl`, nv1, nv2)
}

nanoival_setdiff_time_interval_impl <- function(nv1, nv2) {
    .Call(`_nanotime_nanoival_setdiff_time_interval_impl`, nv1, nv2)
}

nanoival_union_impl <- function(nv1, nv2) {
    .Call(`_nanotime_nanoival_union_impl`, nv1, nv2)
}

nanoival_intersect_impl <- function(nv1, nv2) {
    .Call(`_nanotime_nanoival_intersect_impl`, nv1, nv2)
}

nanoival_setdiff_impl <- function(nv1, nv2) {
    .Call(`_nanotime_nanoival_setdiff_impl`, nv1, nv2)
}

nanoival_is_unsorted_impl <- function(nvec, strictlyvec) {
    .Call(`_nanotime_nanoival_is_unsorted_impl`, nvec, strictlyvec)
}

nanoival_sort_impl <- function(nvec, decreasingvec) {
    .Call(`_nanotime_nanoival_sort_impl`, nvec, decreasingvec)
}

nanoival_sort_impl2 <- function(nvec, decreasing) {
    .Call(`_nanotime_nanoival_sort_impl2`, nvec, decreasing)
}

nanoival_lt_impl <- function(n1, n2) {
    .Call(`_nanotime_nanoival_lt_impl`, n1, n2)
}

nanoival_le_impl <- function(n1, n2) {
    .Call(`_nanotime_nanoival_le_impl`, n1, n2)
}

nanoival_gt_impl <- function(n1, n2) {
    .Call(`_nanotime_nanoival_gt_impl`, n1, n2)
}

nanoival_ge_impl <- function(n1, n2) {
    .Call(`_nanotime_nanoival_ge_impl`, n1, n2)
}

nanoival_eq_impl <- function(n1, n2) {
    .Call(`_nanotime_nanoival_eq_impl`, n1, n2)
}

nanoival_ne_impl <- function(n1, n2) {
    .Call(`_nanotime_nanoival_ne_impl`, n1, n2)
}

nanoival_plus_impl <- function(n1, n2) {
    .Call(`_nanotime_nanoival_plus_impl`, n1, n2)
}

nanoival_minus_impl <- function(n1, n2) {
    .Call(`_nanotime_nanoival_minus_impl`, n1, n2)
}

nanoival_setdiff_idx_time_interval_impl <- function(nv1, cv2) {
    .Call(`_nanotime_nanoival_setdiff_idx_time_interval_impl`, nv1, cv2)
}

nanoival_new_impl <- function(sv, ev, sopenv, eopenv) {
    .Call(`_nanotime_nanoival_new_impl`, sv, ev, sopenv, eopenv)
}

nanoival_get_start_impl <- function(cv) {
    .Call(`_nanotime_nanoival_get_start_impl`, cv)
}

nanoival_get_end_impl <- function(cv) {
    .Call(`_nanotime_nanoival_get_end_impl`, cv)
}

nanoival_get_sopen_impl <- function(cv) {
    .Call(`_nanotime_nanoival_get_sopen_impl`, cv)
}

nanoival_get_eopen_impl <- function(cv) {
    .Call(`_nanotime_nanoival_get_eopen_impl`, cv)
}

nanoival_isna_impl <- function(cv) {
    .Call(`_nanotime_nanoival_isna_impl`, cv)
}

nanoival_make_impl <- function(nt_v, tz_v) {
    .Call(`_nanotime_nanoival_make_impl`, nt_v, tz_v)
}

nanoival_subset_numeric_impl <- function(v, idx) {
    .Call(`_nanotime_nanoival_subset_numeric_impl`, v, idx)
}

nanoival_subset_logical_impl <- function(v, idx_p) {
    .Call(`_nanotime_nanoival_subset_logical_impl`, v, idx_p)
}

nanotime_wday_impl <- function(tm_v, tz_v) {
    .Call(`_nanotime_nanotime_wday_impl`, tm_v, tz_v)
}

nanotime_mday_impl <- function(tm_v, tz_v) {
    .Call(`_nanotime_nanotime_mday_impl`, tm_v, tz_v)
}

nanotime_month_impl <- function(tm_v, tz_v) {
    .Call(`_nanotime_nanotime_month_impl`, tm_v, tz_v)
}

nanotime_year_impl <- function(tm_v, tz_v) {
    .Call(`_nanotime_nanotime_year_impl`, tm_v, tz_v)
}

nanotime_make_impl <- function(nt_v, tz_v) {
    .Call(`_nanotime_nanotime_make_impl`, nt_v, tz_v)
}

nanotime_subset_numeric_impl <- function(v, idx) {
    .Call(`_nanotime_nanotime_subset_numeric_impl`, v, idx)
}

nanotime_subset_logical_impl <- function(v, idx_p) {
    .Call(`_nanotime_nanotime_subset_logical_impl`, v, idx_p)
}

period_from_string_impl <- function(str) {
    .Call(`_nanotime_period_from_string_impl`, str)
}

period_from_parts_impl <- function(months_v, days_v, dur_v) {
    .Call(`_nanotime_period_from_parts_impl`, months_v, days_v, dur_v)
}

period_to_string_impl <- function(prd) {
    .Call(`_nanotime_period_to_string_impl`, prd)
}

period_from_integer64_impl <- function(i64) {
    .Call(`_nanotime_period_from_integer64_impl`, i64)
}

period_from_integer_impl <- function(iint) {
    .Call(`_nanotime_period_from_integer_impl`, iint)
}

period_from_double_impl <- function(dbl) {
    .Call(`_nanotime_period_from_double_impl`, dbl)
}

plus_period_period_impl <- function(e1_nv, e2_nv) {
    .Call(`_nanotime_plus_period_period_impl`, e1_nv, e2_nv)
}

minus_period_impl <- function(e1_cv) {
    .Call(`_nanotime_minus_period_impl`, e1_cv)
}

minus_period_period_impl <- function(e1_cv, e2_cv) {
    .Call(`_nanotime_minus_period_period_impl`, e1_cv, e2_cv)
}

eq_period_period_impl <- function(e1_p, e2_p) {
    .Call(`_nanotime_eq_period_period_impl`, e1_p, e2_p)
}

ne_period_period_impl <- function(e1_p, e2_p) {
    .Call(`_nanotime_ne_period_period_impl`, e1_p, e2_p)
}

plus_period_integer64_impl <- function(e1_cv, e2_nv) {
    .Call(`_nanotime_plus_period_integer64_impl`, e1_cv, e2_nv)
}

minus_period_integer64_impl <- function(e1_cv, e2_nv) {
    .Call(`_nanotime_minus_period_integer64_impl`, e1_cv, e2_nv)
}

multiplies_period_integer64_impl <- function(e1_cv, e2_nv) {
    .Call(`_nanotime_multiplies_period_integer64_impl`, e1_cv, e2_nv)
}

divides_period_integer64_impl <- function(e1_cv, e2_nv) {
    .Call(`_nanotime_divides_period_integer64_impl`, e1_cv, e2_nv)
}

multiplies_period_double_impl <- function(e1_cv, e2_nv) {
    .Call(`_nanotime_multiplies_period_double_impl`, e1_cv, e2_nv)
}

divides_period_double_impl <- function(e1_cv, e2_nv) {
    .Call(`_nanotime_divides_period_double_impl`, e1_cv, e2_nv)
}

minus_integer64_period_impl <- function(e1_nv, e2_cv) {
    .Call(`_nanotime_minus_integer64_period_impl`, e1_nv, e2_cv)
}

plus_nanotime_period_impl <- function(e1_nv, e2_cv, tz_v) {
    .Call(`_nanotime_plus_nanotime_period_impl`, e1_nv, e2_cv, tz_v)
}

minus_nanotime_period_impl <- function(e1_nv, e2_cv, tz_v) {
    .Call(`_nanotime_minus_nanotime_period_impl`, e1_nv, e2_cv, tz_v)
}

plus_nanoival_period_impl <- function(e1_cv, e2_cv, tz_v) {
    .Call(`_nanotime_plus_nanoival_period_impl`, e1_cv, e2_cv, tz_v)
}

minus_nanoival_period_impl <- function(e1_cv, e2_cv, tz_v) {
    .Call(`_nanotime_minus_nanoival_period_impl`, e1_cv, e2_cv, tz_v)
}

period_month_impl <- function(e_n) {
    .Call(`_nanotime_period_month_impl`, e_n)
}

period_day_impl <- function(e_n) {
    .Call(`_nanotime_period_day_impl`, e_n)
}

period_duration_impl <- function(e_n) {
    .Call(`_nanotime_period_duration_impl`, e_n)
}

period_isna_impl <- function(cv) {
    .Call(`_nanotime_period_isna_impl`, cv)
}

period_seq_from_to_impl <- function(from_nv, to_nv, by_cv, tz) {
    .Call(`_nanotime_period_seq_from_to_impl`, from_nv, to_nv, by_cv, tz)
}

period_seq_from_length_impl <- function(from_nv, by_cv, n_nv, tz) {
    .Call(`_nanotime_period_seq_from_length_impl`, from_nv, by_cv, n_nv, tz)
}

period_subset_numeric_impl <- function(v, idx) {
    .Call(`_nanotime_period_subset_numeric_impl`, v, idx)
}

period_subset_logical_impl <- function(v, idx_p) {
    .Call(`_nanotime_period_subset_logical_impl`, v, idx_p)
}

ceiling_tz_impl <- function(nt_v, prd_v, orig_v, tz_v) {
    .Call(`_nanotime_ceiling_tz_impl`, nt_v, prd_v, orig_v, tz_v)
}

ceiling_impl <- function(nt_v, dur_v, orig_v) {
    .Call(`_nanotime_ceiling_impl`, nt_v, dur_v, orig_v)
}

floor_tz_impl <- function(nt_v, prd_v, orig_v, tz_v) {
    .Call(`_nanotime_floor_tz_impl`, nt_v, prd_v, orig_v, tz_v)
}

floor_impl <- function(nt_v, dur_v, orig_v) {
    .Call(`_nanotime_floor_impl`, nt_v, dur_v, orig_v)
}

library(nanotime)
suppressMessages(library(zoo))          

set.seed(42)
x <- 100 + cumsum(rnorm(10))
y <- 100 + cumsum(rnorm(10))
mat <- cbind(x, y)

now <- Sys.Date()
z <- zoo(mat, now + 0:9)
z

now <- Sys.time()
z <- zoo(mat, now + 0:9)
z

now <- nanotime(Sys.time())                  
z <- zoo(mat, now + 11*(0:9))           # inc. by 11 to more visible, 1 works too
z

##' @rdname nanoduration
setClass("nanoduration", contains = "integer64")

##' Duration type with nanosecond precision
##'
##' The type \code{nanoduration} is a length of time (implemented as
##' an S4 class) with nanosecond precision. It is a count of
##' nanoseconds and may be negative. The expected arithmetic
##' operations are provided, including sequence generation.
##'
##' A \code{nanoduration} can be constructed with the function
##' \code{as.nanoduration} which can take the types \code{integer64},
##' \code{integer} and \code{numeric} (all indicating the count in
##' nanosecond units) or the type \code{character}.
##'
##' It can also be constructed by specifying with individual arguments
##' the hours, minutes, seconds and nanoseconds with a call to
##' \code{nanoduration}.
##'
##' A \code{nanoduration} is displayed as hours, minutes, seconds and
##' nanoseconds like this: \code{110:12:34.123_453_001}. The nanosecond
##' precision displayed is adjusted as necessary, so e.g. 1 second is
##' displayed as \code{00:00:01}.
##'
##' @param hours number of hours
##' @param minutes number of minutes
##' @param seconds number of seconds
##' @param nanoseconds number of nanoseconds
##' @param x a \code{nanoduration} object
##' @param ... further arguments passed to or from methods.
##' @param e1 Operand of class \code{nanoival}
##' @param e2 Operand of class \code{nanoival}
##' @param object argument for method \code{show}
##' @param i index specifying elements to extract or replace.
##' @param j Required for \code{[} signature but ignored here
##' @param drop Required for \code{[} signature but ignored here
##' @param value argument for \code{nanoduration-class}
##' @param na.rm if \code{TRUE} NA values are removed for the
##'     computation
##' @param quote indicates if the output of \code{print} should be
##'     quoted
##' @return A nanoduration object
##' @author Dirk Eddelbuettel
##' @author Leonardo Silvestri
##' @examples
##' ## constructors:
##' nanoduration(hours=10, minutes=3, seconds=2, nanoseconds=999999999)
##' as.nanoduration("10:03:02.999_999_999")
##' as.nanoduration(36182999999999)
##'
##' ## arithmetic:
##' as.nanoduration(10e9) - as.nanoduration(9e9)
##' as.nanoduration(10e9) + as.nanoduration(-9e9)
##' as.nanoduration("24:00:00") / 2
##' as.nanoduration("24:00:00") / as.nanoduration("12:00:00")
##'
##' ## comparison:
##' as.nanoduration("10:03:02.999_999_999") == 36182999999999
##' as.nanoduration("10:03:02.999_999_999") > as.nanoduration("10:03:02.999_999_998")
##' as.nanoduration("10:03:02.999_999_998") < "10:03:02.999_999_999"
##'
##' @seealso
##' \code{\link{nanotime}}
##'
##' @aliases  *,ANY,nanoduration-method
##' @aliases  *,nanoduration,ANY-method
##' @aliases  *,nanoduration,nanoduration-method
##' @aliases  +,ANY,nanoduration-method
##' @aliases  /,ANY,nanoduration-method
##' @aliases  /,nanoduration,ANY-method
##' @aliases  Complex,nanoduration-method
##' @aliases  Logic,ANY,nanoduration-method
##' @aliases  Logic,nanoduration,ANY-method
##' @aliases  Logic,nanoduration,nanoduration-method
##' @aliases  Math2,nanoduration-method
##' @aliases  Math,nanoduration-method
##' @aliases  Summary,nanoduration-method
##'
##' @rdname nanoduration
nanoduration <- function(hours = 0L, minutes = 0L, seconds = 0L, nanoseconds = 0L) {
    if (nargs()==0) {
        as.nanoduration(NULL)
    } else {
        make_duration_impl(as.integer64(hours),
                           as.integer64(minutes),
                           as.integer64(seconds),
                           as.integer64(nanoseconds))
    }
}

setOldClass("difftime")

##' @noRd
setGeneric("as.nanoduration", function(x) standardGeneric("as.nanoduration"))

##' @rdname nanoduration
##' @aliases as.nanoduration
setMethod("as.nanoduration",
          "character",
          function(x) {
              duration_from_string_impl(x)
          })

setAs("character", "nanoduration", function(from) as.nanoduration(from))

##' @rdname nanoduration
setMethod("as.nanoduration",
          "integer64",
          function(x) {
              new("nanoduration", x)
          })

setAs("integer64", "nanoduration", function(from) as.nanoduration(from))


##' @rdname nanoduration
setMethod("as.nanoduration",
          "numeric",
          function(x) {
              nm <- names(x)
              res <- new("nanoduration", as.integer64(x))
              if (!is.null(nm)) {
                  names(res) <- nm
              }
              res
          })

setAs("numeric", "nanoduration", function(from) as.nanoduration(from))


##' @rdname nanoduration
setMethod("as.nanoduration",
          "integer",
          function(x) {
              nm <- names(x)
              res <- new("nanoduration", as.integer64(x))
              if (!is.null(nm)) {
                  names(res) <- nm
              }
              res
          })

setAs("integer", "nanoduration", function(from) as.nanoduration(from))

##' @rdname nanoduration
setMethod("as.nanoduration",
          "difftime",
          function(x) {
              x <- as.numeric(x, units = "secs")
              s <- as.integer(x) # seconds
              n <- as.integer(1e9 * (x - s)) # nanoseconds
              nanoduration(seconds = s, nanoseconds = n)
})

setAs("difftime", "nanoduration", function(from) as.nanoduration(from))


##' @rdname nanoduration
setMethod("as.nanoduration",
          "NULL",
          function(x) {
              new("nanoduration", as.integer64(NULL))
          })

##' @rdname nanoduration
setMethod("as.nanoduration",
          "missing",
          function(x) {
              new("nanoduration", integer64())
          })

##' @rdname nanoduration
setMethod("show",
          signature("nanoduration"),
          function(object) print(object))

##' @rdname nanoduration
setMethod("print",
          "nanoduration",
          function(x, quote=FALSE, ...) {
              if (length(x) == 0) {
                  print("nanoduration(0)", quote=quote)
              } else {
                  print(duration_to_string_impl(x), quote=quote)
              }
          })

##' @rdname nanoduration
format.nanoduration <- function(x, ...) {
    as.character(x)
}


##' @rdname nanoduration
##' @method as.integer64 nanoduration
as.integer64.nanoduration <- function(x, ...) {
    S3Part(x, strictS3=TRUE)
}


##' @rdname nanoduration
setMethod("as.character",
          signature("nanoduration"),
          function(x) {
              duration_to_string_impl(x)
          })


##' @rdname nanoduration
setMethod("is.na",
          "nanoduration",
          function(x) {
              duration_is_na_impl(x)
          })


## ------------ `-`

## nanoduration - other
##' @rdname nanoduration
setMethod("-", c("nanoduration", "nanoduration"),
          function(e1, e2) {
              new("nanoduration", S3Part(e1, strictS3=TRUE) - S3Part(e2, strictS3=TRUE))
          })
##' @rdname nanoduration
setMethod("-", c("nanoduration", "integer64"),
          function(e1, e2) {
              new("nanoduration", S3Part(e1, strictS3=TRUE) - e2)
          })
##' @rdname nanoduration
setMethod("-", c("nanoduration", "integer"),
          function(e1, e2) {
              new("nanoduration", S3Part(e1, strictS3=TRUE) - e2)
          })
##' @rdname nanoduration
setMethod("-", c("nanoduration", "numeric"),
          function(e1, e2) {
              new("nanoduration", S3Part(e1, strictS3=TRUE) - e2)
          })
##' @rdname nanoduration
setMethod("-", c("nanoduration", "difftime"),
          function(e1, e2) {
              e1 - as.nanoduration(e2)
          })
##' @rdname nanoduration
setMethod("-", c("nanoduration", "ANY"),
          function(e1, e2) {
              if (missing(e2)) {
                  new("nanoduration", -S3Part(e1, strictS3=TRUE))
              }
              else {
                  stop("invalid operand types")
              }
          })

## other - nanoduration

##' @rdname nanoduration
setMethod("-", c("nanotime", "nanoduration"),
          function(e1, e2) {
              new("nanotime", S3Part(e1, strictS3=TRUE) - e2)
          })
##' @rdname nanoduration
setMethod("-", c("nanotime", "difftime"),
          function(e1, e2) {
              e1 - as.nanoduration(e2)
          })
##' @rdname nanoduration
setMethod("-", c("integer64", "nanoduration"),
          function(e1, e2) {
              new("nanoduration", e1 - S3Part(e2, strictS3=TRUE))
          })
##' @rdname nanoduration
setMethod("-", c("integer", "nanoduration"),
          function(e1, e2) {
              new("nanoduration", e1 - S3Part(e2, strictS3=TRUE))
          })
##' @rdname nanoduration
setMethod("-", c("numeric", "nanoduration"),
          function(e1, e2) {
              new("nanoduration", e1 - S3Part(e2, strictS3=TRUE))
          })
##' @rdname nanoduration
setMethod("-", c("difftime", "nanoduration"),
          function(e1, e2) {
              as.nanoduration(e1) - e2
          })
##' @rdname nanoduration
setMethod("-", c("ANY", "nanoduration"),
          function(e1, e2) {
              stop("invalid operand types")
          })

## ----------- `+`

##' @rdname nanoduration
setMethod("+", c("nanoduration", "ANY"),
          function(e1, e2) {
              if (missing(e2)) {
                  e1
              }
              else {
                  stop("invalid operand types")
              }
          })
##' @rdname nanoduration
setMethod("+", c("nanoduration", "nanoduration"),
          function(e1, e2) {
              new("nanoduration", S3Part(e1, strictS3=TRUE) + e2)
          })
##' @rdname nanoduration
setMethod("+", c("nanoduration", "integer64"),
          function(e1, e2) {
              new("nanoduration", S3Part(e1, strictS3=TRUE) + e2)
          })
##' @rdname nanoduration
setMethod("+", c("nanoduration", "numeric"),
          function(e1, e2) {
              new("nanoduration", S3Part(e1, strictS3=TRUE) + e2)
          })
##' @rdname nanoduration
setMethod("+", c("nanoduration", "difftime"),
          function(e1, e2) {
              e1 + as.nanoduration(e2)
          })
##' @rdname nanoduration
setMethod("+", c("nanotime", "nanoduration"),
          function(e1, e2) {
              new("nanotime", S3Part(e1, strictS3=TRUE) + S3Part(e2, strictS3=TRUE))
          })
##' @rdname nanoduration
setMethod("+", c("nanotime", "difftime"),
          function(e1, e2) {
              e1 + as.nanoduration(e2)
          })
##' @rdname nanoduration
setMethod("+", c("nanoduration", "nanotime"),
          function(e1, e2) {
              new("nanotime", S3Part(e1, strictS3=TRUE) + S3Part(e2, strictS3=TRUE))
          })
##' @rdname nanoduration
setMethod("+", c("difftime", "nanotime"),
          function(e1, e2) {
              as.nanoduration(e1) + e2
          })

##' @rdname nanoduration
setMethod("+", c("nanoival", "nanoduration"),
          function(e1, e2) {
              new("nanoival", nanoival_plus_impl(e1, e2))
          })
##' @rdname nanoduration
setMethod("-", c("nanoival", "nanoduration"),
          function(e1, e2) {
              new("nanoival", nanoival_plus_impl(e1, -e2))
          })
##' @rdname nanoduration
setMethod("+", c("nanoduration", "nanoival"),
          function(e1, e2) {
              new("nanoival", nanoival_plus_impl(e2, e1))
          })
##' @rdname nanoduration
setMethod("+", c("nanoival", "difftime"),
          function(e1, e2) {
              new("nanoival", nanoival_plus_impl(e1, as.nanoduration(e2)))
          })
##' @rdname nanoduration
setMethod("-", c("nanoival", "difftime"),
          function(e1, e2) {
              new("nanoival", nanoival_plus_impl(e1, -as.nanoduration(e2)))
          })
##' @rdname nanoduration
setMethod("+", c("difftime", "nanoival"),
          function(e1, e2) {
              new("nanoival", nanoival_plus_impl(e2, as.nanoduration(e1)))
          })

##' @noRd
setMethod("+", c("ANY", "nanoduration"),
          function(e1, e2) {
              stop("invalid operand types")
          })
##' @rdname nanoduration
setMethod("+", c("integer64", "nanoduration"),
          function(e1, e2) {
              new("nanoduration", e1 + S3Part(e2, strictS3=TRUE))
          })
##' @rdname nanoduration
setMethod("+", c("numeric", "nanoduration"),
          function(e1, e2) {
              new("nanoduration", e1 + S3Part(e2, strictS3=TRUE))
          })
##' @rdname nanoduration
setMethod("+", c("difftime", "nanoduration"),
          function(e1, e2) {
              as.nanoduration(e1) + e2
          })

## ----------- `*`

##' @noRd
setMethod("*", c("nanoduration", "nanoduration"),
          function(e1, e2) {
              stop("invalid operand types")
          })
##' @rdname nanoduration
setMethod("*", c("nanoduration", "numeric"),
          function(e1, e2) {
              new("nanoduration", S3Part(e1, strictS3=TRUE) * e2)
          })
##' @rdname nanoduration
setMethod("*", c("nanoduration", "integer64"),
          function(e1, e2) {
              new("nanoduration", S3Part(e1, strictS3=TRUE) * e2)
          })
##' @rdname nanoduration
setMethod("*", c("numeric", "nanoduration"),
          function(e1, e2) {
              new("nanoduration", e1 * S3Part(e2, strictS3=TRUE))
          })
##' @rdname nanoduration
setMethod("*", c("integer64", "nanoduration"),
          function(e1, e2) {
              new("nanoduration", e1 * S3Part(e2, strictS3=TRUE))
          })
##' @noRd
setMethod("*", c("nanoduration", "ANY"),
          function(e1, e2) {
              stop("invalid operand types")
          })
##' @noRd
setMethod("*", c("ANY", "nanoduration"),
          function(e1, e2) {
              stop("invalid operand types")
          })

## ----------- `/`

##' @rdname nanoduration
setMethod("/", c("nanoduration", "nanoduration"),
          function(e1, e2) {
              S3Part(e1, strictS3=TRUE) / S3Part(e2, strictS3=TRUE)
          })
##' @rdname nanoduration
setMethod("/", c("nanoduration", "integer64"),
          function(e1, e2) {
              new("nanoduration", as.integer64(S3Part(e1, strictS3=TRUE) / e2))
          })
##' @rdname nanoduration
setMethod("/", c("nanoduration", "numeric"),
          function(e1, e2) {
              new("nanoduration", as.integer64(S3Part(e1, strictS3=TRUE) / e2))
          })
##' @noRd
setMethod("/", c("nanoduration", "ANY"),
          function(e1, e2) {
              stop("invalid operand types")
          })
##' @noRd
setMethod("/", c("ANY", "nanoduration"),
          function(e1, e2) {
              stop("invalid operand types")
          })

## ---------- other ops

##' @rdname nanoduration
setMethod("Arith", c("nanoduration", "ANY"),
          function(e1, e2) {
              callNextMethod(S3Part(e1, strictS3=TRUE), e2)
          })

##' @rdname nanoduration
setMethod("Compare", c("nanoduration", "character"),
          function(e1, e2) {
              ne2 <- as.nanoduration(e2)
              callNextMethod(e1, ne2)
          })

##' @rdname nanoduration
setMethod("Compare", c("character", "nanoduration"),
          function(e1, e2) {
              ne1 <- as.nanoduration(e1)
              callNextMethod(ne1, e2)
          })

##' @rdname nanoduration
setMethod("Compare", c("nanoduration", "ANY"),
          function(e1, e2) {
              callNextMethod(S3Part(e1, strictS3=TRUE), e2)
          })

##' @noRd
setMethod("Logic", c("nanoduration", "nanoduration"),
          function(e1, e2) {
              ## this is the same error message that R gives for "A" | "A"
              stop("operations are possible only for numeric, logical or complex types")
          })

##' @noRd
setMethod("Logic", c("nanoduration", "ANY"),
          function(e1, e2) {
              ## this is the same error message that R gives for "A" | "A"
              stop("operations are possible only for numeric, logical or complex types")
          })

##' @noRd
setMethod("Logic", c("ANY", "nanoduration"),
          function(e1, e2) {
              ## this is the same error message that R gives for "A" | "A"
              stop("operations are possible only for numeric, logical or complex types")
          })

##' @rdname nanoduration
setMethod("abs", c("nanoduration"),
          function(x) {
              new("nanoduration", callNextMethod(S3Part(x, strictS3=TRUE)))
          })
##' @rdname nanoduration
setMethod("sign", c("nanoduration"),
          function(x) {
              callNextMethod(S3Part(x, strictS3=TRUE))
          })


##' @noRd
setMethod("Math", c("nanoduration"),
          function(x) {
              ## this is the same error message that R gives for abs("A")
              stop("non-numeric argument to mathematical function")
          })

##' @noRd
setMethod("Math2", c("nanoduration"),
          function(x, digits) {
              ## this is the same error message that R gives for round("A")
              stop("non-numeric argument to mathematical function")
          })

##' @noRd
setMethod("Summary", c("nanoduration"),
          function(x, ..., na.rm = FALSE) {
              ## this is the same error message that R gives for sum("A")
              stop("invalid 'type' (nanoduration) of argument")
          })

##' @rdname nanoduration
setMethod("sum", c("nanoduration"),
          function(x, ..., na.rm = FALSE) {
              new("nanoduration", callNextMethod())
          })

##' @rdname nanoduration
setMethod("min", c("nanoduration"),
          function(x, ..., na.rm = FALSE) {
              new("nanoduration", callNextMethod())
          })

##' @rdname nanoduration
setMethod("max", c("nanoduration"),
          function(x, ..., na.rm = FALSE) {
              new("nanoduration", callNextMethod())
          })

##' @rdname nanoduration
setMethod("range", c("nanoduration"),
          function(x, ..., na.rm = FALSE) {
              new("nanoduration", callNextMethod())
          })


##' @noRd
setMethod("Complex", c("nanoduration"),
          function(z) {
              ## this is the same error message that R gives for Arg("A")
              stop("non-numeric argument to function")
          })


## non-ops:
## -------

## subset/subassign

##' @rdname nanoduration
setMethod("[[",
          signature("nanoduration"),
          function (x, i, j, ..., drop=FALSE) {
              new("nanoduration", callNextMethod())
          })

##' @rdname nanoduration
setMethod("[",
          signature("nanoduration", "numeric"),
          function (x, i, j, ..., drop=FALSE) {
              if (!missing(j) || length(list(...)) > 0) {
                  warning("unused indices or arguments in 'nanoduration' subsetting")
              }
              if (isTRUE(any(i < 0))) {
                  new("nanoduration", S3Part(x, strictS3=TRUE)[i])
              } else {            
                  nanoduration_subset_numeric_impl(x, i)
              }
          })

##' @rdname nanoduration
setMethod("[",
          signature("nanoduration", "logical"),
          function (x, i, j, ..., drop=FALSE) {
              if (!missing(j) || length(list(...)) > 0) {
                  warning("unused indices or arguments in 'nanoduration' subsetting")
              }
              nanoduration_subset_logical_impl(x, i)
          })

##' @rdname nanoduration
setMethod("[",
          signature("nanoduration", "character"),
          function (x, i, j, ..., drop=FALSE) {
              ## we don't implement 'period_subset_character_impl' but
              ## do the gymnastic of finding the NAs here at R level;
              ## it's not as efficient, but using 'character' indexing
              ## is by itself inefficient, so the overhead should have
              ## no practical consequences.
              if (!missing(j) || length(list(...)) > 0) {
                  warning("unused indices or arguments in 'nanoduration' subsetting")
              }
              na_index <- !(i %in% names(x)) | is.na(i)
              res <- new("nanoduration", S3Part(x, strictS3=TRUE)[i])
              res[na_index] <- NA_nanoduration_
              res
          })

##' @rdname nanoduration
setMethod("[",
          signature("nanoduration", "ANY"),
          function (x, i, j, ..., drop=FALSE) {
              stop("']' not defined on 'nanoduration' for index of type 'ANY'")
          })

##' @rdname nanoduration
setMethod("[<-",
          signature("nanoduration"),
          function (x, i, j, ..., value) {
              new("nanoduration", callNextMethod())
          })

##' @rdname nanoduration
c.nanoduration <- function(...) {
    as.nanoduration(c.integer64(...))
}

##' @rdname nanotime
as.data.frame.nanoduration <- function(x, ...) {
    ret <- as.data.frame(S3Part(x, strictS3=TRUE), ...)
    ## this works, but see if there's a more idiomatic and efficient way
    ## of doing this:
    ret[] <- as.nanoduration(S3Part(x, strictS3=TRUE))
    ret
}

##' Sequence Generation
##'
##' Generate a sequence of \code{nanoduration}
##'
##' @param ... arguments passed to or from methods
##' @param from,to the starting and (maximal) end values of the
##'     sequence
##' @param by the increment of the sequence
##' @param length.out integer indicating the desired length of the sequence
##' @param along.with take the length from the length of this argument.
##'
##' @examples
##' seq(from=as.nanoduration(0), by=as.nanoduration("01:00:00"), length.out=10)
##' @method seq nanoduration
seq.nanoduration <- function(from, to=NULL, by=NULL, length.out=NULL, along.with=NULL, ...) {
    return(as.nanoduration(seq(as.integer64(from), to, by, length.out, along.with, ...)))
}


##' Test if Two Objects are (Nearly) Equal
##'
##' Compare \code{target} and \code{current} testing \sQuote{near
##' equality}.  If they are different, comparison is still made to
##' some extent, and a report of the differences is returned.  Do not
##' use \code{all.equal} directly in \code{if} expressions---either
##' use \code{isTRUE(all.equal(....))} or \code{\link{identical}} if
##' appropriate.
##'
##' @param target,current \code{nanoduration} arguments to be compared
##' @param ... further arguments for different methods
##' @param tolerance numeric >= 0. Differences smaller than
##'     \code{tolerance} are not reported.  The default value is close
##'     to \code{1.5e-8}.
##' @param  scale \code{NULL} or numeric > 0, typically of length 1 or
##'          \code{length(target)}.  See \sQuote{Details}.
##' @param  countEQ logical indicating if the \code{target == current} cases should be
##'          counted when computing the mean (absolute or relative)
##'          differences.  The default, \code{FALSE} may seem misleading in
##'          cases where \code{target} and \code{current} only differ in a few
##'          places; see the extensive example.
##' @param formatFUN a \code{function} of two arguments, \code{err}, the relative, absolute
##'          or scaled error, and \code{what}, a character string indicating
##'          the _kind_ of error; maybe used, e.g., to format relative and
##'          absolute errors differently.
##' @param check.attributes logical indicating if the \code{attributes} of \code{target}
##'          and \code{current} (other than the names) should be compared.
##'
##' @seealso \code{\link{identical}}, \code{\link{isTRUE}},
##'     \code{\link{==}}, and \code{\link{all}} for exact equality
##'     testing.
##'
##' @method all.equal nanoduration
##'
all.equal.nanoduration <- function(target, current, tolerance = sqrt(.Machine$double.eps), 
                                   scale = NULL, countEQ = FALSE,
                                   formatFUN = function(err, what) format(err), ..., check.attributes = TRUE) {
    ## a bit of a kludge, but both 'nanoduration' and 'nanotime' are
    ## based on 'integer64'; because of a bug in 'all.equal.integer64'
    ## we have to redefine 'all.equal.nanotime' and it is correct for
    ## all 'integer64'-based S4 classes; in theory they would all
    ## three be the same function:
    if (inherits(target, "nanoduration")) target  <- as.nanotime(target)
    if (inherits(current, "nanoduration")) current <- as.nanotime(current)
    all.equal.nanotime(target, current, tolerance, scale, countEQ, formatFUN, ..., check.attributes=check.attributes)
}

##' @rdname all.equal.nanoduration
setMethod("all.equal", c(target = "nanoduration", current="ANY"), all.equal.nanoduration)

##' @rdname nanoduration
NA_nanoduration_  <- as.nanoduration(NA_integer_)


## rounding ops:

##' @rdname rounding
setMethod("nano_ceiling", c(x="nanotime", precision="nanoduration"),
          function(x, precision, origin=nanotime()) {
              if (!inherits(origin, "nanotime")) {
                  stop("'origin' must be of class 'nanotime'")
              }
              ceiling_impl(x, precision, origin)
          })

##' @rdname rounding
setMethod("nano_floor",   c(x="nanotime", precision="nanoduration"),
          function(x, precision, origin=nanotime()) {
              if (!inherits(origin, "nanotime")) {
                  stop("'origin' must be of class 'nanotime'")
              }
              floor_impl(x, precision, origin)
          })


##' Replicate Elements
##'
##' Replicates the values in 'x' similarly to the default method.
##'
##' @param x a vector of \code{nanoduration}
##' @param ... further arguments:
##' 
##'        'times' an integer-valued vector giving the (non-negative)
##'            number of times to repeat each element if of length
##'            'length(x)', or to repeat the whole vector if of length
##'            1. Negative or 'NA' values are an error. A 'double'
##'            vector is accepted, other inputs being coerced to an
##'            integer or double vector.
##' 
##'        'length.out' non-negative integer. The desired length of the
##'            output vector. Other inputs will be coerced to a double
##'            vector and the first element taken. Ignored if 'NA' or
##'            invalid.
##' 
##'        'each' non-negative integer. Each element of 'x' is repeated
##'            'each' times.  Other inputs will be coerced to an integer
##'            or double vector and the first element taken. Treated as
##'            '1' if 'NA' or invalid.
setMethod("rep", c(x = "nanoduration"), function(x, ...) {
    new("nanoduration", callNextMethod())
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nanotime.R
\name{nano_wday}
\alias{nano_wday}
\alias{nano_wday,nanotime-method}
\alias{nano_mday,nanotime-method}
\alias{nano_month,nanotime-method}
\alias{nano_year,nanotime-method}
\alias{nano_mday}
\alias{nano_month}
\alias{nano_year}
\title{Get a component of a date time}
\usage{
nano_wday(x, tz)

nano_mday(x, tz)

nano_month(x, tz)

nano_year(x, tz)
}
\arguments{
\item{x}{a \code{nanotime} object}

\item{tz}{\code{character} a string representing a timezone}
}
\description{
Get a component of a date time. \code{nano_wday} returns the
numeric position in a week, with Sunday == 0. \code{nano_mday}
returns the numeric day (i.e. a value from 1 to
31). \code{nano_month} returns the month (i.e. a value from 1 to
12). \code{nano_year} returns the year.
}
\details{
Note that the \code{tz} parameter is mandatory because the day
boundary is different depending on the time zone and
\code{nanotime} does not store the timezone as it is just an
offset in nanoseconds from the epoch.
}
\examples{
\dontrun{
nano_wday(as.nanotime("2020-03-14 23:32:00-04:00"), "America/New_York")
nano_wday(as.nanotime("2020-03-14 23:32:00 America/New_York"), "Europe/Paris")
nano_mday(as.nanotime("2020-03-14 23:32:00-04:00"), "America/New_York")
nano_mday(as.nanotime("2020-03-14 23:32:00 America/New_York"), "Europe/Paris")
nano_month(as.nanotime("2020-12-31 23:32:00-04:00"), "America/New_York")
nano_month(as.nanotime("2020-12-31 23:32:00 America/New_York"), "Europe/Paris")
nano_year(as.nanotime("2020-12-31 23:32:00-04:00"), "America/New_York")
nano_year(as.nanotime("2020-12-31 23:32:00 America/New_York"), "Europe/Paris")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nanotime.R, R/nanoduration.R
\docType{class}
\name{nanotime-class}
\alias{nanotime-class}
\alias{nanotime}
\alias{as.nanotime}
\alias{nanotime,character-method}
\alias{as.nanotime,character-method}
\alias{nanotime.matrix}
\alias{nanotime,POSIXct-method}
\alias{as.nanotime,POSIXct-method}
\alias{nanotime,POSIXlt-method}
\alias{as.nanotime,POSIXlt-method}
\alias{nanotime,Date-method}
\alias{as.nanotime,Date-method}
\alias{print,nanotime-method}
\alias{show,nanotime-method}
\alias{unique,nanotime-method}
\alias{format.nanotime}
\alias{index2char.nanotime}
\alias{as.POSIXct.nanotime}
\alias{as.POSIXlt.nanotime}
\alias{as.Date.nanotime}
\alias{as.data.frame.nanotime}
\alias{as.integer64.nanotime}
\alias{-,nanotime,character-method}
\alias{-,nanotime,nanotime-method}
\alias{-,nanotime,integer64-method}
\alias{-,nanotime,numeric-method}
\alias{-,ANY,nanotime-method}
\alias{-,nanotime,ANY-method}
\alias{+,nanotime,ANY-method}
\alias{+,nanotime,integer64-method}
\alias{+,nanotime,numeric-method}
\alias{+,ANY,nanotime-method}
\alias{+,integer64,nanotime-method}
\alias{+,numeric,nanotime-method}
\alias{+,nanotime,nanotime-method}
\alias{Arith,nanotime,nanotime-method}
\alias{Arith,nanotime,ANY-method}
\alias{Arith,ANY,nanotime-method}
\alias{Compare,nanotime,character-method}
\alias{Compare,character,nanotime-method}
\alias{Compare,nanotime,POSIXt-method}
\alias{Compare,POSIXt,nanotime-method}
\alias{Compare,nanotime,ANY-method}
\alias{Logic,nanotime,ANY-method}
\alias{Logic,ANY,nanotime-method}
\alias{Math,nanotime-method}
\alias{Math2,nanotime-method}
\alias{Summary,nanotime-method}
\alias{min,nanotime-method}
\alias{max,nanotime-method}
\alias{range,nanotime-method}
\alias{Complex,nanotime-method}
\alias{[[,nanotime-method}
\alias{[,nanotime,numeric-method}
\alias{[,nanotime,logical-method}
\alias{[,nanotime,character-method}
\alias{[,nanotime,ANY-method}
\alias{[<-,nanotime,ANY,ANY,ANY-method}
\alias{c.nanotime}
\alias{nanotime-package}
\alias{names<-,nanotime-method}
\alias{is.na,nanotime-method}
\alias{NA_nanotime_}
\alias{as.character.nanotime}
\alias{as.data.frame.nanoduration}
\title{Nanosecond resolution datetime functionality}
\format{
An object of class \code{nanotime} of length 1.
}
\usage{
nanotime(from, ...)

as.nanotime(from, ...)

\S4method{nanotime}{character}(from, format = "", tz = "")

\S4method{as.nanotime}{character}(from, format = "", tz = "")

nanotime.matrix(x)

\S4method{nanotime}{POSIXct}(from, accurate = TRUE)

\S4method{as.nanotime}{POSIXct}(from, accurate = TRUE)

\S4method{nanotime}{POSIXlt}(from)

\S4method{as.nanotime}{POSIXlt}(from)

\S4method{nanotime}{Date}(from)

\S4method{as.nanotime}{Date}(from)

\S4method{print}{nanotime}(x, format = "", tz = "", quote = FALSE, ...)

\S4method{show}{nanotime}(object)

\method{format}{nanotime}(x, format = "", tz = "", ...)

\method{index2char}{nanotime}(x, ...)

\method{as.POSIXct}{nanotime}(x, tz = "", ...)

\method{as.POSIXlt}{nanotime}(x, tz = "", ...)

\method{as.Date}{nanotime}(x, ...)

\method{as.data.frame}{nanotime}(x, ...)

\method{as.integer64}{nanotime}(x, ...)

\S4method{-}{nanotime,character}(e1, e2)

\S4method{-}{nanotime,nanotime}(e1, e2)

\S4method{-}{nanotime,integer64}(e1, e2)

\S4method{-}{nanotime,numeric}(e1, e2)

\S4method{-}{ANY,nanotime}(e1, e2)

\S4method{-}{nanotime,ANY}(e1, e2)

\S4method{+}{nanotime,ANY}(e1, e2)

\S4method{+}{nanotime,integer64}(e1, e2)

\S4method{+}{nanotime,numeric}(e1, e2)

\S4method{+}{ANY,nanotime}(e1, e2)

\S4method{+}{integer64,nanotime}(e1, e2)

\S4method{+}{numeric,nanotime}(e1, e2)

\S4method{+}{nanotime,nanotime}(e1, e2)

\S4method{Arith}{nanotime,nanotime}(e1, e2)

\S4method{Arith}{nanotime,ANY}(e1, e2)

\S4method{Arith}{ANY,nanotime}(e1, e2)

\S4method{Compare}{nanotime,character}(e1, e2)

\S4method{Compare}{character,nanotime}(e1, e2)

\S4method{Compare}{nanotime,POSIXt}(e1, e2)

\S4method{Compare}{POSIXt,nanotime}(e1, e2)

\S4method{Compare}{nanotime,ANY}(e1, e2)

\S4method{Logic}{nanotime,ANY}(e1, e2)

\S4method{Logic}{ANY,nanotime}(e1, e2)

\S4method{Math}{nanotime}(x)

\S4method{Math2}{nanotime}(x, digits)

\S4method{Summary}{nanotime}(x, ..., na.rm = FALSE)

\S4method{min}{nanotime}(x, ..., na.rm = FALSE)

\S4method{max}{nanotime}(x, ..., na.rm = FALSE)

\S4method{range}{nanotime}(x, ..., na.rm = FALSE)

\S4method{Complex}{nanotime}(z)

\S4method{[[}{nanotime}(x, i, j, ..., drop = FALSE)

\S4method{[}{nanotime,numeric}(x, i, j, ..., drop = FALSE)

\S4method{[}{nanotime,logical}(x, i, j, ..., drop = FALSE)

\S4method{[}{nanotime,character}(x, i, j, ..., drop = FALSE)

\S4method{[}{nanotime,ANY}(x, i, j, ..., drop = FALSE)

\S4method{[}{nanotime,ANY,ANY,ANY}(x, i, j, ...) <- value

\method{c}{nanotime}(...)

\S4method{names}{nanotime}(x) <- value

\S4method{is.na}{nanotime}(x)

NA_nanotime_

\method{as.character}{nanotime}(x, ...)

\method{as.data.frame}{nanoduration}(x, ...)
}
\arguments{
\item{...}{further arguments passed to or from methods.}

\item{format}{A character string. Can also be set via
\code{options("nanotimeFormat")} and uses
\sQuote{\%Y-\%m-\%dT\%H:\%M:\%E9S\%Ez} as a default and
fallback}

\item{tz}{character specifying a timezone which is required for
\code{as.POSIXct}, \code{as.POSIXlt} and can be specified for
\code{as.nanotime}, \code{format} and \code{print}; it can
also be set via \code{options("nanotimeTz")} and uses
\sQuote{UTC} as a default and fallback}

\item{x, from}{\code{nanotime} objects}

\item{accurate}{in the conversion from \code{POSIXct} to
\code{nanotime}, indicates if one wants to preserve the
maximum precision possible; the default is \code{TRUE}, but in
most situations the loss of precision is negligible, and
setting this parameter to \code{TRUE} will make the conversion
nearly an order of magnitude faster}

\item{quote}{indicates if the output of \code{print} should be
quoted}

\item{object}{argument for method \code{show}}

\item{e1}{Operand of class \code{nanotime}}

\item{e2}{Operand of class \code{nanotime}}

\item{digits}{Required for \code{Math2} signature but ignored here}

\item{na.rm}{a logical indicating whether missing values should be
removed.}

\item{z}{Required for \code{Complex} signature but ignored here}

\item{i}{index specifying elements to extract or replace.}

\item{j}{Required for \code{[} signature but ignored here}

\item{drop}{Required for \code{[} signature but ignored here}

\item{value}{argument for \code{nanotime-class}}
}
\value{
A nanotime object
}
\description{
Functions to operate on nanosecond time resolution using integer64
bit representation. Conversion functions for several standard R
types are provided, and more will be added as needed.
}
\details{
Notice that the conversion from POSIXct explicitly sets the last
three digits to zero. Nanosecond time stored in a 64-bit integer
has nineteen digits precision where doubles (which are used
internally for POSIXct as well) only have sixteen digits.  So
rather than showing three more (essentially \emph{random}) digits
it is constructed such that these three additional digits are
zeros.
}
\section{Caveats}{


Working with dates and times is \emph{difficult}. One needs a
representation of both \emph{time points} and \emph{time
duration}. In R, think of \code{Date} or \code{POSIXct} objects
for the former, and \code{difftime} for the later. Here we have
time points \code{nanotime}, an interval type \code{nanoival} and
two flavors of duration which are a simple count of nanoseconds
\code{nanoduration} and a calendar duration that is able to track
concepts such as months and days \code{nanoperiod}. Point in time
and intervals are all based on durations relative to the epoch of
January 1, 1970.
}

\section{Input and Output Format}{


Formatting and character conversion for \code{nanotime} objects is
done by functions from the \pkg{RcppCCTZ} package relying
on code from its embedded \code{CCTZ} library. The default format
is ISO3339 compliant: \code{\%Y-\%m-\%dT\%H:\%M:\%E9S\%Ez}. It
specifies a standard ISO 8601 part for date and time --- as well
as nine digits of precision for fractional seconds (down to
nanoseconds) and on offset (typically zero as we default to UTC).
It can be overriden by using \code{options()} with the key of
\code{nanotimeFormat} and a suitable value. Similarly,
\code{nanotimeTz} can be used to select a different timezone.

For input, some slack it cut, and various shortened formats are
accepted by default such as \code{2020-03-10} or \code{2020-03-10
18:16:00}, or \code{2020-03-10 18:16:00.001} (and the \sQuote{T}
separator is optional.
}

\section{\code{tz} parameter usage in constructors}{


The \code{tz} parameter is allowed only when constructing a
\code{nanotime} from a \code{character}. This is because any
\code{numeric}, \code{Date} and \code{POSIXct} is de facto
considered an offset since the epoch. On the contrary, a
\code{character} is considered interpretable and hence if it does
not contain a timezone in its representation, it is possible to
specify the \code{tz} argument to specify in which timezone it
should be interpreted. This is useful in particular if one wants
to convert a \code{Date} to be aligned to the beginning of the day
in a specific timezone; in this case one should convert the
\code{Date} to a \code{character} before calling the
\code{nanotime} constructor with the desired timezone.
}

\examples{
\dontrun{
x <- nanotime(1)
print(x)
as.nanotime("1970-01-01T00:00:00.000000001+00:00")
as.nanotime("2020-03-10 Europe/Berlin")
as.nanotime("2020-03-10 18:31:23.001", tz="America/New_York")
as.nanotime("2020-03-10T040947190301440", format="\%Y-\%m-\%dT\%H\%M\%S\%E*f")
x <- x + 1
print(x)
format(x)
x <- x + 10
print(x)
format(x)
nanotime(Sys.time()) + 1:3  # three elements each 1 ns apart
seq(x, by=as.nanoperiod("1d"), length.out=5, tz="Asia/Tokyo")
}
}
\seealso{
\code{\link{nanoival}}, \code{\link{nanoduration}},
\code{\link{nanoperiod}}, \code{\link{seq.nanotime}} as well as
the documentation in package \pkg{RcppCCTZ}.
}
\author{
Dirk Eddelbuettel

Leonardo Silvestri
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nanoival.R
\name{intersect,nanoival,nanoival-method}
\alias{intersect,nanoival,nanoival-method}
\alias{union,nanoival,nanoival-method}
\alias{setdiff,nanoival,nanoival-method}
\alias{intersect.idx,nanotime,nanoival-method}
\alias{intersect.idx}
\alias{\%in\%.nanotime}
\alias{\%in\%,nanotime,nanoival-method}
\alias{intersect,nanotime,nanoival-method}
\alias{setdiff,nanotime,nanoival-method}
\alias{setdiff.idx,nanotime,nanoival-method}
\alias{setdiff.idx}
\alias{intersect,nanotime,nanotime-method}
\alias{union,nanotime,nanotime-method}
\alias{setdiff,nanotime,nanotime-method}
\title{Set operations}
\usage{
\S4method{intersect}{nanoival,nanoival}(x, y)

\S4method{union}{nanoival,nanoival}(x, y)

\S4method{setdiff}{nanoival,nanoival}(x, y)

\S4method{intersect.idx}{nanotime,nanoival}(x, y)

\method{\%in\%}{nanotime}(x, table)

\S4method{\%in\%}{nanotime,nanoival}(x, table)

\S4method{intersect}{nanotime,nanoival}(x, y)

\S4method{setdiff}{nanotime,nanoival}(x, y)

\S4method{setdiff.idx}{nanotime,nanoival}(x, y)

\S4method{intersect}{nanotime,nanotime}(x, y)

\S4method{union}{nanotime,nanotime}(x, y)

\S4method{setdiff}{nanotime,nanotime}(x, y)
}
\arguments{
\item{x, y}{a temporal type}

\item{table}{\code{nanoival}: used in \code{\%in\%}}
}
\value{
\code{intersect}, \code{union}, \code{setdiff} return
    temporal types that are the result of the intersection. For
    instance, set operations on two \code{nanoival} return a
    \code{nanoival}, whereas intersection between a
    \code{nanoival} and a \code{nanotime} returns a
    \code{nanotime}. \code{intersect.idx} return a list of vectors
    representing the element indices that intersect and
    \code{setdiff.idx} returns a vector representing the element
    indices to be removed.
}
\description{
Performs set intersection, union and difference between vectors of
temporal types from the \code{nanotime} package.
}
\details{
Set operations between \code{nanoival} operands allow the
construction of complex interval vectors (i.e. a \code{nanoival}
vector can specify any number of inclusions and exclusions of
time). Set operations between \code{nanotime} and \code{nanoival}
allow to subset time vectors with interval vectors. In addition to
the generic set functions, the function \code{intersect.idx} is
defined which returns the indices of the intersection, and the
operator \code{\%in\%} is overloaded for \code{nanotime-nanoival}
which returns a logical vector that indicates which elements
belong to the interval vector.
}
\examples{
\dontrun{
## a vector of 'nanotime' can be subsetted by a 'nanoival' which is equivalent to 'intersect':
one_second <- 1e9
a <- seq(nanotime("2012-12-12 12:12:12+00:00"), length.out=10, by=one_second)
idx <- c(as.nanoival("-2012-12-12 12:12:10+00:00 -> 2012-12-12 12:12:14+00:00-"),
         as.nanoival("+2012-12-12 12:12:18+00:00 -> 2012-12-12 12:12:20+00:00+"))
a[idx]
intersect(a, idx)

## 'nanoival' also has the set operations 'union', 'intersect', 'setdiff':
a <- seq(nanotime("2012-12-12 12:12:12+00:00"), length.out=10, by=one_second)
i <- as.nanoival("-2012-12-12 12:12:14+00:00 -> 2012-12-12 12:12:18+00:00-")
setdiff(a, i)

i1 <- as.nanoival("+2012-12-12 12:12:14+00:00 -> 2012-12-12 12:12:17+00:00-")
i2 <- as.nanoival("+2012-12-12 12:12:16+00:00 -> 2012-12-12 12:12:18+00:00-")
union(i1, i2)

## 'intersect.idx' returns the indices of the intersection:
a <- seq(nanotime("2012-12-12 12:12:12+00:00"), length.out=10, by=one_second)
idx <- as.nanoival("+2012-12-12 12:12:14+00:00 -> 2012-12-12 12:12:19+00:00+")
idx_intersect <- intersect.idx(a, idx)

## Intersection can be performed using these indices:
a[idx_intersect$x]

## which is equivalent to:
a[idx]

## The logical vector indicating intersection can be obtained like this:
a \%in\% idx
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nanoduration.R
\name{seq.nanoduration}
\alias{seq.nanoduration}
\title{Sequence Generation}
\usage{
\method{seq}{nanoduration}(from, to = NULL, by = NULL, length.out = NULL, along.with = NULL, ...)
}
\arguments{
\item{from, to}{the starting and (maximal) end values of the
sequence}

\item{by}{the increment of the sequence}

\item{length.out}{integer indicating the desired length of the sequence}

\item{along.with}{take the length from the length of this argument.}

\item{...}{arguments passed to or from methods}
}
\description{
Generate a sequence of \code{nanoduration}
}
\examples{
seq(from=as.nanoduration(0), by=as.nanoduration("01:00:00"), length.out=10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nanoival.R
\docType{class}
\name{nanoival-class}
\alias{nanoival-class}
\alias{nanoival}
\alias{+,ANY,nanoival-method}
\alias{+,nanoival,ANY-method}
\alias{+,nanoival,nanoival-method}
\alias{-,ANY,nanoival-method}
\alias{-,nanoival,ANY-method}
\alias{-,nanoival,nanoival-method}
\alias{Arith,nanoival,ANY-method}
\alias{Compare,nanoival,ANY-method}
\alias{Complex,nanoival-method}
\alias{Logic,ANY,nanoival-method}
\alias{Logic,nanoival,ANY-method}
\alias{Logic,nanoival,nanoival-method}
\alias{Math2,nanoival-method}
\alias{Math,nanoival-method}
\alias{Summary,nanoival-method}
\alias{nanoival.start,nanoival-method}
\alias{nanoival.start}
\alias{nanoival.end,nanoival-method}
\alias{nanoival.end}
\alias{nanoival.sopen,nanoival-method}
\alias{nanoival.sopen}
\alias{nanoival.eopen,nanoival-method}
\alias{nanoival.eopen}
\alias{format.nanoival}
\alias{print,nanoival-method}
\alias{show,nanoival-method}
\alias{as.nanoival,character-method}
\alias{as.nanoival}
\alias{as.nanoival,NULL-method}
\alias{as.nanoival,missing-method}
\alias{is.na,nanoival-method}
\alias{is.na<-,nanoival-method}
\alias{<,nanoival,nanoival-method}
\alias{<=,nanoival,nanoival-method}
\alias{>,nanoival,nanoival-method}
\alias{>=,nanoival,nanoival-method}
\alias{==,nanoival,nanoival-method}
\alias{!=,nanoival,nanoival-method}
\alias{-,nanoival,integer64-method}
\alias{-,nanoival,numeric-method}
\alias{+,nanoival,integer64-method}
\alias{+,nanoival,numeric-method}
\alias{+,integer64,nanoival-method}
\alias{+,numeric,nanoival-method}
\alias{[[,nanoival-method}
\alias{[,nanoival,logical-method}
\alias{[,nanoival,numeric-method}
\alias{[,nanoival,character-method}
\alias{[,nanoival,ANY-method}
\alias{[<-,nanoival,logical,ANY,nanoival-method}
\alias{c.nanoival}
\alias{t,nanoival-method}
\alias{[,nanotime,nanoival-method}
\alias{NA_nanoival_}
\alias{as.character.nanoival}
\title{Interval type with nanosecond precision}
\format{
An object of class \code{nanoival} of length 1.
}
\usage{
nanoival(start, end, sopen = FALSE, eopen = TRUE)

\S4method{nanoival.start}{nanoival}(x)

\S4method{nanoival.end}{nanoival}(x)

\S4method{nanoival.sopen}{nanoival}(x)

\S4method{nanoival.eopen}{nanoival}(x)

\method{format}{nanoival}(x, ...)

\S4method{print}{nanoival}(x, quote = FALSE, ...)

\S4method{show}{nanoival}(object)

\S4method{as.nanoival}{character}(from, format = "", tz = "")

\S4method{as.nanoival}{`NULL`}(from, format = "", tz = "")

\S4method{as.nanoival}{missing}(from, format = "", tz = "")

\S4method{is.na}{nanoival}(x)

\S4method{is.na}{nanoival}(x) <- value

\S4method{<}{nanoival,nanoival}(e1, e2)

\S4method{<=}{nanoival,nanoival}(e1, e2)

\S4method{>}{nanoival,nanoival}(e1, e2)

\S4method{>=}{nanoival,nanoival}(e1, e2)

\S4method{==}{nanoival,nanoival}(e1, e2)

\S4method{!=}{nanoival,nanoival}(e1, e2)

\S4method{-}{nanoival,integer64}(e1, e2)

\S4method{-}{nanoival,numeric}(e1, e2)

\S4method{+}{nanoival,integer64}(e1, e2)

\S4method{+}{nanoival,numeric}(e1, e2)

\S4method{+}{integer64,nanoival}(e1, e2)

\S4method{+}{numeric,nanoival}(e1, e2)

\S4method{[[}{nanoival}(x, i, j, ..., drop = FALSE)

\S4method{[}{nanoival,logical}(x, i, j, ..., drop = FALSE)

\S4method{[}{nanoival,numeric}(x, i, j, ..., drop = FALSE)

\S4method{[}{nanoival,character}(x, i, j, ..., drop = FALSE)

\S4method{[}{nanoival,ANY}(x, i, j, ..., drop = FALSE)

\S4method{[}{nanoival,logical,ANY,nanoival}(x, i, j, ...) <- value

\method{c}{nanoival}(...)

\S4method{t}{nanoival}(x)

\S4method{[}{nanotime,nanoival}(x, i, j, ..., drop = TRUE)

NA_nanoival_

\method{as.character}{nanoival}(x, ...)
}
\arguments{
\item{start}{\code{nanotime} start of interval}

\item{end}{\code{nanotime} end of interval}

\item{sopen}{logical indicating if the start of the interval is open}

\item{eopen}{logical indicating if the end of the interval is open}

\item{x, from}{a \code{nanoival} object}

\item{...}{further arguments passed to or from methods.}

\item{quote}{indicates if the output of \code{print} should be
quoted}

\item{object}{argument for method \code{show}}

\item{format}{A character string. Can also be set via
\code{options("nanotimeFormat")} and uses
\sQuote{\%Y-\%m-\%dT\%H:\%M:\%E9S\%Ez} as a default and
fallback}

\item{tz}{\code{character} indicating a timezone}

\item{value}{argument for \code{nanoival-class}}

\item{e1}{Operand of class \code{nanoival}}

\item{e2}{Operand of class \code{nanoival}}

\item{i}{index specifying elements to extract or replace.}

\item{j}{Required for \code{[} signature but ignored here}

\item{drop}{Required for \code{[} signature but ignored here}
}
\value{
A nanoival object
}
\description{
\code{nanoival} is a time interval type (an S4 class) with
nanosecond precision. One of its purposes is to allow quick
subsetting of a \code{nanotime} vector. \code{nanoival} is
composed of a \code{nanotime} pair which defines the start and end
of the time interval. Additionally, it has a pair of logical
values which determine if the start and end of the time interval
are open (true) or closed (false).
}
\details{
An interval object can be constructed with the constructor
\code{nanoival} which takes as arguments two \code{nanotime}
objects that define the start and the end of the interval,
together with two \code{logical} arguments that define if the
start and the end of the interval are open (true) or closed
(false) (note that these objects can all be vector, and therefore
the interval object is not necessarily scalar). Alternatively, an
interval can be constructed with a \code{character}: the format
follows that of \code{nanotime}; the start time is preceeded by
either \code{-} or \code{+} indicating if the interval start is
open (-) or closed (+); the start and end times are separated by
an arrow \code{->}; the end is folloed by either \code{-} or
\code{+} which have the same semantics as the start time.

The most important set of methods defined for \code{interval} are
set functions \code{intersect}, \code{union} and \code{setdiff}.

Additionally, \code{interval} allows the subsetting into a
\code{nanotime} vector. Note that subsetting is allowed only if
the \code{nanotime} vector is sorted.

Finally, accessors are provided to get the interval start
(\code{start}), the end (\code{end}), the open/close status of the
start (\code{sopen}) and the open/close status of the end
(\code{eopen}). The former return a \code{nanotime} while the
latter return a \code{logical}.
}
\section{Output Format}{


Formatting and character conversion for \code{nanoival} objects is
identical to \code{nanotime} objects. The default format is
ISO3339 compliant: \code{\%Y-\%m-\%dT\%H:\%M:\%E9S\%Ez}. It
specifies a standard ISO 8601 part for date and time --- as well
as nine digits of precision for fractional seconds (down to
nanoseconds) and on offset (typically zero as we default to UTC).
It can be overriden by using \code{options()} with the key of
\code{nanotimeFormat} and a suitable value. Similarly,
\code{nanotimeTz} can be used to select a different timezone.
}

\examples{
\dontrun{
## creating a \code{nanoival}, with the start time included ('+') and the end
## time excluded ('-')
as.nanoival("+2012-03-01T21:21:00.000000001+00:00->2015-01-01T21:22:00.000000999+04:00-")

## a \code{nanoival} can also be created with a pair of \code{nanotime} objects, a start
## and an end, and optionally two logicals determining if the interval start(end) are open
## or closed; by default the start is closed and end is open:
start <- nanotime("2012-03-01T21:21:00.000000001+00:00")
end <- nanotime("2013-03-01T21:21:00.000000001+00:00")
nanoival(start, end)

## a vector of 'nanotime' can be subsetted by a 'nanoival':
one_second <- 1e9
a <- seq(nanotime("2012-12-12 12:12:12+00:00"), length.out=10, by=one_second)
idx <- c(as.nanoival("-2012-12-12 12:12:10+00:00 -> 2012-12-12 12:12:14+00:00-"),
         as.nanoival("+2012-12-12 12:12:18+00:00 -> 2012-12-12 12:12:20+00:00+"))
a[idx]
}
}
\seealso{
\code{\link{intersect.idx}}, \code{\link{setdiff.idx}},
}
\author{
Dirk Eddelbuettel

Leonardo Silvestri
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nanotime.R, R/nanoduration.R, R/nanoperiod.R
\name{nano_ceiling}
\alias{nano_ceiling}
\alias{nano_floor}
\alias{nano_ceiling,nanotime,nanoduration-method}
\alias{nano_floor,nanotime,nanoduration-method}
\alias{nano_ceiling,nanotime,nanoperiod-method}
\alias{nano_floor,nanotime,nanoperiod-method}
\title{Rounding down or up a \code{nanotime} type}
\usage{
nano_ceiling(x, precision, ...)

nano_floor(x, precision, ...)

\S4method{nano_ceiling}{nanotime,nanoduration}(x, precision, origin = nanotime())

\S4method{nano_floor}{nanotime,nanoduration}(x, precision, origin = nanotime())

\S4method{nano_ceiling}{nanotime,nanoperiod}(x, precision, origin = nanotime(), tz)

\S4method{nano_floor}{nanotime,nanoperiod}(x, precision, origin = nanotime(), tz)
}
\arguments{
\item{x}{a \code{nanotime} object which must be sorted}

\item{precision}{a \code{nanoduration} or \code{nanoperiod} object
indicating the rounding precision}

\item{...}{for future additional arguments}

\item{origin}{a \code{nanotime} scalar indicating the origin at which the rounding is considered}

\item{tz}{a \code{character} scalar indicating the time zone in which to conduct the rounding}
}
\description{
The functions \code{nano_floor} and \code{nano_ceiling} round down or up, respectively. Although
the underlying implementation of \code{nanotime} has negative numbers for values before
1970-01-01 UTC, the rounding is always done backward in time for \code{nano_floor} and forward
in time for \code{nano_ceiling}. The functions take a \code{nanotime} argument \code{x} which is
the instance to round, together with a second argument \code{precision} which indicates an
arbitrary precision to which the rounding should be performed. This argument can be either a
\code{nanoduration} or or a \code{nanoperiod}. In the latter case, the argument \code{tz} must
also be specified in order to give the \code{nanoperiod} a meaning. Finally, the \code{nanotime}
argument \code{origin} can be optionally specified to fix the rounding to a specific point in
time.
}
\details{
This flexible rounding must be understood in the context of a vector. The rounding precision can
then be considered as an interval that defines a grid over which the elements are either
assigned to the starting value of the interval to which they belong (\code{nano_floor}) or the
ending value of the interval to which they belong (\code{nano_ceiling}). This allows for a
grouping of a \code{nanotime} vector on which a statistic may then be run. In the examples
below, such a use case is shown in the context of a \code{data.table} object.

If "business" concepts such as month or days are needed, the \code{argument} precision must be
of type \code{period}. It is then mandatory to specify the timezone argument \code{tz} as this
ensures timezone correctness of the intervals including for example for the rare hourly
transitions of some countries going from a timezone with a whole hour difference with UTC to one
with a fractional hour difference. In the case of a \code{period}, the functions align the
rounding if the precision is an integer divisor of a larger quantity. For instance, if one
specifies a rounding of 6 hours, a divisor of a day, the hours are aligned on days and the
rounding is made to a grid at hours 0, 6, 12 and 18 in the specified timezone. If the precision
is not a divisor, the grid is aligned to the nearest hour before the first element of the vector
to round.

The argument \code{origin} controls the reference point of the rounding, allowing arbitrary
specification of the reference point of the rounding.
}
\examples{
\dontrun{
## "classic" rounding:
nano_floor(as.nanotime("2010-10-10 11:12:15 UTC"), as.nanoduration("01:00:00"))
## rounding with arbitrary precision:
nano_floor(as.nanotime("2010-10-10 11:12:15 UTC"), as.nanoduration("06:00:00"))
nano_floor(as.nanotime("2010-10-10 11:23:15 UTC"), as.nanoduration("00:15:00"))
nano_ceiling(as.nanotime("2010-10-10 11:23:15 UTC"), as.nanoduration("01:15:23"))
## controlling the reference point via the 'origin' argument:
nano_ceiling(as.nanotime("2010-10-10 11:23:15 UTC"),
             as.nanoduration("01:15:23"),
             origin=as.nanotime("2010-10-10 11:23:15 UTC"))
## using business concepts and rounding across a daylight saving change:
v <- seq(as.nanotime("2020-03-08 America/New_York"),
         by=as.nanoperiod("06:00:00"), length.out=8, tz="America/New_York")
print(nano_floor(v, as.nanoperiod("1d"), tz="America/New_York"), tz="America/New_York")
## using the concept in a 'data.table':
library(data.table)
n <- 3 * 24
idx <- seq(as.nanotime("2020-03-07 America/New_York"),
           by=as.nanoperiod("01:00:00"), length.out=n, tz="America/New_York")
dt <- data.table(idx, a=1:n, b=2:(n+1))
dt_mean <- dt[, list(mean = mean(a)),
              by=nano_ceiling(idx, as.nanoperiod("1d"), tz="America/New_York")]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nanoival.R
\name{seq,nanoival-method}
\alias{seq,nanoival-method}
\title{Sequence Generation}
\usage{
\S4method{seq}{nanoival}(from, to = NULL, by = NULL, length.out = NULL, along.with = NULL, ...)
}
\arguments{
\item{from, to}{the starting and (maximal) end values of the
sequence}

\item{by}{\code{nanoduration} or \code{nanoperiod} increment of
the sequence; note that if the class is \code{nanoperiod} the
additional argument \code{tz} must be speficied and is of
\code{character} type indicating a timezone}

\item{length.out}{an integer desired length of the sequence}

\item{along.with}{take the length from the length of this argument.}

\item{...}{arguments passed to or from methods; the only
interesting additional argument is \code{tz} where the
\code{to} argument is of type \code{nanoperiod}}
}
\description{
Generate a sequence of \code{nanoival}
}
\examples{
\dontrun{
from <- as.nanoival("-2018-01-14T13:00:00+00:00 -> 2018-01-14T15:00:00+00:00+")
seq(from, by=as.nanoperiod("1m"), length.out=5, tz="America/New_York")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nanoduration.R
\docType{class}
\name{nanoduration-class}
\alias{nanoduration-class}
\alias{nanoduration}
\alias{*,ANY,nanoduration-method}
\alias{*,nanoduration,ANY-method}
\alias{*,nanoduration,nanoduration-method}
\alias{+,ANY,nanoduration-method}
\alias{/,ANY,nanoduration-method}
\alias{/,nanoduration,ANY-method}
\alias{Complex,nanoduration-method}
\alias{Logic,ANY,nanoduration-method}
\alias{Logic,nanoduration,ANY-method}
\alias{Logic,nanoduration,nanoduration-method}
\alias{Math2,nanoduration-method}
\alias{Math,nanoduration-method}
\alias{Summary,nanoduration-method}
\alias{as.nanoduration,character-method}
\alias{as.nanoduration}
\alias{as.nanoduration,integer64-method}
\alias{as.nanoduration,numeric-method}
\alias{as.nanoduration,integer-method}
\alias{as.nanoduration,difftime-method}
\alias{as.nanoduration,NULL-method}
\alias{as.nanoduration,missing-method}
\alias{show,nanoduration-method}
\alias{print,nanoduration-method}
\alias{format.nanoduration}
\alias{as.integer64.nanoduration}
\alias{as.character,nanoduration-method}
\alias{is.na,nanoduration-method}
\alias{-,nanoduration,nanoduration-method}
\alias{-,nanoduration,integer64-method}
\alias{-,nanoduration,integer-method}
\alias{-,nanoduration,numeric-method}
\alias{-,nanoduration,difftime-method}
\alias{-,nanoduration,ANY-method}
\alias{-,nanotime,nanoduration-method}
\alias{-,nanotime,difftime-method}
\alias{-,integer64,nanoduration-method}
\alias{-,integer,nanoduration-method}
\alias{-,numeric,nanoduration-method}
\alias{-,difftime,nanoduration-method}
\alias{-,ANY,nanoduration-method}
\alias{+,nanoduration,ANY-method}
\alias{+,nanoduration,nanoduration-method}
\alias{+,nanoduration,integer64-method}
\alias{+,nanoduration,numeric-method}
\alias{+,nanoduration,difftime-method}
\alias{+,nanotime,nanoduration-method}
\alias{+,nanotime,difftime-method}
\alias{+,nanoduration,nanotime-method}
\alias{+,difftime,nanotime-method}
\alias{+,nanoival,nanoduration-method}
\alias{-,nanoival,nanoduration-method}
\alias{+,nanoduration,nanoival-method}
\alias{+,nanoival,difftime-method}
\alias{-,nanoival,difftime-method}
\alias{+,difftime,nanoival-method}
\alias{+,integer64,nanoduration-method}
\alias{+,numeric,nanoduration-method}
\alias{+,difftime,nanoduration-method}
\alias{*,nanoduration,numeric-method}
\alias{*,nanoduration,integer64-method}
\alias{*,numeric,nanoduration-method}
\alias{*,integer64,nanoduration-method}
\alias{/,nanoduration,nanoduration-method}
\alias{/,nanoduration,integer64-method}
\alias{/,nanoduration,numeric-method}
\alias{Arith,nanoduration,ANY-method}
\alias{Compare,nanoduration,character-method}
\alias{Compare,character,nanoduration-method}
\alias{Compare,nanoduration,ANY-method}
\alias{abs,nanoduration-method}
\alias{sign,nanoduration-method}
\alias{sum,nanoduration-method}
\alias{min,nanoduration-method}
\alias{max,nanoduration-method}
\alias{range,nanoduration-method}
\alias{[[,nanoduration-method}
\alias{[,nanoduration,numeric-method}
\alias{[,nanoduration,logical-method}
\alias{[,nanoduration,character-method}
\alias{[,nanoduration,ANY-method}
\alias{[<-,nanoduration,ANY,ANY,ANY-method}
\alias{c.nanoduration}
\alias{NA_nanoduration_}
\title{Duration type with nanosecond precision}
\format{
An object of class \code{nanoduration} of length 1.
}
\usage{
nanoduration(hours = 0L, minutes = 0L, seconds = 0L, nanoseconds = 0L)

\S4method{as.nanoduration}{character}(x)

\S4method{as.nanoduration}{integer64}(x)

\S4method{as.nanoduration}{numeric}(x)

\S4method{as.nanoduration}{integer}(x)

\S4method{as.nanoduration}{difftime}(x)

\S4method{as.nanoduration}{`NULL`}(x)

\S4method{as.nanoduration}{missing}(x)

\S4method{show}{nanoduration}(object)

\S4method{print}{nanoduration}(x, quote = FALSE, ...)

\method{format}{nanoduration}(x, ...)

\method{as.integer64}{nanoduration}(x, ...)

\S4method{as.character}{nanoduration}(x)

\S4method{is.na}{nanoduration}(x)

\S4method{-}{nanoduration,nanoduration}(e1, e2)

\S4method{-}{nanoduration,integer64}(e1, e2)

\S4method{-}{nanoduration,integer}(e1, e2)

\S4method{-}{nanoduration,numeric}(e1, e2)

\S4method{-}{nanoduration,difftime}(e1, e2)

\S4method{-}{nanoduration,ANY}(e1, e2)

\S4method{-}{nanotime,nanoduration}(e1, e2)

\S4method{-}{nanotime,difftime}(e1, e2)

\S4method{-}{integer64,nanoduration}(e1, e2)

\S4method{-}{integer,nanoduration}(e1, e2)

\S4method{-}{numeric,nanoduration}(e1, e2)

\S4method{-}{difftime,nanoduration}(e1, e2)

\S4method{-}{ANY,nanoduration}(e1, e2)

\S4method{+}{nanoduration,ANY}(e1, e2)

\S4method{+}{nanoduration,nanoduration}(e1, e2)

\S4method{+}{nanoduration,integer64}(e1, e2)

\S4method{+}{nanoduration,numeric}(e1, e2)

\S4method{+}{nanoduration,difftime}(e1, e2)

\S4method{+}{nanotime,nanoduration}(e1, e2)

\S4method{+}{nanotime,difftime}(e1, e2)

\S4method{+}{nanoduration,nanotime}(e1, e2)

\S4method{+}{difftime,nanotime}(e1, e2)

\S4method{+}{nanoival,nanoduration}(e1, e2)

\S4method{-}{nanoival,nanoduration}(e1, e2)

\S4method{+}{nanoduration,nanoival}(e1, e2)

\S4method{+}{nanoival,difftime}(e1, e2)

\S4method{-}{nanoival,difftime}(e1, e2)

\S4method{+}{difftime,nanoival}(e1, e2)

\S4method{+}{integer64,nanoduration}(e1, e2)

\S4method{+}{numeric,nanoduration}(e1, e2)

\S4method{+}{difftime,nanoduration}(e1, e2)

\S4method{*}{nanoduration,numeric}(e1, e2)

\S4method{*}{nanoduration,integer64}(e1, e2)

\S4method{*}{numeric,nanoduration}(e1, e2)

\S4method{*}{integer64,nanoduration}(e1, e2)

\S4method{/}{nanoduration,nanoduration}(e1, e2)

\S4method{/}{nanoduration,integer64}(e1, e2)

\S4method{/}{nanoduration,numeric}(e1, e2)

\S4method{Arith}{nanoduration,ANY}(e1, e2)

\S4method{Compare}{nanoduration,character}(e1, e2)

\S4method{Compare}{character,nanoduration}(e1, e2)

\S4method{Compare}{nanoduration,ANY}(e1, e2)

\S4method{abs}{nanoduration}(x)

\S4method{sign}{nanoduration}(x)

\S4method{sum}{nanoduration}(x, ..., na.rm = FALSE)

\S4method{min}{nanoduration}(x, ..., na.rm = FALSE)

\S4method{max}{nanoduration}(x, ..., na.rm = FALSE)

\S4method{range}{nanoduration}(x, ..., na.rm = FALSE)

\S4method{[[}{nanoduration}(x, i, j, ..., drop = FALSE)

\S4method{[}{nanoduration,numeric}(x, i, j, ..., drop = FALSE)

\S4method{[}{nanoduration,logical}(x, i, j, ..., drop = FALSE)

\S4method{[}{nanoduration,character}(x, i, j, ..., drop = FALSE)

\S4method{[}{nanoduration,ANY}(x, i, j, ..., drop = FALSE)

\S4method{[}{nanoduration,ANY,ANY,ANY}(x, i, j, ...) <- value

\method{c}{nanoduration}(...)

NA_nanoduration_
}
\arguments{
\item{hours}{number of hours}

\item{minutes}{number of minutes}

\item{seconds}{number of seconds}

\item{nanoseconds}{number of nanoseconds}

\item{x}{a \code{nanoduration} object}

\item{object}{argument for method \code{show}}

\item{quote}{indicates if the output of \code{print} should be
quoted}

\item{...}{further arguments passed to or from methods.}

\item{e1}{Operand of class \code{nanoival}}

\item{e2}{Operand of class \code{nanoival}}

\item{na.rm}{if \code{TRUE} NA values are removed for the
computation}

\item{i}{index specifying elements to extract or replace.}

\item{j}{Required for \code{[} signature but ignored here}

\item{drop}{Required for \code{[} signature but ignored here}

\item{value}{argument for \code{nanoduration-class}}
}
\value{
A nanoduration object
}
\description{
The type \code{nanoduration} is a length of time (implemented as
an S4 class) with nanosecond precision. It is a count of
nanoseconds and may be negative. The expected arithmetic
operations are provided, including sequence generation.
}
\details{
A \code{nanoduration} can be constructed with the function
\code{as.nanoduration} which can take the types \code{integer64},
\code{integer} and \code{numeric} (all indicating the count in
nanosecond units) or the type \code{character}.

It can also be constructed by specifying with individual arguments
the hours, minutes, seconds and nanoseconds with a call to
\code{nanoduration}.

A \code{nanoduration} is displayed as hours, minutes, seconds and
nanoseconds like this: \code{110:12:34.123_453_001}. The nanosecond
precision displayed is adjusted as necessary, so e.g. 1 second is
displayed as \code{00:00:01}.
}
\examples{
## constructors:
nanoduration(hours=10, minutes=3, seconds=2, nanoseconds=999999999)
as.nanoduration("10:03:02.999_999_999")
as.nanoduration(36182999999999)

## arithmetic:
as.nanoduration(10e9) - as.nanoduration(9e9)
as.nanoduration(10e9) + as.nanoduration(-9e9)
as.nanoduration("24:00:00") / 2
as.nanoduration("24:00:00") / as.nanoduration("12:00:00")

## comparison:
as.nanoduration("10:03:02.999_999_999") == 36182999999999
as.nanoduration("10:03:02.999_999_999") > as.nanoduration("10:03:02.999_999_998")
as.nanoduration("10:03:02.999_999_998") < "10:03:02.999_999_999"

}
\seealso{
\code{\link{nanotime}}
}
\author{
Dirk Eddelbuettel

Leonardo Silvestri
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nanoperiod.R
\docType{class}
\name{nanoperiod-class}
\alias{nanoperiod-class}
\alias{nanoperiod}
\alias{Compare,ANY,nanoperiod-method}
\alias{Compare,nanoperiod,ANY-method}
\alias{-,ANY,nanoperiod-method}
\alias{-,nanoperiod,nanotime-method}
\alias{-,nanotime,nanoperiod-method}
\alias{/,ANY,nanoperiod-method}
\alias{/,nanoperiod,ANY-method}
\alias{Complex,nanoperiod-method}
\alias{Math,nanoperiod-method}
\alias{Math2,nanoperiod-method}
\alias{Summary,nanoperiod-method}
\alias{minus,nanoperiod,nanoival,character-method}
\alias{as.nanoperiod,character-method}
\alias{as.nanoperiod}
\alias{as.nanoperiod,integer64-method}
\alias{as.nanoperiod,numeric-method}
\alias{as.nanoperiod,integer-method}
\alias{as.nanoperiod,nanoduration-method}
\alias{as.nanoperiod,NULL-method}
\alias{as.nanoperiod,missing-method}
\alias{show,nanoperiod-method}
\alias{print,nanoperiod-method}
\alias{format.nanoperiod}
\alias{as.character,nanoperiod-method}
\alias{is.na,nanoperiod-method}
\alias{is.na<-,nanoperiod-method}
\alias{[[,nanoperiod-method}
\alias{[,nanoperiod,numeric-method}
\alias{[,nanoperiod,logical-method}
\alias{[,nanoperiod,character-method}
\alias{[,nanoperiod,ANY-method}
\alias{[<-,nanoperiod,ANY,ANY,ANY-method}
\alias{c.nanoperiod}
\alias{names,nanoperiod-method}
\alias{names<-,nanoperiod-method}
\alias{-,nanoperiod,ANY-method}
\alias{-,nanoperiod,nanoperiod-method}
\alias{-,nanoperiod,nanoduration-method}
\alias{-,nanoperiod,integer64-method}
\alias{-,nanoperiod,numeric-method}
\alias{-,nanoduration,nanoperiod-method}
\alias{-,integer64,nanoperiod-method}
\alias{-,numeric,nanoperiod-method}
\alias{+,nanoperiod,ANY-method}
\alias{+,ANY,nanoperiod-method}
\alias{+,nanoperiod,nanoperiod-method}
\alias{+,nanoperiod,nanoduration-method}
\alias{+,nanoperiod,integer64-method}
\alias{+,nanoperiod,nanotime-method}
\alias{+,nanoival,nanoperiod-method}
\alias{+,nanoperiod,nanoival-method}
\alias{+,nanotime,nanoperiod-method}
\alias{+,nanoperiod,numeric-method}
\alias{+,nanoduration,nanoperiod-method}
\alias{+,integer64,nanoperiod-method}
\alias{+,numeric,nanoperiod-method}
\alias{*,nanoperiod,integer64-method}
\alias{*,nanoperiod,ANY-method}
\alias{*,ANY,nanoperiod-method}
\alias{*,nanoperiod,numeric-method}
\alias{*,integer64,nanoperiod-method}
\alias{*,numeric,nanoperiod-method}
\alias{/,nanoperiod,integer64-method}
\alias{/,nanoperiod,numeric-method}
\alias{==,nanoperiod,nanoperiod-method}
\alias{!=,nanoperiod,nanoperiod-method}
\alias{plus,nanotime,nanoperiod,character-method}
\alias{plus}
\alias{plus,nanoperiod,nanotime,character-method}
\alias{minus,nanotime,nanoperiod,character-method}
\alias{minus}
\alias{minus,nanoperiod,nanotime,character-method}
\alias{plus,nanoival,nanoperiod,character-method}
\alias{plus,nanoperiod,nanoival,character-method}
\alias{minus,nanoival,nanoperiod,character-method}
\alias{NA_nanoperiod_}
\title{Period type with nanosecond precision}
\format{
An object of class \code{nanoperiod} of length 1.
}
\usage{
nanoperiod(months = 0, days = 0, duration = as.nanoduration(0))

\S4method{as.nanoperiod}{character}(x)

\S4method{as.nanoperiod}{integer64}(x)

\S4method{as.nanoperiod}{numeric}(x)

\S4method{as.nanoperiod}{integer}(x)

\S4method{as.nanoperiod}{nanoduration}(x)

\S4method{as.nanoperiod}{`NULL`}(x)

\S4method{as.nanoperiod}{missing}(x)

\S4method{show}{nanoperiod}(object)

\S4method{print}{nanoperiod}(x, quote = FALSE, ...)

\method{format}{nanoperiod}(x, ...)

\S4method{as.character}{nanoperiod}(x)

\S4method{is.na}{nanoperiod}(x)

\S4method{is.na}{nanoperiod}(x) <- value

\S4method{[[}{nanoperiod}(x, i, j, ..., drop = FALSE)

\S4method{[}{nanoperiod,numeric}(x, i, j, ..., drop = FALSE)

\S4method{[}{nanoperiod,logical}(x, i, j, ..., drop = FALSE)

\S4method{[}{nanoperiod,character}(x, i, j, ..., drop = FALSE)

\S4method{[}{nanoperiod,ANY}(x, i, j, ..., drop = FALSE)

\S4method{[}{nanoperiod,ANY,ANY,ANY}(x, i, j, ...) <- value

\method{c}{nanoperiod}(...)

\S4method{names}{nanoperiod}(x)

\S4method{names}{nanoperiod}(x) <- value

\S4method{-}{nanoperiod,ANY}(e1, e2)

\S4method{-}{nanoperiod,nanoperiod}(e1, e2)

\S4method{-}{nanoperiod,nanoduration}(e1, e2)

\S4method{-}{nanoperiod,integer64}(e1, e2)

\S4method{-}{nanoperiod,numeric}(e1, e2)

\S4method{-}{nanoduration,nanoperiod}(e1, e2)

\S4method{-}{integer64,nanoperiod}(e1, e2)

\S4method{-}{numeric,nanoperiod}(e1, e2)

\S4method{+}{nanoperiod,ANY}(e1, e2)

\S4method{+}{nanoperiod,nanoperiod}(e1, e2)

\S4method{+}{nanoperiod,nanoduration}(e1, e2)

\S4method{+}{nanoperiod,integer64}(e1, e2)

\S4method{+}{nanoperiod,nanotime}(e1, e2)

\S4method{+}{nanoival,nanoperiod}(e1, e2)

\S4method{+}{nanoperiod,nanoival}(e1, e2)

\S4method{+}{nanotime,nanoperiod}(e1, e2)

\S4method{+}{nanoperiod,numeric}(e1, e2)

\S4method{+}{nanoduration,nanoperiod}(e1, e2)

\S4method{+}{integer64,nanoperiod}(e1, e2)

\S4method{+}{numeric,nanoperiod}(e1, e2)

\S4method{*}{nanoperiod,integer64}(e1, e2)

\S4method{*}{nanoperiod,numeric}(e1, e2)

\S4method{*}{integer64,nanoperiod}(e1, e2)

\S4method{*}{numeric,nanoperiod}(e1, e2)

\S4method{/}{nanoperiod,integer64}(e1, e2)

\S4method{/}{nanoperiod,numeric}(e1, e2)

\S4method{==}{nanoperiod,nanoperiod}(e1, e2)

\S4method{!=}{nanoperiod,nanoperiod}(e1, e2)

\S4method{plus}{nanotime,nanoperiod,character}(e1, e2, tz)

\S4method{plus}{nanoperiod,nanotime,character}(e1, e2, tz)

\S4method{minus}{nanotime,nanoperiod,character}(e1, e2, tz)

\S4method{minus}{nanoperiod,nanotime,character}(e1, e2, tz)

\S4method{plus}{nanoival,nanoperiod,character}(e1, e2, tz)

\S4method{plus}{nanoperiod,nanoival,character}(e1, e2, tz)

\S4method{minus}{nanoival,nanoperiod,character}(e1, e2, tz)

NA_nanoperiod_
}
\arguments{
\item{months}{Used in the constructor to indicate the number of
months of the \code{nanoperiod}}

\item{days}{Used in the constructor to indicate the number of
days of the \code{nanoperiod}}

\item{duration}{Used in the constructor to indicate the duration
component of the \code{nanoperiod}}

\item{x, value}{An object of class \code{nanoperiod}}

\item{object}{argument for method \code{show}}

\item{quote}{indicates if the output of \code{print} should be
quoted}

\item{...}{further arguments}

\item{i}{index specifying elements to extract or replace.}

\item{j}{Required for \code{[} signature but ignored here}

\item{drop}{Required for \code{[} signature but ignored here}

\item{e1}{Operand of class \code{nanoperiod}}

\item{e2}{Operand of class \code{nanoperiod}}

\item{tz}{\code{character} indicating a timezone}
}
\description{
\code{nanoperiod} is a length of time type (implemented as an S4
class) with nanosecond precision. It differs from
\code{nanoduration} because it is capable of representing calendar
months and days. It can thus represent years (12 months) and weeks
(7 days). A period is a somewhat abstract representation of time:
it is only when anchored to a point in time and in a specific time
zone that it is possible to convert it to a specific
duration. This means that many of the operations involving periods
need the additional argument \code{tz}.
}
\section{Constructors}{


The true constructor is
}

\section{Output Format}{


A \code{nanoperiod} is displayed as months, days, and \code{nanoduration}
like this: \code{10m2d/10:12:34.123_453_000}.
}

\section{Details}{




Adding or subtracting \code{nanoperiod} and \code{nanotime}
require a timezone as third argument. For this reason it is not
possible to use the binary operator `\code{+}`. Instead the
functions `\code{plus}` and `\code{minus}` are defined. These
functions attempt to keep the same offset within a day in the
specified timezone: this means for instance that adding a day when
that day crosses a time zone adjustment such as a daylight saving
time, results in a true time increment of less or more than 24
hours to preserve the offset. Preserving the offset works for
increments that are smaller than a day too, provided the increment
results in a datetime where the timezone adjustment is valid. When
this is not the case, adding a `nanoperiod` behaves in the same
way as adding a `nanoduration`.
}

\examples{
\dontrun{
p <- nanoperiod(months=12, days=7, duration="01:00:00")
print(p)

#  when adding a \code{nanoperiod} to a \code{nanotime} or to a
# \code{nanoival}, a time zone must be specified:
y <- nanotime("1970-01-01T00:00:00+00:00")
plus(y, p, tz="America/Chicago")
}

}
\seealso{
\code{\link{nanotime}}, \code{\link{nanoduration}},
    \code{\link{nanoival}}, \code{\link{nanoperiod.month,nanoperiod-method}}
}
\author{
Dirk Eddelbuettel

Leonardo Silvestri
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nanoival.R
\name{rep,nanoival-method}
\alias{rep,nanoival-method}
\title{Replicate Elements}
\usage{
\S4method{rep}{nanoival}(x, ...)
}
\arguments{
\item{x}{a vector of \code{nanoival}}

\item{...}{further arguments:

       'times' an integer-valued vector giving the (non-negative)
           number of times to repeat each element if of length
           'length(x)', or to repeat the whole vector if of length
           1. Negative or 'NA' values are an error. A 'double'
           vector is accepted, other inputs being coerced to an
           integer or double vector.

       'length.out' non-negative integer. The desired length of the
           output vector. Other inputs will be coerced to a double
           vector and the first element taken. Ignored if 'NA' or
           invalid.

       'each' non-negative integer. Each element of 'x' is repeated
           'each' times.  Other inputs will be coerced to an integer
           or double vector and the first element taken. Treated as
           '1' if 'NA' or invalid.}
}
\description{
Replicates the values in 'x' similarly to the default method.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nanotime.R
\name{seq.nanotime}
\alias{seq.nanotime}
\alias{seq,nanotime-method}
\title{Sequence Generation}
\usage{
\method{seq}{nanotime}(from, to = NULL, by = NULL, length.out = NULL, along.with = NULL, ...)

\S4method{seq}{nanotime}(from, to = NULL, by = NULL, length.out = NULL, along.with = NULL, ...)
}
\arguments{
\item{from, to}{the starting and (maximal) end values of the
sequence}

\item{by}{\code{nanoduration} or \code{nanoperiod} increment of
the sequence; note that if the class is \code{nanoperiod} the
additional argument \code{tz} must be speficied and is of
\code{character} type indicating a timezone}

\item{length.out}{integer indicating the desired length of the sequence}

\item{along.with}{take the length from the length of this argument.}

\item{...}{arguments passed to or from methods; the only
interesting additional argument is \code{tz} where the
\code{to} argument is of type \code{nanoperiod}}
}
\description{
Generate a sequence of \code{nanotime}
}
\examples{
\dontrun{
from <- as.nanotime("2018-01-14T12:44:00+00:00")
to   <- as.nanotime("2019-01-14T12:44:00+00:00")
seq(from, to, by=as.nanoperiod("1m"), tz="America/New_York")
seq(from, by=as.nanoperiod("1y"), length.out=4, tz="Europe/London")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nanoival.R
\name{all.equal.nanoival}
\alias{all.equal.nanoival}
\alias{all.equal,nanoival-method}
\title{Test if Two Objects are (Nearly) Equal}
\usage{
\method{all.equal}{nanoival}(target, current, ..., check.attributes = TRUE)

\S4method{all.equal}{nanoival}(target, current, ..., check.attributes = TRUE)
}
\arguments{
\item{target, current}{\code{nanoival} arguments to be compared}

\item{...}{further arguments for different methods}

\item{check.attributes}{logical indicating if the
\code{attributes} of \code{target} and \code{current} (other than
the names) should be compared.}
}
\description{
Compare \code{target} and \code{current} testing \sQuote{near
equality}.  If they are different, comparison is still made to
some extent, and a report of the differences is returned.  Do not
use \code{all.equal} directly in \code{if} expressions---either
use \code{isTRUE(all.equal(....))} or \code{\link{identical}} if
appropriate.
}
\seealso{
\code{\link{identical}}, \code{\link{isTRUE}},
    \code{\link{==}}, and \code{\link{all}} for exact equality
    testing.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nanoperiod.R
\name{all.equal.nanoperiod}
\alias{all.equal.nanoperiod}
\alias{all.equal,nanoperiod-method}
\title{Test if Two Objects are (Nearly) Equal}
\usage{
\method{all.equal}{nanoperiod}(target, current, ..., check.attributes = TRUE)

\S4method{all.equal}{nanoperiod}(target, current, ..., check.attributes = TRUE)
}
\arguments{
\item{target, current}{\code{nanoperiod} arguments to be compared}

\item{...}{further arguments for different methods}

\item{check.attributes}{logical indicating if the
\code{attributes} of \code{target} and \code{current} (other than
the names) should be compared.}
}
\description{
Compare \code{target} and \code{current} testing \sQuote{near
equality}.  If they are different, comparison is still made to
some extent, and a report of the differences is returned.  Do not
use \code{all.equal} directly in \code{if} expressions---either
use \code{isTRUE(all.equal(....))} or \code{\link{identical}} if
appropriate.
}
\seealso{
\code{\link{identical}}, \code{\link{isTRUE}},
    \code{\link{==}}, and \code{\link{all}} for exact equality
    testing.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nanoival.R
\name{sort,nanoival-method}
\alias{sort,nanoival-method}
\title{Sorting or Ordering Vectors}
\usage{
\S4method{sort}{nanoival}(x, decreasing = FALSE)
}
\arguments{
\item{x}{a vector of \code{nanoival}}

\item{decreasing}{logical.  Should the sort be increasing or
decreasing?}
}
\description{
Sort (or _order_) a vector of \code{nanoival} into ascending or
descending order
}
\seealso{
\code{\link{is.unsorted}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nanotime.R
\name{all.equal.nanotime}
\alias{all.equal.nanotime}
\alias{all.equal,nanotime-method}
\title{Test if Two Objects are (Nearly) Equal}
\usage{
\method{all.equal}{nanotime}(
  target,
  current,
  tolerance = sqrt(.Machine$double.eps),
  scale = NULL,
  countEQ = FALSE,
  formatFUN = function(err, what) format(err),
  ...,
  check.attributes = TRUE
)

\S4method{all.equal}{nanotime}(
  target,
  current,
  tolerance = sqrt(.Machine$double.eps),
  scale = NULL,
  countEQ = FALSE,
  formatFUN = function(err, what) format(err),
  ...,
  check.attributes = TRUE
)
}
\arguments{
\item{target, current}{\code{nanotime} arguments to be compared}

\item{tolerance}{numeric >= 0. Differences smaller than
\code{tolerance} are not reported.  The default value is close
to \code{1.5e-8}.}

\item{scale}{\code{NULL} or numeric > 0, typically of length 1 or
\code{length(target)}.  See \sQuote{Details}.}

\item{countEQ}{logical indicating if the \code{target == current} cases should be
counted when computing the mean (absolute or relative)
differences.  The default, \code{FALSE} may seem misleading in
cases where \code{target} and \code{current} only differ in a few
places; see the extensive example.}

\item{formatFUN}{a \code{function} of two arguments, \code{err}, the relative, absolute
or scaled error, and \code{what}, a character string indicating
the _kind_ of error; maybe used, e.g., to format relative and
absolute errors differently.}

\item{...}{further arguments for different methods}

\item{check.attributes}{logical indicating if the \code{attributes} of \code{target}
and \code{current} (other than the names) should be compared.}
}
\description{
Compare \code{target} and \code{current} testing \sQuote{near
equality}.  If they are different, comparison is still made to
some extent, and a report of the differences is returned.  Do not
use \code{all.equal} directly in \code{if} expressions---either
use \code{isTRUE(all.equal(....))} or \code{\link{identical}} if
appropriate.
}
\seealso{
\code{\link{identical}}, \code{\link{isTRUE}},
    \code{\link{==}}, and \code{\link{all}} for exact equality
    testing.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nanoperiod.R
\name{nanoperiod.month,nanoperiod-method}
\alias{nanoperiod.month,nanoperiod-method}
\alias{nanoperiod.month}
\alias{nanoperiod.day}
\alias{nanoperiod.nanoduration}
\alias{nanoperiod.day,nanoperiod-method}
\alias{nanoperiod.nanoduration,nanoperiod-method}
\title{Nanoperiod accessors}
\usage{
\S4method{nanoperiod.month}{nanoperiod}(x)

\S4method{nanoperiod.day}{nanoperiod}(x)

\S4method{nanoperiod.nanoduration}{nanoperiod}(x)
}
\arguments{
\item{x}{A \code{nanoperiod}}
}
\value{
\code{nanoperiod.month} and \code{nanoperiod.day} return
    an \code{integer64} whereas \code{nanoperiod.nanoduration}
    returns a \code{nanoduration}
}
\description{
These functions allow access to the components of a \code{nanoperiod}
}
\examples{
p <- as.nanoperiod("2y1m1d/12:00:00")
nanoperiod.month(p)
nanoperiod.day(p)
nanoperiod.nanoduration(p)
}
\seealso{
\code{\link{nanoduration}}
}
\author{
Dirk Eddelbuettel

Leonardo Silvestri
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nanoival.R
\name{is.unsorted,nanoival-method}
\alias{is.unsorted,nanoival-method}
\title{Test if a \code{nanoival} vector is Not Sorted}
\usage{
\S4method{is.unsorted}{nanoival}(x, na.rm = FALSE, strictly = FALSE)
}
\arguments{
\item{x}{a \code{nanoival} vector}

\item{na.rm}{logical.  Should missing values be removed before
checking?}

\item{strictly}{logical indicating if the check should be for
_strictly_ increasing values.}
}
\description{
Test if an object is not sorted (in increasing order), without the
cost of sorting it.
}
\seealso{
\code{\link{sort}}
}
